/**
 * \file gre/layer.h
 * \brief GRE::LAYER class definition
 *
 * \if NODOC
 * $Id: layer.h_v 1.222 2012/11/16 16:14:03 mju Exp $
 *
 * $Log: layer.h_v $
 * Revision 1.222  2012/11/16 16:14:03  mju
 * Add ability to set delegate for datatip content.
 *
 * Revision 1.221  2012/06/13 14:51:44  mju
 * Remove unused methods.
 *
 * Revision 1.220  2012/05/02 17:56:59  mju
 * Don't inline serialRead.
 *
 * Revision 1.219  2011/11/21 16:41:43  mju
 * Add estimateRangeZ and virtual override.
 *
 * Revision 1.218  2011/11/09 17:33:56  mju
 * Add typedefn flag/method for hasPossibleOverlay.
 *
 * Revision 1.217  2011/10/20 21:47:14  mju
 * Use methods to setCanViewIn2/3D and add get/setCanViewStereo2D.
 *
 * Revision 1.216  2011/10/20 12:45:42  mju
 * Remove redundant hdrs.
 *
 * Revision 1.215  2011/10/13 16:19:44  mju
 * Add scalePointZ.
 *
 * Revision 1.214  2011/10/12 21:01:08  mju
 * Add getActiveElement return rvc::element reference.
 *
 * Revision 1.213  2011/10/12 16:35:38  mju
 * Add scalePointsZ.
 *
 * Revision 1.212  2011/10/10 20:32:34  mju
 * Make callDrawFilter public and add rvc::element variant.
 *
 * Revision 1.211  2011/10/10 17:47:43  mju
 * Move callDrawFilter here from vector/shape/pinmap.
 *
 * Revision 1.210  2011/10/10 14:38:45  mju
 * Add isManagingPointSymbols.
 *
 * Revision 1.209  2011/10/07 15:05:06  mju
 * Use trans2dmapgen hdr.
 *
 * Revision 1.208  2011/09/30 21:16:19  mju
 * Remove canDrawAsTiles.
 *
 * Revision 1.207  2011/08/15 22:25:57  mju
 * Add dispparm members for showing region when zoomed out.
 *
 * Revision 1.206  2011/08/05 22:14:00  mju
 * Make methods for density virtual.
 *
 * Revision 1.205  2011/08/05 17:58:44  mju
 * Make checkDenseDetail virtual.
 *
 * Revision 1.204  2011/06/17 17:25:32  mju
 * Move dispflags, layerflags, layerdrawflags to gre namespace.
 *
 * Revision 1.203  2011/04/14 20:37:47  bcolombe
 * Add listpos and reflayer to PromptAndConstruct
 *
 * Revision 1.202  2011/03/21 16:44:31  mju
 * Add constructLayerView and corresponding virtual.
 *
 * Revision 1.201  2011/02/24 15:38:02  mju
 * Add dispflag_ExcludeFromGroupExtents and remove layerflag_IgnoreExtents.
 *
 * Revision 1.200  2011/02/24 14:07:39  mju
 * Add is/setExcludedFromGroupExtents, remove unused setIgnoreExtents.
 *
 * Revision 1.199  2011/02/07 23:20:32  bcolombe
 * Add v_DrawOverlay
 * Add v_DataTipGetObjectNamesString
 *
 * Revision 1.198  2011/01/31 23:02:52  bcolombe
 * Add v_CanRender
 *
 * Revision 1.197  2011/01/24 21:28:02  bcolombe
 * Add bingmaps layer type
 *
 * Revision 1.196  2010/11/22 16:59:53  bcolombe
 * Add typedefn ctor that takes objtypeset and file extn
 * Used for tileset layer
 *
 * Revision 1.195  2010/11/19 22:51:54  bcolombe
 * Remove use of DLG_GETOBJECT::ITEM
 *
 * Revision 1.194  2010/10/13 18:11:56  mju
 * Add snapPoint/IsAllowed and corresponding virtual methods.
 *
 * Revision 1.193  2010/09/03 20:55:34  mju
 * In testElementByRegion default to region being in object coords.
 *
 * Revision 1.192  2010/08/19 23:09:35  bcolombe
 * For use with adding layers via DLG_GETOBJECT::ITEM, added:
 * AsignItem, v_AssignItem
 * IsItemSelectable, v_IsItemSelectable
 * ConstructFromItem, CanConstructFromItem, ConstructFromItemList
 * PromptObjects that takes itemlist
 *
 * Revision 1.191  2010/08/19 17:40:06  mju
 * Use elemnum type.
 *
 * Revision 1.190  2010/05/07 18:12:23  ldyas
 * added kml layer type
 *
 * Revision 1.189  2010/04/23 16:54:07  mju
 * Add lidar layer.
 * Use unsignedset.
 *
 * Revision 1.188  2010/03/30 22:49:19  bcolombe
 * Added CREATEFLAG_NoChangeGroupRenderCRS
 *
 * Revision 1.187  2010/03/29 19:51:31  mju
 * Add typedefn::flag_None.
 *
 * Revision 1.186  2010/03/15 16:38:34  mju
 * Add default for style in drawElement/sLow.
 *
 * Revision 1.185  2010/01/28 17:31:10  mju
 * Add dlgHighlightByQuery to layer and compinfo.
 *
 * Revision 1.184  2010/01/13 17:15:49  mju
 * Change dlgMarkByQuery to be dlgshell ptr.
 *
 * Revision 1.183  2009/11/06 17:28:38  mju
 * Change typedefn to work with mgui::dlgGetObject... and filter.
 *
 * Revision 1.181  2009/09/23 21:54:48  mju
 * Add isSortLegendNatural.
 *
 * Revision 1.180  2009/08/21 16:19:46  mju
 * Add compinfo::dlg_reclassify and supporting methods.
 *
 * Revision 1.179  2009/08/11 20:46:38  mju
 * Add tableinfo copy ctor.
 *
 * Revision 1.178  2009/08/10 16:42:27  mju
 * Replace canZoomOneToOne to canZoomToMaxDetail.
 *
 * Revision 1.177  2009/08/10 13:47:01  mju
 * Add getScaleMaxDetail.
 *
 * Revision 1.176  2009/08/07 19:58:37  mju
 * Add canZoomOneToOne.
 *
 * Revision 1.175  2009/08/07 16:13:25  bcolombe
 * Change to use TILESET layer
 *
 * Revision 1.174  2009/07/27 14:22:08  bcolombe
 * Added google maps
 *
 * Revision 1.173  2009/04/15 19:37:00  bcolombe
 * Removed usage of geocat dialog
 *
 * Revision 1.172  2009/03/31 13:50:48  mju
 * Add legendredrawdisabler.
 *
 * Revision 1.171  2008/10/10 17:53:18  mju
 * Add includeGroup parm to getMin/MaxScaleVisible and getVisibleScaleRagne.
 *
 * Revision 1.170  2008/08/08 21:28:05  mju
 * Add v_getNumElementsDeleted.
 *
 * Revision 1.168  2008/06/19 22:06:30  mju
 * Add hasDrawFilter.
 *
 * Revision 1.166  2008/05/21 22:03:16  mju
 * Add groupviews to invalidateAllViews.
 *
 * Revision 1.163  2008/05/08 16:25:01  mju
 * Add isRemote.
 *
 * Revision 1.162  2008/05/07 20:11:34  mju
 * Add typedefn.isRemote and corresponding flag.
 * Remove typedefn::flag_NonRotatable as not used.
 *
 * Revision 1.161  2008/03/17 21:32:25  mju
 * Change v_getLayerView to v_getLayerSection
 *
 * Revision 1.160  2008/03/03 18:36:41  scowan
 * Added pointer button enum include file.
 *
 * Revision 1.159  2008/03/03 17:12:07  mju
 * Add setListPos with enum and ref layer.
 * Add treeview stuff for drag-and-drop reordering.
 *
 * Revision 1.158  2008/02/29 17:08:48  mju
 * Add skiplocator parm to treeViewGomputeStateIdx.
 *
 * Revision 1.157  2008/02/08 22:07:31  mju
 * Make compinfo::onMenuRegionGen public.
 *
 * Revision 1.156  2008/01/02 22:44:49  mju
 * Add menuAddViewPopupItems and virtual overridable.
 *
 * Revision 1.155  2007/12/27 23:59:26  scowan
 * Added parms to constructFromObjList for list positioning.
 *
 * Revision 1.154  2007/12/03 16:52:04  mju
 * Add v_menuAddElemTypeItems.
 *
 * Revision 1.153  2007/10/10 17:14:50  mju
 * Add dispflag for raster lines/labels.
 *
 * Revision 1.152  2007/10/04 21:22:13  mju
 * Exclude macros from docs.
 *
 * Revision 1.151  2007/09/17 14:35:01  mju
 * Change commonparm::form::v_create to take initial values.
 *
 * Revision 1.150  2007/09/14 21:46:53  mju
 * Add commonparms stuff for user interface.
 *
 * Revision 1.149  2007/09/10 17:49:13  mju
 * Add group to commonparms.
 *
 * Revision 1.148  2007/09/10 14:20:44  mju
 * Remove prev method, will add for layers as needed with layer-specific return type.
 *
 * Revision 1.147  2007/09/10 14:17:51  mju
 * Add getCommonParms.
 *
 * Revision 1.146  2007/09/06 21:56:40  mju
 * Add support for common parms.
 *
 * Revision 1.145  2007/08/31 20:36:12  mju
 * Add type_generic.
 *
 * Revision 1.144  2007/06/22 21:07:10  mju
 * Add compinfo.setElemType.
 *
 * Revision 1.143  2007/05/08 15:52:37  dwilliss
 * SML namespace
 *
 * Revision 1.142  2007/03/30 17:49:23  dwilliss
 * Added virtual methods for datatips
 *
 * Revision 1.141  2007/03/27 14:39:46  mju
 * Change dispflag for raster to not conflict with historical one.
 *
 * Revision 1.140  2007/03/26 17:50:32  dwilliss
 * added some virtual method
 *
 * Revision 1.139  2007/03/12 17:42:36  mju
 * Add v_setObjectDescriptor.
 *
 * Revision 1.138  2007/03/07 13:47:10  mju
 * Add dataTipSetDefault variant to do for all components.
 *
 * Revision 1.137  2007/02/28 21:50:46  mju
 * Add dispflag_RastBlack/WhiteTransp.
 *
 * Revision 1.136  2007/01/18 21:58:17  mju
 * Add type_image_wms.
 *
 * Revision 1.135  2006/12/11 16:28:56  mju
 * Remove unused deprecated typedefs.
 *
 * Revision 1.134  2006/12/06 21:03:25  dwilliss
 * Added a perameter to DataTipGetString
 *
 * Revision 1.133  2006/11/16 15:50:36  mju
 * Add type_image_spatmod.
 *
 * Revision 1.132  2006/10/13 14:03:41  mju
 * Use getobjflags hdr.
 *
 * Revision 1.131  2006/10/10 14:20:10  mju
 * Use iconid hdr.
 *
 * Revision 1.130  2006/10/10 13:56:18  mju
 * Use miconstruct* instead of micon.
 *
 * Revision 1.129  2006/10/09 17:50:21  mju
 * Move active element member to layer.
 * Make methods referencing dispparm through pointer non-inline.
 *
 * Revision 1.128  2006/10/05 20:02:23  mju
 * Use color hdr.
 *
 * Revision 1.127  2006/10/04 13:59:41  mju
 * Add compinfo::markAttachedElements using dbedit_msg.
 *
 * Revision 1.126  2006/10/03 21:34:20  dwilliss
 * changed database editor callback struct
 *
 * Revision 1.125  2006/10/02 17:47:25  mju
 * More legacy rvc stuff removed.
 * DataTip dialog separate.
 *
 * Revision 1.124  2006/09/28 19:15:37  mju
 * Fwd declare rvc::object.
 *
 * Revision 1.123  2006/09/21 21:58:47  mju
 * Use mg2enums hdr.
 *
 * Revision 1.122  2006/09/20 17:16:14  mju
 * Fwd declare menuitem, menuitemarray.
 *
 * Revision 1.121  2006/09/19 17:52:57  mju
 * Remove old wireframe member.
 *
 * Revision 1.120  2006/09/19 17:28:44  mju
 * Add globaloptions.forceGeorefAffine.
 *
 * Revision 1.119  2006/09/18 20:41:35  mju
 * Use otypeset hdr.
 *
 * Revision 1.118  2006/09/18 17:42:33  mju
 * Remove 3dwireframe hdr.
 *
 * Revision 1.117  2006/09/15 17:24:10  mju
 * Remove wireframeCreate/Destroy.
 *
 * Revision 1.116  2006/09/15 15:29:57  mju
 * Serializer changes.
 * Add v_onNotifySurfaceChanged.
 *
 * Revision 1.115  2006/09/08 19:40:11  mju
 * Use rvc::dispparmusage.
 *
 * Revision 1.114  2006/09/08 15:25:57  mju
 * Remove loadDispParmSubObject as no longer needed.
 *
 * Revision 1.113  2006/09/07 21:25:46  mju
 * Change openObject to not have opemode parm and have virtual use v_ prefix.
 *
 * Revision 1.112  2006/08/01 17:51:26  mju
 * Add layerflag_noInvalidateViews.
 *
 * Revision 1.111  2006/04/19 20:36:28  mju
 * Add treeViewUpdateDetails.
 *
 * Revision 1.110  2006/02/22 17:16:54  mju
 * Add typedefn support for 'edit' icon.
 *
 * Revision 1.109  2006/02/16 21:02:03  mju
 * Add treeview tooltip delegates.
 *
 * Revision 1.108  2006/01/05 17:50:04  dwilliss
 * changed parm type on a method
 *
 * Revision 1.107  2006/01/04 16:53:15  mju
 * Deprecate loadDispParmSubObject.
 *
 * Revision 1.106  2005/12/22 14:19:31  mju
 * Add comment.
 *
 * Revision 1.105  2005/12/22 14:16:49  mju
 * Exclude type_next if no_deprecated.
 *
 * Revision 1.104  2005/12/19 17:12:14  mju
 * Allow getMenuItemCreate unless deprecate_all.
 *
 * Revision 1.103  2005/12/08 22:40:26  mju
 * Remove mem24 virtual.
 *
 * Revision 1.102  2005/11/28 16:37:58  mju
 * Support legend update in treeview.
 *
 * Revision 1.101  2005/11/21 16:05:47  scowan
 * Allow using layer type as counter.
 *
 * Revision 1.100  2005/11/16 22:46:42  mju
 * Add promptGeoDatabasesAndConstruct.
 *
 * Revision 1.99  2005/11/11 21:39:46  mju
 * Add treeViewOnCustomDraw for layer.
 *
 * Revision 1.98  2005/11/10 20:38:46  scowan
 * Do not define v_setupTreeViewShared if not mGUI.
 *
 * Revision 1.97  2005/11/10 18:46:11  mju
 * Add treeview delegate for getItemInfo.
 *
 * Revision 1.96  2005/11/07 21:10:37  mju
 * Deprecate getMenuItemCreate.
 *
 * Revision 1.95  2005/11/07 17:21:51  mju
 * Add compinfo method to handle treeview child visibility changing.
 *
 * Revision 1.94  2005/11/04 18:52:03  mju
 * enums in gre namespace.
 * Add showSystemTables to compinfo.
 *
 * Revision 1.93  2005/11/03 19:44:00  mju
 * Remove stuff to get setting for showing internal tables.
 *
 * Revision 1.92  2005/11/03 18:27:02  mju
 * Make v_MenuAddToolItems protected instead of private.
 *
 * Revision 1.91  2005/11/02 23:00:44  mju
 * Remove objmgr stuff.
 *
 * Revision 1.90  2005/10/27 14:04:53  mju
 * Fix doc comments.
 *
 * Revision 1.89  2005/10/26 16:55:13  mju
 * Remove stuff for tabular view toggle in tableinfo.
 *
 * Revision 1.88  2005/10/19 20:03:37  mju
 * Use gre namespace instead of legacy typedefs.
 *
 * Revision 1.87  2005/10/14 17:30:20  mju
 * GRE namespace and treeview.
 *
 * Revision 1.86  2005/09/02 17:05:09  mju
 * Add global option for dftMultiPassRendering.
 *
 * Revision 1.85  2005/08/29 20:44:20  scowan
 * Fixed g++ duplication warning.
 *
 * Revision 1.84  2005/08/16 20:29:05  dwilliss
 * Added a private static method
 *
 * Revision 1.83  2005/06/17 16:48:10  mju
 * Forward-declare mgui:;dlgshell.
 *
 * Revision 1.82  2005/05/24 16:12:13  mju
 * Add layerflag_DataTipIgnoreDrawnSet.
 *
 * Revision 1.81  2005/05/11 20:58:18  mju
 * Add dispflag_noMergeLegendDups.
 *
 * Revision 1.80  2005/05/02 17:43:45  mju
 * Remove highlightmode for layer.
 *
 * Revision 1.79  2005/04/29 17:49:30  mju
 * Support rendertarget.
 *
 * Revision 1.78  2005/04/14 19:39:50  mju
 * Add dispflag_enableMultiPassRender.
 *
 * Revision 1.77  2005/03/31 16:57:06  fileserver!dwilliss
 * Rename one of our types to MIUNICODE because it conflicted with a Microsoft #define
 *
 * Revision 1.76  2005/03/22 22:00:50  mju
 * Add invalidateAllViews.
 *
 * Revision 1.75  2005/03/07 17:23:15  scowan
 * Fixed some unix warnings.
 *
 * Revision 1.74  2005/03/01 16:50:27  scowan
 * Made v_copy pure virtual and created an interface to call it.
 * Support v_copy method.
 *
 * Revision 1.73  2005/03/01 14:20:38  mju
 * Add getElemHighlightSet, remove elemsearch stuff.
 *
 * Revision 1.72  2005/02/25 18:57:47  mju
 * Add v_hasHighighlighted.
 *
 * Revision 1.71  2005/02/25 18:54:20  mju
 * Remove layerview defns to own header.
 * Add v_Copy virtual method.
 *
 * Revision 1.70  2005/02/25 17:47:16  mju
 * More layerview methods.
 *
 * Revision 1.69  2005/02/24 18:52:23  mju
 * Remove layerview getBackgroundColor.
 *
 * Revision 1.68  2005/02/23 18:52:32  mju
 * Add layerview stuff for highlight device.
 * Move labelplacement methods to layerview.
 * Add layerview_labeled for common labelplacement handling.
 *
 * Revision 1.67  2005/02/18 19:06:32  mju
 * Changes to layerview for separated rendering.
 *
 * Revision 1.66  2005/02/17 21:18:55  mju
 * Add layerview stuff for separate rendering.
 *
 * Revision 1.65  2005/02/15 17:50:24  scowan
 * No real changes.
 *
 * Revision 1.64  2005/02/15 17:34:28  scowan
 * Do not include grtree unless in x only.
 *
 * Revision 1.63  2005/02/15 17:22:20  mju
 * Add full/used rectangles to layerview.
 *
 * Revision 1.62  2005/02/14 21:39:36  mju
 * Use device in usesTransparency.
 *
 * Revision 1.61  2005/02/14 18:52:47  mju
 * Move some stuff from layerview to groupview.
 *
 * Revision 1.60  2005/01/27 21:42:32  mju
 * Add layerview stuff for labels.
 *
 * Revision 1.59  2005/01/24 21:31:08  mju
 * Move members to layerview from layerdc.
 *
 * Revision 1.58  2005/01/20 16:54:34  mju
 * Enums for createflags, drawmode, highlightmode.
 * Add static methods to replace legacy mdisp fns.
 * Redesign typedefn class to use textid, iconid, virtual methods, etc.
 *
 * Revision 1.57  2005/01/12 17:12:42  mju
 * Include transformations in layerview.
 *
 * Revision 1.56  2005/01/04 15:22:53  mju
 * Deprecate assign method using fnameinode.
 *
 * Revision 1.55  2005/01/03 21:46:48  mju
 * Add trans and mgd dev to layerview.
 *
 * Revision 1.54  2004/10/15 21:25:33  ldyas
 * added method for needsmem24
 *
 * Revision 1.49  2004/10/06 13:53:48  mju
 * Not necessary to forward-declare inlines for get/set antialias/thinhint methods.
 *
 * Revision 1.47  2004/10/01 17:19:32  mju
 * Add methods for detail density checking.
 * Add dlgparent to assignObject.
 *
 * Revision 1.46  2004/09/29 18:25:42  ldyas
 * now when set antialias also set use transparency.
 *
 * Revision 1.45  2004/09/28 22:17:59  ldyas
 * added antialiasing to globals.
 *
 * Revision 1.43  2004/09/24 00:48:53  ldyas
 * added thin line width hinting
 *
 * Revision 1.41  2004/09/23 22:55:02  ldyas
 * added antialiasing to dispparm.
 *
 * Revision 1.40  2004/09/20 19:11:44  dglasser
 * added setOldPrj for sml grelayer.
 *
 * Revision 1.39  2004/09/03 22:49:50  dwilliss
 * Added some members to dispparm
 * DrawLabel no longer needs to be virtual
 *
 * Revision 1.38  2004/08/19 22:19:50  vdronov
 * changed transp from 100 to 255
 *
 * Revision 1.37  2004/08/18 16:25:33  mju
 * Make updateExtents non-inline.
 *
 * Revision 1.35  2004/07/21 21:55:54  mju
 * Add layerview support.
 *
 * Revision 1.31  2004/07/19 22:56:10  dwilliss
 * LABELPLACEMENT changes
 *
 * Revision 1.30  2004/07/14 20:02:22  dwilliss
 * Forgot to add the inline wrapper to call a private virtaul method
 *
 * Revision 1.29  2004/07/13 22:33:03  dwilliss
 * Added a virtual method for labels
 *
 * Revision 1.28  2004/07/12 23:14:34  mju
 * Nod georegion.
 *
 * Revision 1.27  2004/07/07 22:00:34  mju
 * Add updateMapRegion.
 *
 * Revision 1.26  2004/07/07 20:51:29  vdronov
 * added SetPedestal and IsPedestalSet
 *
 * Revision 1.25  2004/06/29 21:35:24  mju
 * Add highlightElementsByRegion using region2d.
 *
 * Revision 1.24  2004/06/28 17:53:20  mju
 * Convert to spatref system.
 *
 * Revision 1.22  2004/06/11 17:48:34  vdronov
 * removed flags related to RayCasting
 *
 * Revision 1.21  2004/06/04 19:20:44  dglasser
 * Don't use memset with TEXTSTYLEs in SML.  Added SMLSET_TextStyleMember.
 *
 * Revision 1.20  2004/05/05 22:49:56  dwilliss
 * Started adding support for LABELPLACEMENT
 *
 * Revision 1.19  2004/04/02 18:54:43  mju
 * Add ModifyStamp methods.
 *
 * Revision 1.18  2004/03/31 21:32:04  mju
 * Add DrawElement using rvc::ELEMENT.
 *
 * Revision 1.17  2004/02/12 22:43:22  dwilliss
 * deprecate old mgd
 *
 * Revision 1.16  2004/02/06 16:23:28  mju
 * Reorder fns.
 *
 * Revision 1.15  2004/02/06 16:01:34  mju
 * Use GRE::FINDFLAGS.
 *
 * Revision 1.14  2004/02/05 22:08:25  mju
 * Integrate datatip support.
 *
 * Revision 1.12  2003/12/15 16:52:51  mju
 * Add dispflag_VectNoFillHighlighted.
 *
 * Revision 1.11  2003/11/20 13:26:33  mju
 * Add SetVisiblityGroupFromLayout.
 *
 * Revision 1.7  2003/09/23 13:08:08  mju
 * Explicitly init serializble in copy dispparm copy ctor.
 *
 * Revision 1.4  2003/07/31 13:38:40  mju
 * Add Filter method to LAYER::FILTER.
 *
 * Revision 1.3  2003/07/30 15:15:58  mju
 * Ignore private sections.
 * Move DISPPARM defn outside layer so will be doc'd.  Add inlines where req'd.
 *
 * Revision 1.1  2003/06/16 19:39:43  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_LAYER_H
#define	INC_GRE_LAYER_H

#ifndef	INC_GRE_OBJECTBASE_H
	#include <gre/objectbase.h>
#endif

#if !defined(INC_MGUI_TREE_H) && defined(USE_MGUI)
	#include <mgui/tree.h>
#endif

#ifndef	INC_MI32_GRTREE_H
	#include <mi32/grtree.h>
#endif

#ifndef	INC_RVC_ELEMENT_H
	#include <rvc/element.h>
#endif

#ifndef	INC_RVC_DISPPARMUSAGE_H
	#include <rvc/dispparmusage.h>
#endif

#ifndef	INC_MGUI_DLGGETOBJECTSTYLE_H
	#include <mgui/dlggetobjectstyle.h>
#endif

#ifndef	INC_RVC_OBJECTNAME_H
	#include <rvc/objectname.h>
#endif

#ifndef	INC_MI32_REGION2D_H
	#include <mi32/region2d.h>
#endif

#ifndef	INC_MI32_SERIALIZABLE_H
	#include <mi32/serializable.h>
#endif

#ifndef	INC_MI32_SERIALIZERITEM_H
	#include <mi32/serializeritem.h>
#endif

#ifndef	INC_MI32_ELEMSTYL_H
	#include <mi32/elemstyl.h>
#endif

#ifndef	INC_RVC_OTYPESET_H
	#include <rvc/otypeset.h>
#endif

#ifndef	INC_MI32_SMLBASE_H
	#include <mi32/smlbase.h>
#endif

#ifndef	INC_MI32_TRANS2DMAPGEN_H
	#include <mi32/trans2dmapgen.h>
#endif

#ifndef	INC_MI32_MG2ENUMS_H
	#include <mi32/mg2enums.h>
#endif

#ifndef	INC_MI32_UNSIGNEDSET_H
	#include <mi32/unsignedset.h>
#endif

#ifndef	INC_MI32_DELEGATEFILTER_H
	#include <mi32/delegatefilter.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations
class DBEDIT;
struct DATATIPPARMS;
struct DBEDIT_MSG;
struct MENUITEM;
class MENUITEMARRAY;
namespace RVC {
	class DESCRIPTOR;
	class DBTABLE;
	class DBASE_ELEMENT;
	class OBJECT;
	class OBJITEM;
	class OBJITEMLIST;
	}
namespace MGD {
	class DEVICE;
	}
namespace SPATMOD {
	namespace IMAGE {
		class STAGE_ARRAY;
		}
	}
namespace MGUI {
	class DLGSHELL;
	class MENU;
	class DLG_GETOBJECT_FILTER;
	}
#endif	// GENERATING_DOXYGEN_OUTPUT


namespace GRE {

//===================================================================================================================

//! Flags stored in xxxxDISPPARM.dispflags
enum DISPFLAGS {
	DISPFLAG_None =							0x00000000,
//	DISPFLAG_SaveDrawnSetOBS =				0x00000001,	// OBSOLETE, DO NOT USE, used in old dispparms
//	DISPFLAG_KeepOpenOBS =					0x00000002,	// OBSOLETE, DO NOT USE, used in old dispparms
	DISPFLAG_ExcludeFromGroupExtents =	0x00000004,	//!< Exclude layer from group extents
//	DISPFLAG_EditObjectOBS =				0x00000008,	// OBSOLETE, DO NOT USE, used in old dispparms

	DISPFLAG_WarpToGeorefModel =			0x00000100,	//!< Warp to model specified with georeference
	DISPFLAG_UsesTransparency =			0x00000200,	//!< Uses transparency effects
	DISPFLAG_NoDrawWireframe =				0x00000400,	//!< Don't draw layer in wireframe mode
	DISPFLAG_MayUseTransparency =			0x00000800,	//!< Use of transparency unknown, assume it does until first time draw
	DISPFLAG_Antialias =						0x00001000,	//!< Draw Antialiased
	DISPFLAG_HintThinLines =				0x00002000,	//!< Imply thin lines via transparency
	DISPFLAG_EnableMultiPassRender =		0x00004000,	//!< Enable multipass rendering for styles
	DISPFLAG_NoMergeLegendDups =			0x00008000,	//!< Don't combine duplicate styles in legend
	// Layer-specific flags use			0x0XXX0000

	DISPFLAG_NoDrawLabels =					0x40000000,	//!< Don't draw labels
	DISPFLAG_DrawLabelsOnly =				0x80000000,	//!< Only draw labels

	// Layer-specific flags can use range of 0x0FFF0000.

	// Vector layer flags
	DISPFLAG_VectDrawNodes =				0x00010000,	//!< Draw nodes for vector object
	DISPFLAG_VectLinesFirst =				0x00020000,	//!< Draw vector lines before polygons
	DISPFLAG_VectNoFillPoly =				0x00040000,	//!< Disable polygon filling
	DISPFLAG_VectNoFillHighlighted =		0x00080000,	//!< Disable polygon filling when highlighted

	// TIN layer flags
	DISPFLAG_TINEdgesFirst =				0x00020000,	//!< Draw TIN edges before triangles
	DISPFLAG_TINNoFillTri =					0x00040000,	//!< Don't fill TIN triangles

	// Raster layer flags
	DISPFLAG_RastNullTransp =				0x00010000,	//!< Make "null" cells transparent
	DISPFLAG_RastSkipOddLines =			0x00020000,	//!< Skip odd screen lines (for special rendering)
	DISPFLAG_RastSkipEvenLines =			0x00040000,	//!< Skip even screen lines (for special rendering)
	DISPFLAG_RastDoBlendMask =				0x00080000,	//!< Do mask transparency blending if possible
	DISPFLAG_RastBlackTransp =				0x00100000,	//!< Make "black" cells transparent
//													0x00200000,	//   Used in legacy layouts, cannot reuse
	DISPFLAG_RastHyperspectLossy =		0x00400000,	//!< Display hyperspectral in fast-lossy mode
	DISPFLAG_RastWhiteTransp =				0x00800000,	//!< Make "white" cells transparent
	DISPFLAG_RastLinesBetweenCells =		0x01000000,	//!< Show lines between cells when zoom in
	DISPFLAG_RastLabelsInCells =			0x02000000,	//!< Show labels in cells when zoom in

	// Surface layer flags
	DISPFLAG_SurfProjOffEdge =				0x00010000,	//!< Project data off edge of surface

	// Formula layer flags
	DISPFLAG_FormulaNullTransp =			0x00010000,	//!< Make "null" cells transparent
	DISPFLAG_FormulaSkipOddLines =		0x00020000,	//!< Skip odd screen lines (for 3Dmax glasses)
	DISPFLAG_FormulaSkipEvenLines =		0x00040000,	//!< Skip even screen lines (for 3Dmax glasses)
	};
#ifndef	GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(DISPFLAGS)
#endif

//===================================================================================================================

enum LAYERFLAGS {
	LAYERFLAG_None =							0x00000000,	//!< No flags
	LAYERFLAG_NoSelNewObj =					0x00000010,	//!< Don't allow user to select new object(s) via CtrlPanel
	LAYERFLAG_HiliteColorSet =				0x00000040,	//!< Separate highlight color has been set for layer
	LAYERFLAG_DataTipIgnoreDrawnSet =	0x00000080,	//!< Ignore DrawnSet when searching for DataTip element
	LAYERFLAG_NoInvalidateViews =			0x00000100,	//!< Don't allow invalidation of render buffers
	LAYERFLAG_3D =								0x00000200,	//!< Layer is fully 3-D, no "surface" required
	LAYERFLAG_KeepOpen =						0x00020000,	//!< Set if object should remain open until layer is destroyed
	LAYERFLAG_Editable =						0x00040000,	//!< Set if layer can be edited
	LAYERFLAG_CanShowInLocator =			0x00100000,	//!< Set if can show this layer in the Locator window
	LAYERFLAG_NoGeoreference =				0x00080000,	//!< Set if no georeference info should be read for layer
	LAYERFLAG_NoDrawActiveElem =			0x01000000,	//!< Don't draw "active" element
	LAYERFLAG_NoShowInLayerMgr =			0x02000000,	//!< Don't show in layer manager, must also set LAYERFLAG_Temporary
	LAYERFLAG_NoAllowHide =					0x04000000,	//!< Don't allow user to hide layer
	LAYERFLAG_NoAllowDelete =				0x08000000,	//!< Don't allow user to delete layer
	LAYERFLAG_Temporary =					0x10000000,	//!< Layer is temporary, cannot save in layout
	LAYERFLAG_MaskAll =						0xFFFFFFFF,	//!< Mask for setting all flags
	};
#ifndef	GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(LAYERFLAGS)
#endif

//===================================================================================================================

enum LAYERDRAWFLAGS {
	LAYERDRAWFLAG_None =					0x0000,	//!< No flags
//	LAYERDRAWFLAG_ClipTo =				0x0001,
	LAYERDRAWFLAG_Clip =					0x0002,	//!< Clip layer
	LAYERDRAWFLAG_NoClipToGroup =		0x0004,	//!< Don't clip layer to group
	LAYERDRAWFLAG_SaveToTemplate =	0x0008,	//!< Save layer as-is to template
	LAYERDRAWFLAG_MaskAll =				0xFFFF,	//!< Mask for setting all flags
	};
#ifndef	GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(LAYERDRAWFLAGS)
#endif

//===================================================================================================================

#define	GRE_MAXLAYERTYPES		32

//! Layer base class.
class LAYER : public GRE::OBJECT {
	public:

		//! Layer types.
		enum TYPE {
			TYPE_None =					0,
			TYPE_Raster =				1,
			TYPE_Vector =				2,
			TYPE_CAD =					3,
			TYPE_DBPinmap =			4,
			TYPE_MapGrid =				5,
			TYPE_ScaleBar =			6,
			TYPE_Text =					7,
			TYPE_Legend =				8,
			TYPE_TIN =					9,
			TYPE_Sketch =				10,
			TYPE_Region =				11,
			TYPE_Surface =				12,
			TYPE_Script =				13,
			TYPE_Formula =				14,
			TYPE_Legend2 =				15,
			TYPE_Shape =				16,
			TYPE_Image_SpatMod =		17,
			TYPE_Image_WMS =			18,
			TYPE_Generic =				19,
			TYPE_Image_ArcIMS =		20,
			TYPE_Image_Tileset =		21,
			TYPE_Lidar =				22,
			TYPE_KML =					23,
			TYPE_Image_BingMaps =	24,
			TYPE_COUNT
			// Check GRE_MAXLAYERTYPES when add new types
			};

		enum CREATEFLAGS {
			CREATEFLAG_None =					0x0000,
			CREATEFLAG_NoCallCreateCB =	0x0001,
			CREATEFLAG_Temporary =			0x0002,
			CREATEFLAG_NoSelNewObj =		0x0004,	//!< Don't allow user to select new object(s) via CtrlPanel
			CREATEFLAG_KeepOpen =			0x0008,	//!< Set if object should remain open until layer is destroyed
			CREATEFLAG_Editable =			0x0010,	//!< Set if creating layer to be edited
			CREATEFLAG_NoGeoreference =	0x0020,	//!< Set if no georeference info should be read for layer
			CREATEFLAG_IgnoreExtents =		0x0040,	//!< Ignore layer extents in group
			CREATEFLAG_NoAllowDelete =		0x0080,	//!< Don't allow user to delete layer
			CREATEFLAG_NoControlPanel =	0x0100,	//!< Don't allow control panel for layer
			CREATEFLAG_NoComputePos =		0x0200,	//!< Don't call ComputeLayerPositions() when adding new layer
			CREATEFLAG_NoSetActive =		0x0400,	//!< Don't make newly-created layer "active"
			CREATEFLAG_NoShowInMgr =		0x0800,	//!< Don't show this layer in layer manager
			CREATEFLAG_NoChangeGroupRenderCRS = 0x1000, //!< Don't check if need to switch to Geographic CRS for widely disparate projections
			};

		enum DIMENSION {
			DIMENSION_Height	= 0,
			DIMENSION_Width	= 1
			};

		enum SETACTIVEFLAGS {
			SETACTIVEFLAG_None =				0x0000,
			SETACTIVEFLAG_First =			0x0001,
			SETACTIVEFLAG_Previous =		0x0002,
			SETACTIVEFLAG_Next =				0x0004,
			SETACTIVEFLAG_Last =				0x0008,
			SETACTIVEFLAG_KeepCurrent =	0x0010,	//!< Keep current active element if highlighted
			SETACTIVEFLAG_NoWrap =			0x0020,	//!< Don't wrap around end/beginning if reached
			SETACTIVEFLAG_UpdateScreen =	0x0040,	//!< Update display window
			SETACTIVEFLAG_UpdateBuffers =	0x0080,	//!< Update rendering buffer
			};

		//! What to snap to.
		enum SNAPTO {
			SNAPTO_EndPoint =		1,		//!< Nearest endpoint/node
			SNAPTO_Vertex =		2,		//!< Nearest vertex/corner
			SNAPTO_Element =		3,		//!< Nearest point on nearest element
			};

		//! Delegate to get DataTip content.
		//! Parameters are same as DataTipGetString.
		//! Return -1 for no DataTip, TRUE if have content, FALSE to use v_DataTipGetString, < -1 if error.
		typedef FastDelegate<int(const DPOINT2D& point, MISTRING& string, SPATMOD::IMAGE::STAGE_ARRAY& images, GRE::VIEW* view)> DELEGATE_DATATIPGETCONTENT;

		//! Container for global layer options.
		class GLOBALOPTIONS {		// LAYER::GLOBALOPTIONS
			public:

				class FORM;									//!< Global options form, defined in <gre/uilrglob.h>

				enum DFTNAMEMODE {
					DFTNAMEMODE_ObjectDesc =		0,		//!< Use object description (or name if no description)
					DFTNAMEMODE_ObjectName =		1,		//!< Use object name
					DFTNAMEMODE_FileObjNames =		2		//!< Use combined file and object names
					};

				GLOBALOPTIONS (
					);

			#ifdef X_NATIVE
				Widget CreateEditForm (
					Widget parent,
					const Arg *arglist,
					int argcount
					);
			#endif

				bool GetAntialiasing (
					) const {return(m_Antialiasing); }

				bool GetAutoOpenDftTableOnSel (
					) const { return (m_AutoOpenDftTableOnSel); }

				DFTNAMEMODE GetDftNameMode (
					) const { return (m_DftNameMode); }

				bool GetDftRenderMultiPass (
					) const { return (m_DftRenderMultiPass); }

				bool GetDftShowSystemTables (
					) const { return (m_DftShowSystemTables); }

				bool GetForceGeorefAffine (
					) const { return (m_ForceGeorefAffine); }

				bool GetHintThinLines (
					) const {return(m_HintThinLines);}

				//! Read settings from Ini file.
				//! By default, settings are shared among all processes.
				//! To retain settings separately, use GRE::SYSTEM::SetGlobalIniGroup() during application initialization.
				void IniRead ();

				//! Write settings to Ini file.
				//! By default, settings are shared among all processes.
				//! To retain settings separately, use GRE::SYSTEM::SetGlobalIniGroup() during application initialization.
				void IniWrite () const;

				void SetAntialiasing (
					bool antialias
					) { m_Antialiasing = antialias; }

				void SetAutoOpenDftTableOnSel (
					bool AutoOpenDftTableOnSel
					) { m_AutoOpenDftTableOnSel = AutoOpenDftTableOnSel; }

				void SetDftNameMode (
					DFTNAMEMODE DftNameMode
					) { m_DftNameMode = DftNameMode; }

				void SetDftShowSystemTables (
					bool DftShowSystemTables
					) { m_DftShowSystemTables = DftShowSystemTables; }
					
				void SetForceGeorefAffine (
					bool ForceGeorefAffine
					) { m_ForceGeorefAffine = ForceGeorefAffine; }
					
				void SetHintThinLines (
					bool hint
					) { m_HintThinLines = hint;}

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				DFTNAMEMODE m_DftNameMode;
				bool m_AutoOpenDftTableOnSel;			//!< Automatically open default table on element selection (highlighting)
				friend class LAYER;
				bool m_Antialiasing;
				bool m_HintThinLines;
				bool m_DftRenderMultiPass;
				bool m_DftShowSystemTables;
				bool m_ForceGeorefAffine;

				#endif // GENERATING_DOXYGEN_OUTPUT
			};	// End of LAYER::GLOBALOPTIONS

		//! Sentry class to disable redrawing of legends.
		class LEGENDREDRAWDISABLER {
			public:
				LEGENDREDRAWDISABLER (GRE::LAYER *layer);
				~LEGENDREDRAWDISABLER ();
			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				GRE::LAYER *m_layer;
				#endif
			};

		class COMMONPARM;
		class DISPPARM;

		class TYPEDEFN;
		class COMPINFO;
		class FILTER;
		class FILTER_EXCLUDE;

		//! Determine if can construct layer from RVC object.
		static bool CanConstructFromObject (
			const RVC::OBJITEM& objitem
			);

		//! Construct layer of specified type and assign specified object.
		static ERRVALUE ConstructAndAssignObject (
			GRE::GROUP *group,								//!< Group to add layer to
			LAYER::TYPE layertype,					//!< Type of layer to construct
			const RVC::OBJITEM& objitem,				//!< Object to assign
			LAYER **retlayer = 0,					//!< Layer returned, NULL if not needed
			CREATEFLAGS createflags = LAYER::CREATEFLAG_None,	//!< Layer creation flags
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,	//!< Position for new layer in list
			LAYER *reflayer = 0						//!< Reference layer for insertion before or after
			);

	#ifdef USE_MGUI
		//! Construct layer of specified type, prompt for object, and optional open settings window.
		static ERRVALUE ConstructAndPromptObject (
			MDLGPARENT dlgparent,						//!< Parent for dialogs
			GRE::GROUP *group,								//!< Group to add layer to
			LAYER::TYPE layertype,					//!< Type of layer to construct
			LAYER **retlayer,						//!< Layer returned, NULL if not needed
			CREATEFLAGS createflags,					//!< Layer creation flags
			GRE::LISTPOS listpos,						//!< Position for new layer in list
			LAYER *reflayer,							//!< Reference layer for insertion before or after
			bool OpenSettings								//!< true to open settings window
			);
	#endif

		//! Construct layer from specified object.
		static ERRVALUE ConstructFromObject (
			GRE::GROUP *group,							//!< Group to contain new layer
			const RVC::OBJITEM& objitem,				//!< Object to construct layer for
			LAYER **retlayer = 0,						//!< New layer returned
			CREATEFLAGS createflags = CREATEFLAG_None,//!< Creation flags
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,	//!< Position of new layer in list
			LAYER *reflayer = 0,						//!< Reference layer for insertion
			MDLGPARENT dlgparent = 0						//!< Parent for verification dialogs, NULL to not use
			);

		//! Construct multiple layers using object list (quick-add).
		static ERRVALUE ConstructFromObjList (
			GRE::GROUP *group,									//!< Group to construct layers in, layers will be added after active layer
			const RVC::OBJITEMLIST& objlist,					//!< List of objects to construct layers for
			MDLGPARENT dlgparent = 0,							//!< Parent for verification dialogs, NULL to not use
			GRE::LISTPOS listpos = GRE::LISTPOS_After,	//!< Position of new layer in list, default to placing after active layer
			LAYER *reflayer = 0									//!< Reference layer for insertion
			);

		//! Get default layer naming mode.
		static GLOBALOPTIONS::DFTNAMEMODE GetDftNameMode (
			) { return (s_GlobalOptions.m_DftNameMode); }

		//! Get global options applying to all layers.
		static const GLOBALOPTIONS& GetGlobalOptions (
			) { InitStatic(); return (s_GlobalOptions); }

		//! Get number of registered layer types
		static UINT16 GetNumTypes (
			) { return (s_NumTypes); }

		//! Get type definition for specific layer type index
		static const TYPEDEFN& GetTypeDefn (
			UINT16 idx
			) { return (*s_TypeDefns[idx]); }

		//! Get type definition for specific layer type
		static const TYPEDEFN* GetTypeDefn (
			TYPE layertype
			);

		//! Get TEXTID for specific layer type name.
		static TEXTID GetTypeName (
			TYPE layertype
			);

	#ifdef USE_MGUI
		//! Prompt for geospatial databases/links construct layers in specified group.
		static ERRVALUE PromptGeoDatabasesAndConstruct (
			MDLGPARENT dlgparent,						//!< Parent for dialog
			GRE::GROUP *group								//!< Group to add layers to
			);
	#endif

	#ifdef USE_MGUI
		//! Prompt for multiple objects for quick-add.
		static ERRVALUE PromptObjects (
			MDLGPARENT dlgparent,						//!< Parent for dialog
			RVC::OBJITEMLIST& objlist,					//!< List of selected objects returned
			MGUI::DLG_GETOBJECT_STYLE dlgstyle = MGUI::DLG_GETOBJECT_STYLE_OnlyExisting,
			const MISTRING *pDlgTitle = 0				//!< Title for prompt dialog (default is TEXTID_Prompt_Objects_Display)
			);
	#endif

	#ifdef USE_MGUI
		//! Prompt for multiple objects and construct layers in specified group.
		static ERRVALUE PromptObjectsAndConstruct (
			MDLGPARENT dlgparent,						//!< Parent for dialog
			GRE::GROUP *group,								//!< Group to add layers to
			MGUI::DLG_GETOBJECT_STYLE dlgstyle = MGUI::DLG_GETOBJECT_STYLE_AllowDuplicate|MGUI::DLG_GETOBJECT_STYLE_WebLayers,	//!< Style
			GRE::LISTPOS listpos = GRE::LISTPOS_After,	//!< Position of new layer in list, default to placing after active layer
			LAYER *reflayer = 0
			);
	#endif

		//! Register layer type
		static void RegisterLayerType (
			TYPEDEFN& typedefn
			);

		//! Set whether the default attribute table should be opened on element selection.
		static void SetAutoOpenDftTableOnSel (
			bool AutoOpenDftTableOnSel
			) { s_GlobalOptions.m_AutoOpenDftTableOnSel = AutoOpenDftTableOnSel; }

		//! Set default layer naming mode
		static void SetDftNameMode (
			GLOBALOPTIONS::DFTNAMEMODE DftNameMode
			) { s_GlobalOptions.m_DftNameMode = DftNameMode; }

		//! Set global options applying to all layers.
		static void SetGlobalOptions (
			const GLOBALOPTIONS& options
			) { s_GlobalOptions = options; }

		//! Method to install SML class, called by internal SML function.
		static void SMLClassInstall (
			SML::CONTEXT *context							//!< SML context
			);

		static SML::CLASSMEMBER* GetSMLClassMembers (
			);

		//! Constructor.
		LAYER (
			GRE::GROUP *group,									//!< Group to contain layer
			TYPE layertype,									//!< Layer type
			DISPPARM& dispparm,								//!< Reference to layer's internal DISPPARM
			CREATEFLAGS createflags,						//!< Creation flags
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,	//!< Position in list
			LAYER *reflayer = 0							//!< Reference layer for insertion
			);

		//! Destructor
		virtual ~LAYER ();

		//! Assign layer to use specified object.
		ERRVALUE AssignObject (
			const RVC::OBJITEM& objitem,				//!< Object to assign
			MDLGPARENT dlgparent = 0					//!< Parent for verification/setting dialogs, NULL to not use such dialogs
			) { return (v_AssignObject(objitem,dlgparent)); }

		int CallDrawFilter (LAYERDC *ldc, const RVC::ELEMENT& element, void *vpStyle) {
			if (m_DrawFilter != 0) return (m_DrawFilter(ldc, element.GetType(), static_cast<INT32>(element.GetNumber()), vpStyle));
			return (1);
			}
		
		int CallDrawFilter (LAYERDC *ldc, ELEMTYPE ElemType, INT32 ElemNum, void *vpStyle) {
			if (m_DrawFilter != 0) return (m_DrawFilter(ldc, ElemType, ElemNum, vpStyle));
			return (1);
			}

		//! Determine if capable of retrieving element extents
		bool CanGetElemExtents (
			) const { return (m_CanGetElemExtents); }

		//! Determine if can open control dialog for layer
		bool CanOpenControlPanel (
			) const { return (!m_IsControlPanelDisabled); }

		//! Determine if can use layer in rendering
		bool CanRender (
			) const { return (v_CanRender()); }

		//! Determine if capable of reporting raw element information
		bool CanReportRawElemInfo (
			) const { return (m_CanReportRawElemInfo); }

		//! Determine if can show "details" (components).
		bool CanShowDetails (
			) const { return (v_CanShowDetails()); }

		//! Determine if can show layer in specific or any view.
		bool CanShowInView (
			const VIEW *view = 0				//!< Specific view or NULL for "any" view
			) const;

		//! Determine if element can be tested if inside/outside region
		bool CanTestElementByRegion (
			) const { return (m_CanTestElementByRegion); }

		//! Determine if layer can be viewed in 2D.
		bool CanViewIn2D () const;

		//! Determine if layer can be viewed in 3D.
		bool CanViewIn3D () const
			{ return (m_CanViewIn3D); }

		//! Determine if can view in stereo 2D overhead mode.
		bool CanViewStereo2D () const
			{ return (m_CanViewStereo2D && CanViewIn2D()); }
		
		//! Determine if layer can be zoomed to maximum detail.
		bool CanZoomToMaxDetail (
			) const;

		//! Change position in list
		bool ChangeListPos (
			LAYER* reflayer,							//!< Reference layer for LISTPOS_Before and LISTPOS_After
			GRE::LISTPOS listpos							//!< New position in list
			);

		//! Check for high feature density/detail and let user set visibility options if exceeds threshold.
		ERRVALUE CheckDenseDetail (
			MDLGPARENT dlgparent							//!< Parent for dialog
			);

		//! Request close of object(s) associated with layer.
		void CloseObject (
			) { v_CloseObject(); };

		//! Compute default maximum visible scale based on object element detail/density.
		double ComputeDftMaxVisibleScale () const
			{ return (v_ComputeDftMaxVisibleScale()); }

		//! Compute "margins" for layer
		void ComputeMargins (
			double& tmargin,								//!< Top margin in "layer" coordinates returned
			double& bmargin,								//!< Bottom margin in "layer" coordinates returned
			double& lmargin,								//!< Left margin in "layer" coordinates returned
			double& rmargin								//!< Right margin in "layer" coordinates returned
			) { v_ComputeMargins(tmargin,bmargin,lmargin,rmargin); }

	#ifndef GENERATING_DOXYGEN_OUTPUT
		// Construct LAYERSECTION instance for this layer and specified view.
		GRE::LAYERSECTION* ConstructLayerSection (
			GRE::VIEW *view
			) { return (v_ConstructLayerSection(view)); }
	#endif
	
	#ifndef GENERATING_DOXYGEN_OUTPUT
		// Construct LAYERVIEW instance for this layer and specified view.
		GRE::LAYERVIEW* ConstructLayerView (
			GRE::VIEW *view
			) { return (v_ConstructLayerView(view)); }
	#endif
	
		//! Make a copy of 'this' given a parent group to belong to.
		LAYER* Copy (
			GRE::GROUP* NewGroup
			) const { return (v_Copy(NewGroup)); }

		//! Close datatip tables.
		void DataTipCloseTables (
			);

		//! Get default DataTip prefix for specified component.
		void DataTipGetDefaultPrefix (
			int component,									//!< Component index
			DATATIPPARMS& DataTipParms					//!< DataTip parameters passed/returned
			) const { v_DataTipGetDefaultPrefix(component,DataTipParms); }

		//! Get ObjectNames datatip string.
		void DataTipGetObjectNamesString (
			const DPOINT2D& point,
			MISTRING& string,
			int indent = 0,
			GRE::VIEW *view = 0				//!< Optional view to determine whether to limit to 'visible' elements
			) const { return (v_DataTipGetObjectNamesString(point, string, indent, view)); }

		//! Get DataTip settings for specified component.
		//! @return Pointer to DataTip settings, NULL if not valid for component
		const DATATIPPARMS *DataTipGetParms (
			int component									//!< Component index
			) const { return (v_DataTipGetParms(component)); }

		//! Get DataTip search distance in pixels.
		int DataTipGetSearchDist (
			) const;

		//! Get DataTip text for specified element.
		//! @return TRUE if have DataTip, FALSE if not, < 0 if error.
		int DataTipGetString (
			ELEMTYPE ElemType,							//!< Element type
			ELEMNUM ElemNum,									//!< Element number
			MISTRING& string,								//!< String to contain result
			SPATMOD::IMAGE::STAGE_ARRAY& images		//!< If the tip will be an image, append an IMAGE_STAGE for it here.
			) { return (v_DataTipGetString(ElemType,ElemNum,string,images)); }

		//! Get DataTip text for specified point.
		//! If a view is specified and the layer is visible in that view then only
		//! visible elements will be searched.
		//! @return TRUE if have string, FALSE if not, < 0 if error
		int DataTipGetString (
			const DPOINT2D& point,						//!< Point in layer coordinates
			MISTRING& string,								//!< String to contain result
			SPATMOD::IMAGE::STAGE_ARRAY& images,	//!< If the tip will be an image, append an IMAGE_STAGE for it here.
			GRE::VIEW *view = 0							//!< Optional view to determine whether to limit to 'visible' elements
			);

		//! Determine if has DataTip available for any component.
		bool DataTipHasAvailable () const;

		//! Determine if has DataTip enabled for any component.
		bool DataTipHasEnabled () const;

		//! Determine if DataTip is enabled for this layer.
		//! May also need to use DataTipHasAvailable to determine if a DataTip is actually available.
		bool DataTipIsEnabled (
			) const { return (m_DataTipEnabled); }

		//! Set DataTip to default for all components.
		void DataTipSetDefault (
			bool force = false							//!< Force to default even if already set
			);

		//! Set DataTip to default for specified component index.
		void DataTipSetDefault (
			int component,									//!< Component index
			bool force = false							//!< Force to default even if already set
			);

		//! Set default DataTip prefix for specified layer component.
		void DataTipSetDefaultPrefix (
			int component									//!< Component index
			);

		//! Set delegate for getting DataTip content.
		//! Delegate will be called before layer-specific method.  If delegate returns 0
		//! then method for layer will be tried.
		void DataTipSetDelegateGetContent (
			DELEGATE_DATATIPGETCONTENT delegate
			) { m_DelegateDataTipGetContent = delegate; }

		//! Set DataTip parameters for specified layer component
		ERRVALUE DataTipSetParms (
			int component,									//!< Component index
			const DATATIPPARMS& DataTipParms			//!< Parameter values to set
			) { return (v_DataTipSetParms(component,DataTipParms)); }

		//! Set DataTip table/field for component.
		ERRVALUE DataTipSetTableField (
			int component,
			const RVC::OBJECTNAME& TableName,		//!< Short table name
			const RVC::OBJECTNAME& FieldName			//!< Short field name
			);

		//! Set DataTip table/field for component.
		ERRVALUE DataTipSetTableField (
			int component,
			const char *TableName,						//!< Short table name
			const char *FieldName						//!< Short field name
			);

		//! Set DataTip visibility for component.
		ERRVALUE DataTipSetVisible (
			int component,
			bool visible
			);

		//! Translate element number to DbList index.
		INT32 DbListElemNumToIndex (
			int component,
			ELEMNUM ElemNum
			) { return (v_DbListElemNumToIndex(component,ElemNum)); }

		//! Determine if DbList index is same as element number.
		bool DbListIsIndexElemNum (
			) const { return (m_DbListIsIndexElemNum); }

		//! Destroy all LAYERVIEW instances associated with this layer.
		void DestroyAllLayerViews (
			);

		//! Disable access to layer controls.
		void DisableControlPanel (
			) { m_IsControlPanelDisabled = true; }

	#ifdef USE_MGUI
		//! Modal dialog for DataTip settings.
		void DlgDataTipSettings (
			MDLGPARENT dlgparent
			);
	#endif

	#ifdef USE_MGUI
		//! Dialog to mark elements by query.
		ERRVALUE DlgHighlightByQuery (
			MDLGPARENT dlgparent
			);
	#endif

		//! Draw this layer.
		ERRVALUE Draw (
			LAYERDC& layerdc								//!< Drawing control parameters
			) { return (v_Draw(layerdc)); }

		//! Draw all views containing layer.
		ERRVALUE DrawAllViews (
			GRE::DRAWCONDITION condition = GRE::DRAWCONDITION_Always
			);

		//! Draw single element given ELEMTYPE and element number.
		ERRVALUE DrawElement (
			GRE::VIEW *view,								//!< View to draw in, 0 for all
			ELEMTYPE ElemType,							//!< Element type
			ELEMNUM ElemNum,									//!< Element number
			GRE::RENDERTARGET Target,					//!< Target buffer(s) to render to
			GRE::DRAWFLAGS drawflags = GRE::DRAWFLAG_None	//!< Drawing flags
			);

		//! Draw single element given RVC::ELEMENT.
		ERRVALUE DrawElement (
			GRE::VIEW *view,								//!< View to draw in, 0 for all
			const RVC::ELEMENT& element,				//!< Element specifier
			GRE::RENDERTARGET Target,					//!< Target buffer(s) to render to
			GRE::DRAWFLAGS drawflags = GRE::DRAWFLAG_None	//!< Drawing flags
			) { return (DrawElement(view,element.GetType(),element.GetNumber(),Target,drawflags)); }

		//! Draw single element for this layer.
		ERRVALUE DrawElementLow (
			LAYERDC& layerdc,								//!< Drawing control parameters
			ELEMTYPE ElemType,							//!< Element type
			ELEMNUM ElemNum,									//!< Element number
			const void *style = 0						//!< Style for override, 0 for default
			) { return (v_DrawElementLow(layerdc,ElemType,ElemNum,style)); }

		//! Draw multiple elements specified by UNSIGNEDSET.
		ERRVALUE DrawElements (
			GRE::VIEW *view,								//!< View to draw in, 0 for all
			ELEMTYPE ElemType,							//!< Element type
			const UNSIGNEDSET *pElemSet,				//!< Set of elements to draw, NULL for all
			GRE::RENDERTARGET Target,					//!< Target buffer(s) to render to
			GRE::DRAWFLAGS drawflags = GRE::DRAWFLAG_None	//!< Drawing flags
			);

		//! Draw set of elements for this layer.
		ERRVALUE DrawElementsLow (
			LAYERDC& layerdc,								//!< Drawing control parameters
			ELEMTYPE ElemType,							//!< Element type
			const UNSIGNEDSET *pElemSet,				//!< Set of elements to draw, NULL for all
			const void *style = 0						//!< Style for override, 0 for default
			) { return (v_DrawElementsLow(layerdc,ElemType,pElemSet,style)); }

		//! Draw element sample.
		ERRVALUE DrawElementSample (
			const GRE::ELEMSAMPLE_DC& sampledc,		//!< Drawing control parameters
			const RVC::ELEMENT& element,				//!< Element
			const LRECT2D& samplerect					//!< Destination rectangle
			) { return (v_DrawElementSample(sampledc,element,samplerect)); }

		//! Draw highlighted elements.
		ERRVALUE DrawHighlighted (
			GRE::VIEW *view,								//!< View to draw in, 0 for all
			GRE::DRAWFLAGS DrawFlags = GRE::DRAWFLAG_None
			);

		//! Draw a single label for this layer
		ERRVALUE DrawLabel (
			LAYERDC& layerdc,								//!< Drawing control parameters
			const GRE::LABELPLACE& place,
			const COLOR* ForegroundColor = 0			//!< Overrides foreground color if not NULL
			);

		//! Draw set of labels for this layer
		ERRVALUE DrawLabels (
			LAYERDC& layerdc								//!< Drawing control parameters
			);

		//! Draw overlay.
		ERRVALUE DrawOverlay (
			LAYERDC& layerdc								//!< Drawing control parameters
			) { return (v_DrawOverlay(layerdc)); }

		//! Enable DataTip for layer if available.
		//! The DataTip is enabled by default, this setting allows the user to easily turn DataTips on/off by layer.
		void EnableDataTip (
			bool enable = true
			) { m_DataTipEnabled = enable; return; }

		//! Estimate range of Z values over portion of layer in view.
		//! May return invalid (ie empty) range if not supported or view is outside data extent.
		ERRVALUE EstimateRangeZ (
			const LAYERVIEW *pLayerView,				//!< Layer view
			DOUBLE_RANGE& range							//!< Range returned
			) const;

		//! Find closest element to specified point.
		//! @return element number, -1 if not found, < -1 if error
		ELEMNUM FindClosestElement (
			const DPOINT2D& point,						//!< Point in "object" coordinates
			double searchdist,							//!< Distance to search in "object" coordinates
			double *distret,								//!< Distance returned in "object" coordinates, (NULL if don't need)
			int component,									//!< Component to search
			const UNSIGNEDSET *pSearchSet,			//!< Set of elements to search, NULL for all elements
			GRE::FINDFLAGS findflags					//!< Flags for searching
			) { return (v_FindClosestElement(point,searchdist,distret,component,pSearchSet,findflags)); }

		//! Find elements matching specified query.
		//! @return number of elements found, < 0 if error
		INT64 FindElementsByQuery (
			ELEMTYPE ElemType,							//!< Element type
			UNSIGNEDSET& ElemSet,						//!< Set of elements found
			const MIUNICODE *QueryScript,				//!< Query script string
			SML::CONTEXT *pQueryContext = 0			//!< Query/SML context
			);

		//! Find elements matching specified query with "match quality".
		//! An array of values, one per element, is returned with values from 0-255 corresponding
		//! to query return values from 0.0 to 1.0.  Values < 0 will be 0, values > 255 will be 255.
		ERRVALUE FindElementsByQueryExt (
			ELEMTYPE ElemType,							//!< Element type
			SIMPLE_ARRAY<UINT8>& Result,				//!< Result for each element
			const MIUNICODE *QueryScript,				//!< Query script string
			SML::CONTEXT *pQueryContext = 0			//!< Query/SML context
			);

		//! Find surface layer for draping this layer onto in 3D.
		//! @return Pointer to surface layer or NULL if none.
		LAYER_SURFACE* FindSurfaceLayer (
			) const;

	#ifndef GENERATING_DOXYGEN_OUTPUT
		// Make protected when possible.
		void FinishCreation (
			bool notify = true							//!< Send creation notification
			);

	#endif // GENERATING_DOXYGEN_OUTPUT

		//! Get active element.
		const RVC::ELEMENT& GetActiveElement () const
			{ return (m_ActiveElem); }

		//! Get active element as RVC::ELEMENT.
		//! @return 'true' if have active element, false if not.
		bool GetActiveElement (
			RVC::ELEMENT& element						//!< Element specifier returned
			) const;

		//! Get active element number.
		//! @return Element number or -1 if no active element.
		ELEMNUM GetActiveElemNum (
			) const { return (m_ActiveElem.GetNumber()); }

		//! Get active element type.
		//! @return Element type, ELEMTYPE_Empty if no active element.
		ELEMTYPE GetActiveElemType (
			) const { return (m_ActiveElem.GetType()); }

		//! Get color for 3D pedestal.
		const COLOR& GetColorPedestal (
			) const;

		//! Get component for specified element type.
		//! @return Component index, -1 if no corresponding component.
		int GetComponent (
			ELEMTYPE ElemType								//!< Element type
			) const { return (v_GetComponent(ElemType)); }

		//! Get icon for component.
		//! @return Icon for component or NULL if none
		MICONSTRUCT* GetComponentIcon (
			int component									//!< Component index
			) const { return (v_GetComponentIcon(component)); }

		//! Get component information.
		//! @return Pointer to component information or NULL if none.
		COMPINFO* GetComponentInfo (
			int component									//!< Component index
			) { return (v_GetComponentInfo(component)); }

		//! Get component information (const version).
		//! @return Pointer to component information or NULL if none.
		const COMPINFO* GetComponentInfo (
			int component
			) const { return (const_cast<LAYER*>(this)->GetComponentInfo(component)); }

		//! Get component information by element type.
		//! @return Pointer to component information or NULL if none.
		COMPINFO* GetComponentInfoByType (
			ELEMTYPE elemtype								//!< Element type
			);

		//! Get localized name of component.
		//! @return Component name.
		MISTRING GetComponentName (
			int component									//!< Component index
			) const { return (v_GetComponentName(component)); }

		//! Get DISPFLAGS for layer.
		DISPFLAGS GetDispFlags (
			) const;

		RVC::DISPPARMUSAGE GetDispParmUsage (
			) const { return ((layerflags & LAYERFLAG_Editable) ? RVC::DISPPARMUSAGE_Edit : RVC::DISPPARMUSAGE_Display); }

		//! Get pointer to default style of element type.
		//! @return Pointer to style, do not free.
		const void* GetElemDefaultStyle (
			ELEMTYPE ElemType								//!< Element type
			) const { return (v_GetElemDefaultStyle(ElemType)); }

		//! Get selection mode for element drawing
		//! @return Selection mode
		UINT16 GetElemDrawSelectionMode (
			ELEMTYPE ElemType								//!< Element type
			) const { return (v_GetElemDrawSelectionMode(ElemType)); }

		//! Get style mode for element drawing.
		//! @return style mode
		UINT16 GetElemDrawStyleMode (
			ELEMTYPE ElemType								//!< Element type
			) const { return (v_GetElemDrawStyleMode(ElemType)); }

		//! Get extents of specified element.
		ERRVALUE GetElemExtents (
			ELEMTYPE ElemType,							//!< Element type
			ELEMNUM ElemNum,									//!< Element number
			DRECT3D& Extents								//!< Extents returned
			) { return (v_GetElemExtents(ElemType,ElemNum,Extents)); }

		//! Get mode for element highlighting.
		GRE::HIGHLIGHTMODE GetElemHighlightMode (
			) const;

		//! Get element highlight set.
		//! @return NULL if element type not supported or cannot be highlighted.
		UNSIGNEDSET* GetElemHighlightSet (
			ELEMTYPE ElemType = ELEMTYPE_All			//!< Element type
			);

		//! Get combined extents of all elements in set.
		ERRVALUE GetElemSetExtents (
			ELEMTYPE ElemType,							//!< Element type
			const UNSIGNEDSET& ElemSet,				//!< Set of elements
			DRECT3D& Extents								//!< Extents in layer object coordinates returned
			);

		//! Get element type associated with specified component.
		ELEMTYPE GetElemType (
			int component									//!< Component index
			) const { return (v_GetElemType(component)); }

		//! Get layer extents in 'object' coordinates.
		const DRECT3D& GetExtents (
			) const { return (m_ObjRegion.GetExtents()); }

		//! Get group containing layer.
		GRE::GROUP * GetGroup (
			) const { return (m_group); }

		//! Get georeference name for layer component.
		//! @return 'True' if layer has a georeference name, 'false' if not
		bool GetGeorefName (
			RVC::OBJECTNAME& gname,
			int component = 0								//!< Component index
			) const { return (v_GetGeorefName(gname,component)); }

		//! Get LAYERDRAWFLAGS for layer.
		LAYERDRAWFLAGS GetLayerDrawFlags (
			) const { return (static_cast<LAYERDRAWFLAGS>(m_drawflags)); }

		//! Get LAYERFLAGS for layer.
		LAYERFLAGS GetLayerFlags (
			) const { return (static_cast<LAYERFLAGS>(layerflags)); }

		//! Get layer type.
		TYPE GetLayerType (
			) const { return (m_layertype); }

		//! Get legend for specified element type.
		//! @return Pointer to legend or NULL if no legend.
		const GRE::LEGEND* GetLegend (
			ELEMTYPE ElemType = ELEMTYPE_Empty		//!< Element type
			) { return (v_GetLegend(ElemType)); }

	#ifdef X_NATIVE
		GraphicTreeItemMsgHandler GetLegendMsgHandler (
			) const { return (m_LegendMsgHandler); }
	#endif

		//! Get current layer drawing order position in list.
		//! @return 0 for first layer, 1 for second, etc.
		int GetListPos (
			);

		//! Get map coordinate reference system used by layer.
		//! This is not necessarily the CRS the layer will be rendered in as that
		//! is determined by the containing group.
		const SPATREF::COORDREFSYS& GetMapCoordRefSys (
			) const { return (GetMapRegion().GetCoordRefSys()); }

		//! Get bounding region in map (georeferenced) coordinates.
		const REGION2D& GetMapRegion (
			) const { return (m_MapRegion); }

		double GetMapScale (
			) const;

		UINT8 GetMaskBlue (
			) const { return (m_MaskBlue); }

		UINT8 GetMaskGreen (
			) const { return (m_MaskGreen); }

		UINT8 GetMaskRed (
			) const { return (m_MaskRed); }

		double GetMaxScaleVisible (
			bool IncludeGroup = false					//!< Include group scale range settings, if any
			) const;

		double GetMinScaleVisible (
			bool IncludeGroup = false					//!< Include group scale range settings, if any
			) const;

		UINT32 GetModifyStamp (
			) const { return (m_ModifyStamp); }

		//! Get name of layer.
		const MISTRING& GetName (
			) const { return (m_Name); }

	#ifndef NO_DEPRECATED
		//! Get name of layer as MISTRING.
		//! \deprecated Use variant returning const MISTRING reference.
		DEPRECATED void GetName (
			MISTRING& retname
			) { retname = m_Name; }
	#endif

		//! Get next layer in group.
		LAYER* GetNext (
			) const { return (m_nextlayer); }

		int GetNumComponents (
			) const;

		//! Get number of elements of specified type.
		//! @return number of elements, < 0 if error.
		INT64 GetNumElements (
			ELEMTYPE ElemType								//!< Element type
			) { return (v_GetNumElements(ElemType)); }

		//! Get number of deleted elements of specified type.
		//! @return number of elements, < 0 if error.
		INT64 GetNumElementsDeleted (
			ELEMTYPE ElemType								//!< Element type
			) { return (v_GetNumElementsDeleted(ElemType)); }

		int GetNumObjects (
			) const;

		//! Get RVC::OBJITEM for layer component.
		//! @return Reference to object item, will be unattached if no object
		const RVC::OBJITEM& GetObjItem (
			int component = 0								//!< Component index
			) const { return (v_GetObjItem(component)); }

		//! Get bounding region in 'object' coordinates.
		const REGION2D& GetObjRegion (
			) const { return (m_ObjRegion); }

		//! Get previous layer in group.
		LAYER* GetPrevious (
			) const { return (m_prevlayer); }

		//! Get scale for rendering layer at maximum detail.
		double GetScaleMaxDetail (
			const DPOINT2D& point,						//!< Point at which to determine scale
			const SPATREF::COORDREFSYS& crs,			//!< CRS of point
			double DisplayPixelSizeMM					//!< Pixel size of target display in millimeters
			) const;

		void GetSizeMeters (
			double& xsize,
			double& ysize
			) const;

		//! Get style information for specified element type
		//! @return 0 if OK, -1 if no style object, < -1 if error
		ERRVALUE GetStyleInfo (
			ELEMTYPE ElemType,							//!< Element type
			RVC::OBJITEM& ObjItem,						//!< Style object item returned
			RVC::OBJECTNAME& TableName					//!< Style assignment table name returned
			) { return (v_GetStyleInfo(ElemType,ObjItem,TableName)); }

		//! Get tranformation from layer to group's map coordinates in TRANS2D_MAPGEN
		void GetTransGroup (
			TRANS2D_MAPGEN& trans						//!< Transformation to fill in
			) const;

		//! Get transformation from layer to map in TRANS2D_MAPGEN
		void GetTransMap (
			TRANS2D_MAPGEN& trans,						//!< Transformation to fill in
			bool setoutput									//!< True to set in output side, false for input side
			) const;

		//! Get type definition for this layer
		const TYPEDEFN& GetTypeDefn (
			) const { return (*GetTypeDefn(m_layertype)); }

		//! Get unique layer ID.
		//! IDs are unique within the scope of the topmost VIEWABLE object.  Thus all
		//! layers in a layout have unique IDs within that layout.
		UINT32 GetUniqueID (
			) const { return (m_UniqueID); }

		//! Get top-level GRE::VIEWABLE which contains layer.
		GRE::VIEWABLE* GetViewable (
			) const;

		//! Get "map scale" of first view containing layer.
		//! @return "inverse" of scale (eg. 24000 for 1:24000)
		double GetViewMapScale (
			) const;

		//! Get visibility for layer in group views.
		UINT32 GetVisibleGroup (
			) const { return (m_VisibleViewGroup); }

		//! Get visibility for layer in layout views.
		UINT32 GetVisibleLayout (
			) const { return (m_VisibleViewLayout); }

		//! Get scale range over which layer is visible
		void GetVisibleScaleRange (
			double& MinScaleVisible,					//!< Minimum scale visible returned, 0 for no minimum
			double& MaxScaleVisible,					//!< Maximum scale visible returned, 0 for no maximum
			bool IncludeGroup = false					//!< Include group scale range settings, if any
			) const;

		//! Get visibility for layer in viewable.
		UINT32 GetVisibleViewable (
			) const;

		//! Get 3D wireframe color for layer.
		const COLOR& GetWireframeColor (
			) const;

		//! Get Z offset (unscaled) for layer.
		double GetZOffset (
			) const;

		//! Get Z range (scaled) for layer.
		void GetZRangeScaled (
			DOUBLE_RANGE& zrange
			) const;

		//! Determine if has valid active element.
		bool HasActiveElement ();

		//! Determine if layer has color plane masks set.
		bool HasColorMasks (
			) const { return (m_MaskRed != 0 || m_MaskGreen != 0 || m_MaskBlue != 0); }

		//! Determine if drawing filter set for layer.
		bool HasDrawFilter (
			) const { return (m_DrawFilter != 0); }

		//! Determine if has existing "display parameters" subobject.
		bool HasDispParmSubObject () const
			{ return (v_HasDispParmSubObject()); }

		//! Determine if layer has highlighted elemente.
		bool HasHighlighted (
			) const { return (v_HasHighlighted()); }

		//! Determine if layer has a legend.
		bool HasLegend (
			) const { return (v_HasLegend()); }

		//! Determine if layer has null "image" cells.
		bool HasNullImageCells (
			) const { return (v_HasNullImageCells()); }

		//! Determine if layer has valid extents
		bool HasValidExtents (
			) const { return (m_HasExtents); }

		//! Highlight single element in layer.
		//! @return TRUE if selected set changed, FALSE if not, < 0 if error
		int HighlightElement (
			ELEMTYPE ElemType,
			ELEMNUM ElemNum,
			GRE::HIGHLIGHTMODE HighlightMode,
			GRE::VIEW *firstview = 0,
			GRE::DRAWFLAGS drawflags = GRE::DRAWFLAG_None	//!< Drawing flags
			);

		//! Highlight set of elements in layer.
		//! @return TRUE if selected set changed, FALSE if not, < 0 if error
		int HighlightElements (
			ELEMTYPE ElemType,
			const UNSIGNEDSET *pElemSet,							//!< Set of elements to highlight, NULL for all/none (depending on highlight mode)
			GRE::HIGHLIGHTMODE HighlightMode,
			GRE::VIEW *firstview = 0,
			GRE::DRAWFLAGS drawflags = GRE::DRAWFLAG_None	//!< Drawing flags
			);

		//! Highlight elements matching specified query.
		//! @return number of elements found, < 0 if error
		int HighlightElementsByQuery (
			ELEMTYPE ElemType,
			SML::CONTEXT* QueryContext,
			int QuerySegment,
			const MIUNICODE* QueryScript,
			GRE::VIEW *firstview = 0,
			GRE::DRAWFLAGS drawflags = GRE::DRAWFLAG_None	//!< Drawing flags
			);

		//! Highlight elements using specified region.
		//! @return TRUE if highlight set changed, FALSE if not < 0 if error.
		int HighlightElementsByRegion (
			ELEMTYPE ElemType,
			const REGION2D& Region,
			TESTCRITERIA RegionTest,
			GRE::VIEW *firstview = 0,
			GRE::DRAWFLAGS drawflags = GRE::DRAWFLAG_None	//!< Drawing flags
			);

		void IncrementModifyStamp (
			) { ++m_ModifyStamp; }

		//! Invalidate all view buffers for this layer.
		void InvalidateAllViews (
			GRE::RENDERTARGET Target = GRE::RENDERTARGET_GroupAll | GRE::RENDERTARGET_LayerAll
			);

		bool IsAntialiased (
			) { return (v_IsAntialiased()); }

		//! Determine if layer is in the process of being created.
		bool IsBeingCreated (
			) const { return (m_IsBeingCreated); }

		//! Determine if specified component is marked as "selectable" for highlighting by user.
		bool IsComponentSelectable (
			int component
			) const;

		//! Determine if layer is editable.
		bool IsEditable (
			) const { return ((layerflags & LAYERFLAG_Editable) != 0); }

		//! Determine if excluded from group extents.
		bool IsExcludedFromGroupExtents () const;

		bool IsLineWidthHinted (
			) const { return (v_IsLineWidthHinted()); }

		//! Determine if managing point symbols (via LABELPLACEMENT).
		bool IsManagingPointSymbols () const;

		//! Returns true if one dimension of the layer size can be adjusted
		//! independantly of the other. 
		//! A return value of true indicates that calling SetRenderSize for
		//! for one dimension would compute the other.
		bool IsProportionVariable (
			DIMENSION dimension
			) const { return (v_IsProportionVariable(dimension)); }

		//! Determine if layer is from remote source (web).
		bool IsRemote (
			) const;

		//! Determine if legend should be sorted in natural order.
		bool IsSortLegendNatural (
			) const;

		//! Determine if table is usable for specified component.
		bool IsTableUsableForComponent (
			int component,
			const RVC::DBTABLE& table
			) const { return (v_IsTableUsableForComponent(component,table)); }

		bool IsValid (
			) const;

		//! Determine if layer visible at specified scale.
		bool IsVisibleAtScale (
			double scale
			) const;

		//! Determine if layer is visible (selected for drawing) in specified view.
		bool IsVisibleInView (
			const GRE::VIEW* view
			) const;

	#ifdef USE_MGUI
		//! Add menu items for specific element type.
		void MenuAddElemTypeItems (
			MGUI::MENU& menu,									//!< Menu to add to
			ELEMTYPE ElemType									//!< Element type
			) { v_MenuAddElemTypeItems(menu,ElemType); }
	#endif

	#ifdef USE_MGUI
		//! Add menu items for entire layer.
		void MenuAddLayerItems (
			MGUI::MENU& menu									//!< Menu to add to
			) { v_MenuAddLayerItems(menu); }
	#endif

	#ifdef USE_MGUI
		//! Add menu items for generating region.
		void MenuAddRegionGenItems (
			MGUI::MENU& menu,									//!< Menu to add to
			ELEMTYPE ElemType									//!< Element type
			) { v_MenuAddRegionGenItems(menu,ElemType); }
	#endif

	#ifdef USE_MGUI
		//! Add popup menu items for point within view.
		void MenuAddViewPopupItems (
			MGUI::MENU& menu,									//!< Menu to add to
			const DPOINT2D *point,							//!< Point in layer coordinates where popup event occurred, NULL if not in view
			VIEW *view											//!< View, NULL if not available
			) { v_MenuAddViewPopupItems(menu,point,view); }
	#endif

		//! Notify layer that associated surface layer has changed.
		void NotifySurfaceChanged (
			) { v_OnNotifySurfaceChanged(); }

	#ifdef USE_MGUI
		int OpenControlPanel (
			MDLGPARENT dlgparent,
			bool AddingLayer = false
			);
	#endif

		//! Open object(s) for this layer
		//! @return TRUE if was opened, FALSE if open already, < 0 if error
		int OpenObject (
			MDLGPARENT dlgparent = 0					//!< Parent for dialog
			) { return (v_OpenObject(dlgparent)); }

		ERRVALUE PanViewToElement (
			GRE::VIEW *view,								//!< View to pan, 0 for all
			ELEMTYPE ElemType,
			ELEMNUM ElemNum,
			GRE::DRAWFLAGS drawflags = GRE::DRAWFLAG_None	//!< Drawing flags
			);

	#ifdef USE_MGUI
		//! Prompt user to select object for layer
		ERRVALUE PromptObject (
			bool OpenCtrlPanel = false,				//!< Whether or not to show control panel after selection
			MDLGPARENT dlgparent = 0					//!< Parent for dialogs
			) { return (v_PromptObject(OpenCtrlPanel,dlgparent)); }
	#endif

		//! Report "raw" information about specified element
		ERRVALUE ReportRawElemInfo (
			MISTRING& infostr,							//!< String to hold reported information
			ELEMTYPE ElemType,							//!< Element type
			ELEMNUM ElemNum									//!< Element number
			) { return (v_ReportRawElemInfo(infostr,ElemType,ElemNum)); }

		//! Save display parameters to subobject of layer's object.
		ERRVALUE SaveDispParmSubObject (
			RVC::DISPPARMUSAGE usage = RVC::DISPPARMUSAGE_Unspecified
			) { return (v_SaveDispParmSubObject(usage)); }

		//! Save display parameters to subobject of specified parent.
		ERRVALUE SaveDispParmSubObject (
			const RVC::OBJITEM& ParentObjItem,			//!< Parent object to save under
			RVC::DISPPARMUSAGE usage = RVC::DISPPARMUSAGE_Unspecified
			);

		//! Apply Z scale/offset to single point.
		void ScalePointZ (
			DPOINT3D& pt							//!< Point containing Z value to scale
			) const;

		//! Apply Z scale/offset to points.
		//! Has no effect unless both layer and polyline are 3D.
		void ScalePointsZ (
			POLYLINE& polyline					//! Polyline containing points
			) const;

		//! Read from serialization
		ERRVALUE SerialRead (
			SERIALIZER& serializer
			);

		//! Write values to serialization
		ERRVALUE SerialWrite (
			SERIALIZER& serializer,
			const char *tagname = 0
			) const { return (v_SerialWrite(serializer,tagname)); }

		//! Set active element.
		//! @return TRUE if have active element, FALSE if not, < 0 if error
		int SetActiveElement (
			ELEMTYPE ElemType,
			ELEMNUM ElemNum,
			SETACTIVEFLAGS flags = SETACTIVEFLAG_None
			);

		void SetAntialiased (
			bool antialias
			) { v_SetAntialiased(antialias); }

		//! Set color for 'active' element.
		//! @return true if color was changed, false if same as before.
		bool SetColorActive (
			const COLOR& color
			);

		//! Set color for 'highlighted' element.
		//! @return true if color was changed, false if same as before.
		bool SetColorHighlight (
			const COLOR& color
			);

		void SetColorMasks (
			UINT8 MaskRed,
			UINT8 MaskGreen,
			UINT8 MaskBlue
			) { m_MaskRed = MaskRed; m_MaskGreen = MaskGreen; m_MaskBlue = MaskBlue; }

		//! Set color for 'pedestal' in 3D mode.
		void SetColorPedestal (
			const COLOR& color
			);

		//! Set whether component is marked as "selectable" for highlighting by user.
		void SetComponentSelectable (
			int component,
			bool selectable = true
			);

		//! Set to use default layer name.
		void SetDftName (
			bool ReplaceExisting = false				//!< Replace existing name
			) { v_SetDftName(ReplaceExisting); }

		//! Set display flags for layer.
		void SetDispFlags (
			DISPFLAGS flags,
			DISPFLAGS mask
			);

		void SetDrawFilter (
			int (*DrawFilter)(LAYERDC*, ELEMTYPE, INT32, void*)
			) { m_DrawFilter = DrawFilter; }

		void SetDrawFlags (
			LAYERDRAWFLAGS flags,						//!< Flags to set
			LAYERDRAWFLAGS mask							//!< Mask for which flags to change
			);

		//! Set selection mode for element drawing
		void SetElemDrawSelectionMode (
			ELEMTYPE ElemType,							//!< Element type
			UINT16 SelectMode								//!< Selection mode
			) { v_SetElemDrawSelectionMode(ElemType,SelectMode); }

		//! Set style mode for element drawing
		void SetElemDrawStyleMode (
			ELEMTYPE ElemType,							//!< Element type
			UINT16 StyleMode								//!< Style mode
			) { v_SetElemDrawStyleMode(ElemType,StyleMode); }

		//! Set whether to exclude layer from group extents.
		void SetExcludedFromGroupExtents (
			bool excluded,
			bool notify = true
			);

		//! Notify layer that extents should be recomputed
		void SetExtentsChanged (
			) { m_HasExtents = false; }

		void SetFlags (
			LAYERFLAGS flags,								//!< Flag settings
			LAYERFLAGS mask,								//!< Mask for which flags to change
			bool notify = true							//!< True to invoke callback lists
			);

		//! Set function to call when user attempts to remove layer.
		void SetFuncRemoveCheck (
			int (*RemoveCheckFunc)(LAYER*,void*),
			void *RemoveCheckData
			) { m_RemoveCheckFunc = RemoveCheckFunc; m_RemoveCheckData = RemoveCheckData; }

		void SetHintThinLines (
			bool hint
			) { v_SetHintThinLines(hint); }

		//! Set whether to attempt to keep associated objects open or not.
		void SetKeepOpen (
			bool KeepOpen = true
			) { if (KeepOpen) this->layerflags |= LAYERFLAG_KeepOpen; else this->layerflags &= ~LAYERFLAG_KeepOpen; }

		//! Notify that legend has been changed.
		void SetLegendChanged (
			bool notify = true							//!< Send change messages to callback lists
			) {
			if (notify) CallbackInvoke(GRE::MSG::ACTION_ChangeBegin);
			m_LegendValid = false;
			if (notify) CallbackInvoke(GRE::MSG::ACTION_ChangeEnd);
			return;
			}

		//! Set specified flags for sample legend.
		void SetLegendSampleFlags (
			MGD::STYLESAMPLEFLAGS SampleFlags					//!< Flag bits to set
			) { v_SetLegendSampleFlags(SampleFlags); }

		//! Set position in list
		void SetListPos (
			GRE::LISTPOS listpos,						//!< Position
			LAYER *reflayer = 0							//!< Reference layer
			);

		void SetListPos (
			int newpos									//!< Position index, starting at 1
			);

		void SetModified (
			bool notify = true						//!< Set to invoke callbacks with "Changed" message
			);

		//! Set layer name to specified string.
		void SetName (
			const MISTRING& name
			);

		//! Set layer name to default based on RVC::OBJITEM.
		void SetName (
			const RVC::OBJITEM& objitem
			);

		//! Set whether group is notified (via SetModified) when changes made to layer.
		void SetNotifyGroup (
			bool notify = true
			) { m_NoSetModGroup = !notify; }

		//! Set descriptor for specified component.
		//! Updates in-memory descriptor only.  Does not send any change notifications.
		//! Should only be used after saving an editable object to permanent location.
		void SetObjectDescriptor (
			int component,
			const RVC::DESCRIPTOR& descriptor
			) { v_SetObjectDescriptor(component,descriptor); }

		//! Set the size of the layer in layer coordinates.
		//! This sets the size in one dimension and computes the size in the
		//! other. This is currently only implemented for setting the width of
		//! text layers, and only if IsProportionVariable() returns true.
		void SetRenderSize (
			DIMENSION dimension,
			double size
			) { v_SetRenderSize(dimension, size); }

		//! Set style object item for layer.
		ERRVALUE SetStyleObjItem (
			const RVC::OBJITEM& ObjItem				//!< Style object item to set
			) { return (v_SetStyleObjItem(ObjItem)); }

		//! Set transformation from layer to map.
		void SetTransMap (
			const TRANS2D_MAPGEN& trans,				//!< Transformation to set from
			bool fromoutput = false						//!< True to set from 'output' side of 'trans', 'false' from output side
			);

		//! Set unique layer ID.
		//! This method does not ensure that the ID is unique and is intended for internal use only.
		void SetUniqueID (
			UINT32 UniqueID
			) { m_UniqueID = UniqueID; }

		//! Set visibility in group from layout setting.
		void SetVisibilityGroupFromLayout (
			) { m_VisibleViewGroup = m_VisibleViewLayout; }

		//! Set layer visibility for group view(s).
		void SetVisibleGroup (
			UINT32 flags,
			bool set
			);

		//! Set layer visibility for layout view(s).
		void SetVisibleLayout (
			UINT32 flags,
			bool set
			);

		//! Set scale range over which layer is visible
		void SetVisibleScaleRange (
			double MinScaleVisible,
			double MaxScaleVisible
			);

		void SetVisibleView (
			GRE::VIEW *view,
			bool visible
			);

		//! Set layer visibility in viewable.
		void SetVisibleViewable (
			UINT32 flags,
			bool set
			);

		//! Set Z scale and offset values for 3D rendering.
		void SetZScaleOffset (
			double zscale,
			double zoffset
			);

		//! Snap point as specified.
		//! @return TRUE if moved, FALSE if not, < 0 if error.
		int SnapPoint (
			DPOINT3D& ptObject,							//!< Point to adjust passed/returned
			SNAPTO SnapTo,									//!< How to adjust
			double MaxDistance,							//!< Maximum distance in object coordinates
			VIEW *view = 0									//!< View to search drawn elements in, NULL to search all elements
			) const { return (v_SnapPoint(ptObject,SnapTo,MaxDistance,view)); }

		//! Determine if allowed to "snap" point as specified.
		bool SnapPointIsAllowed (
			SNAPTO SnapTo
			) const { return (v_SnapPointIsAllowed(SnapTo)); }

		//! Read from template serialization
		ERRVALUE TemplateRead (
			SERIALIZER& serializer
			) { return (v_TemplateRead(serializer)); }

		//! Test if element is inside/outside region
		//! @return TRUE if passes, FALSE if not, < 0 if error
		int TestElementByRegion (
			ELEMTYPE ElemType,							//!< Element type
			ELEMNUM ElemNum,								//!< Element number
			const REGION2D& Region,						//!< Region for testing
			TESTCRITERIA TestMode,						//!< Test to perform
			bool RegionInObjectCoords = true			//!< Indicates if region in object coordinates
			);

	#ifdef USE_MGUI
		//! Add or update details for layer in TreeView.
		void TreeViewAddUpdateDetails (
			GRE::CTRL_TREEVIEW& TreeView,
			MGUI::CTRL_TREE::ITEMHND hndParent,
			MGUI::CTRL_TREE::ITEMHND hndInsertAfter
			);
	#endif

	#ifdef USE_MGUI
		//! Add or update legend for layer in TreeView.
		MGUI::CTRL_TREE::ITEMHND TreeViewAddUpdateLegend (
			GRE::CTRL_TREEVIEW& TreeView,
			MGUI::CTRL_TREE::ITEMHND hndParent,
			MGUI::CTRL_TREE::ITEMHND hndInsertAfter
			);
	#endif

	#ifdef USE_MGUI
		//! Insert item for layer into TreeView.
		MGUI::CTRL_TREE::ITEMHND TreeViewInsertItem (
			GRE::CTRL_TREEVIEW& TreeView,					//!< Tree to insert into
			MGUI::CTRL_TREE::ITEMHND hndParent,			//!< Handle to parent item
			MGUI::CTRL_TREE::ITEMHND hndInsertAfter	//!< Handle to item to insert after
			);
	#endif

	#ifdef USE_MGUI
		//! Set state image in TreeView.
		void TreeViewSetStateImage (
			GRE::CTRL_TREEVIEW& TreeView,
			MGUI::CTRL_TREE::ITEMHND hndLayerItem = MGUI::CTRL_TREE::ITEMHND_None
			);
	#endif

	#ifdef USE_MGUI
		//! Update details for layer in all TreeViews.
		void TreeViewUpdateDetails (
			);
	#endif

		//! Unhighlight all elements for this layer
		void UnhighlightAllElements (
			bool redraw											//!< Set to redraw all views containing layer if needed
			);

		//! Update extents for layer.
		ERRVALUE UpdateExtents (
			bool forceupdate = false						//!< Force to recompute
			);

		//! Determine if layer uses transparency blending effects.
		//! If a DEVICE is specified, the layer may choose to disable some transparency
		//! effects.  For example, if the device resolution is high enough, there
		//! is no reason to do text smoothing, so we don't need transparency just
		//! for that.
		bool UsesTransparency (
			const MGD::DEVICE *pDevice = 0		//!< Optional device
			) const { return (v_UsesTransparency(pDevice)); }

		ERRVALUE ZoomViewToSelectedElements (
			GRE::VIEW *view,								//!< View to zoom, 0 for all
			bool activeonly = false
			);

	protected:

		//! Get serialization item definition array.
		//! @return item definition array.
		static const SERIALIZERITEM* SerialGetItemDef (SERIALIZER& serializer);

		static void SMLGET_DispParmMember (SML::CONTEXT*, SML::CLASSMEMBER*, void*, SML::ARG*);
		static void SMLGET_LayerDispFlag (SML::CONTEXT*, SML::CLASSMEMBER*, void*, SML::ARG*);
		static void SMLGET_LayerFlag (SML::CONTEXT*, SML::CLASSMEMBER*, void*, SML::ARG*);
		static void SMLGET_StyleObject (SML::CONTEXT*, SML::CLASSMEMBER*, void*, SML::ARG*);

		static void SMLSET_DispParmMember (SML::CONTEXT*, SML::CLASSMEMBER*, void*, SML::ARG*);
		static void SMLSET_DispParmMemberExtChg (SML::CONTEXT*, SML::CLASSMEMBER*, void*, SML::ARG*);
		static void SMLSET_LayerDispFlag (SML::CONTEXT*, SML::CLASSMEMBER*, void*, SML::ARG*);
		static void SMLSET_LayerFlag (SML::CONTEXT*, SML::CLASSMEMBER*, void*, SML::ARG*);
		static void SMLSET_StyleObject (SML::CONTEXT*, SML::CLASSMEMBER*, void*, SML::ARG*);
		static void SMLSET_OldProj (SML::CONTEXT*, SML::CLASSMEMBER*, void*, SML::ARG*);
		static void SMLSET_TextStyleMember (SML::CONTEXT*, SML::CLASSMEMBER*, void*, SML::ARG*);

		bool m_CanGetElemExtents;
		bool m_CanReportRawElemInfo;
		bool m_CanTestElementByRegion;
		bool m_HasExtents;
		bool m_LegendValid;
		bool m_DbListIsIndexElemNum;
		UINT8 m_CurPage;								//!< Current tab-page on control panel
		UINT32 layerflags;							//!< LAYERFLAG_xxxxx (NOT saved with layout)
		REGION2D m_ObjRegion;						//!< Bounding region in 'object' coordinates
		REGION2D m_MapRegion;						//!< Bounding region in 'map' coordinates
		TRANS2D_MAPGEN m_TransObjToMap;			//!< Transformation from object to map (using 'input' portion only)
		RVC::ELEMENT m_ActiveElem;
		int (*m_DrawFilter)(LAYERDC*, ELEMTYPE, INT32, void*);
		GraphicTreeItemMsgHandler m_LegendMsgHandler;	//!< Function to handle messages for legend interface

		//! Get pointer to table used for datatip
		RVC::DBTABLE* DataTipGetTable (
			int component								//!< Component index
			) const;

		//! Open table for use by datatip.
		ERRVALUE DataTipOpenTable (
			int component								//!< Component index
			);

		//! Reset all 'drawn' and 'highlighted' element sets for all views of this layer.
		void ResetAllElementSets (
			);

		//! Get serialization tag name.
		//! @return default tag name.
		const char* SerialGetTagName (
			) const { return (v_SerialGetTagName()); }

		//! Set whether can view in 2D.
		void SetCanViewIn2D (bool canview)
			{ m_CanViewIn2D = canview; }

		//! Set whether can view in 3D.
		void SetCanViewIn3D (bool canview)
			{ m_CanViewIn3D = canview; }

		//! Set whether can view in 2D overhead stereo mode.
		void SetCanViewStereo2D (bool canview)
			{ m_CanViewStereo2D = canview; }

		void SetLayerType (
			TYPE layertype
			) { m_layertype = layertype; }

	#ifdef USE_MGUI
		virtual void v_MenuAddElemTypeItems (MGUI::MENU& menu, ELEMTYPE ElemType);
		virtual void v_MenuAddLayerItems (MGUI::MENU& menu);
	#endif

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		static void InitStatic ();
		static ERRVALUE SerializerCB_MaxScaleVisible (SERIALIZER&, const SERIALIZERITEM *, void *, SERIALIZERITEM::ACTION);
		static ERRVALUE SerializerCB_MinScaleVisible (SERIALIZER&, const SERIALIZERITEM *, void *, SERIALIZERITEM::ACTION);
		static void SMLSET_ScaleRange (SML::CONTEXT*, SML::CLASSMEMBER*, void*, SML::ARG*);
		static void SMLFUNC_ShowDetails (SML::CONTEXT*, SML::CLASSMEMBER*, void*, SML::ARG*);

		static bool s_InitDone;
		static GLOBALOPTIONS s_GlobalOptions;
		static TYPEDEFN *s_TypeDefns[GRE_MAXLAYERTYPES];
		static UINT16 s_NumTypes;					// Number of layer types

		LAYER *m_prevlayer;							// Previous layer in group
		LAYER *m_nextlayer;							// Next layer in group
		GRE::GROUP *m_group;							// Pointer to group for this layer
		TYPE m_layertype;								// Layer type
		MISTRING m_Name;								// Layer name
		UINT32 m_drawflags;							// LAYERDRAWFLAG_xxxxx (saved with layout)
		UINT32 m_VisibleViewGroup;					// Bits for whether layer is visible in group views
		UINT32 m_VisibleViewLayout;				// Bits for whether layer is visible in layout views
		bool m_IsControlPanelDisabled;
		bool m_IsBeingCreated;
		bool m_NoSetModGroup;						// Don't set group as "modified" when changes made to layer
		bool m_CanViewIn2D;
		bool m_CanViewIn3D;
		bool m_CanViewStereo2D;
		CREATEFLAGS m_CreationFlags;				// Flags specified in constructor
		UINT8 m_MaskRed;								// Color masks used for specialized displays (eg. anaglyphs)
		UINT8 m_MaskGreen;
		UINT8 m_MaskBlue;
		UINT8 m_DataTipDftPage;
		bool m_DataTipEnabled;
		DISPPARM *m_pDispParm;						// Layer object display parameters, points to subclass member
		int (*m_RemoveCheckFunc)(LAYER*,void*);
		void *m_RemoveCheckData;
		UINT32 m_UniqueID;							// Unique layer ID
		UINT32 m_ModifyStamp;
		RVC::DBTABLE *m_pObjTableDataTip[5];
		DELEGATE_DATATIPGETCONTENT m_DelegateDataTipGetContent;

		ERRVALUE CheckUserRemove ();
		bool HasDenseDetail () const
			{ return (v_HasDenseDetail()); }
		void RestoreViewsFromBuffers (GRE::DRAWFLAGS drawflags, GRE::VIEW *view = 0);
	#ifdef USE_MGUI
		unsigned int TreeViewComputeStateIdx (GRE::VIEW*, bool skiplocator = false) const;
		void TreeViewOnChangePosition (GRE::CTRL_TREEVIEW& TreeView, MGUI::CTRL_TREE::ITEMHND hndLayerItem);
		bool TreeViewOnChildVisibilityChanging (GRE::CTRL_TREEVIEW& TreeView, MGUI::CTRL_TREE::ITEMHND hndLayerItem, MGUI::CTRL_TREE::CHILDVISIBILITY visibility);
		MGUI::CUSTOMDRAW::RETVALUE TreeViewOnCustomDraw (GRE::CTRL_TREEVIEW& TreeView, MGUI::CUSTOMDRAW_TREE& CustomDraw);
		void TreeViewOnGetItemInfo (GRE::CTRL_TREEVIEW&, MGUI::CTRL_TREE::ITEMVALUES, MGUI::CTRL_TREE::ITEMINFO&);
		bool TreeViewOnPointerButton (GRE::CTRL_TREEVIEW&, MGUI::CTRL_TREE::ITEMHND, MGUI::CTRL_TREE::HITRESULT, const LPOINT2D&, MGUI::POINTERBUTTON, MGUI::KEYSTATE);
		bool TreeViewOnToolTipRequest (GRE::CTRL_TREEVIEW&, MGUI::CTRL_TREE::ITEMHND, MGUI::CTRL_TREE::HITRESULT, const LPOINT2D&, MGUI::CTRL_TOOLTIP&);
		void TreeViewPopupMenu (GRE::CTRL_TREEVIEW&, const LPOINT2D& MousePoint);
	#endif
		void UpdateMapRegion ();

		// GRE::OBJECT overrides.
		virtual bool v_Destroy (bool NotifyParent);
		virtual MICONSTRUCT* v_GetIcon () const;
		virtual const char* v_GetTypeName () const;
		
		#endif // GENERATING_DOXYGEN_OUTPUT

		// LAYER overridables.

	#ifdef X_NATIVE
		virtual void v_AppendPopupMenuItems (Widget widget, const DPOINT2D *point, MENUITEMARRAY& menuitems);
	#endif
		virtual ERRVALUE v_AssignObject (const RVC::OBJITEM& objitem, MDLGPARENT dlgparent);
		virtual bool v_CanRender () const;
		virtual bool v_CanShowDetails () const;
		virtual void v_CloseObject ();
		virtual double v_ComputeDftMaxVisibleScale () const;
		virtual void v_ComputeMargins (double& tmargin, double& bmargin, double& lmargin, double& rmargin);
	#ifdef USE_MGUI
		virtual int v_ControlPanel (MDLGPARENT dlgparent, UINT32 flags = 0) = 0;
	#endif
		virtual GRE::LAYERSECTION* v_ConstructLayerSection (GRE::VIEW *view);
		virtual GRE::LAYERVIEW* v_ConstructLayerView (GRE::VIEW *view);
		virtual LAYER* v_Copy (GRE::GROUP *NewGroup) const = 0;
		virtual void v_DataTipGetDefaultPrefix (int component, DATATIPPARMS& DataTipParms) const;
		virtual void v_DataTipGetObjectNamesString (const DPOINT2D& point, MISTRING& string, int indent, GRE::VIEW *view) const;
		virtual const DATATIPPARMS* v_DataTipGetParms (int component) const;
		virtual int v_DataTipGetString (ELEMTYPE ElemType, ELEMNUM ElemNum, MISTRING& string, SPATMOD::IMAGE::STAGE_ARRAY& images);
		virtual int v_DataTipGetString (const DPOINT2D& point, MISTRING& string, SPATMOD::IMAGE::STAGE_ARRAY& images, GRE::VIEW* view);
		virtual bool v_DataTipHasAvailable () const;
		virtual bool v_DataTipHasEnabled () const;
		virtual void v_DataTipSetDefault (int component, bool force);
		virtual ERRVALUE v_DataTipSetParms (int component, const DATATIPPARMS& DataTipParms);
		virtual INT32 v_DbListElemNumToIndex (int component, ELEMNUM ElemNum);
		virtual ERRVALUE v_Draw (LAYERDC& layerdc) = 0;
		virtual ERRVALUE v_DrawElementLow (LAYERDC& layerdc, ELEMTYPE ElemType, ELEMNUM ElemNum, const void *style);
		virtual ERRVALUE v_DrawElementsLow (LAYERDC& layerdc, ELEMTYPE ElemType, const UNSIGNEDSET *pElemSet, const void *style);
		virtual ERRVALUE v_DrawElementSample (const GRE::ELEMSAMPLE_DC& sampledc, const RVC::ELEMENT& element, const LRECT2D& samplerect);
		virtual ERRVALUE v_DrawOverlay (LAYERDC& layerdc);
		virtual ERRVALUE v_EstimateRangeZ (const LAYERVIEW *pLayerView, DOUBLE_RANGE& range) const;
		virtual ELEMNUM v_FindClosestElement (const DPOINT2D& point, double searchdist, double *distret, int component, const UNSIGNEDSET *pSearchSet, GRE::FINDFLAGS findflags);
		virtual int v_GetComponent (ELEMTYPE ElemType) const;
		virtual MICONSTRUCT* v_GetComponentIcon (int component) const;
		virtual COMPINFO* v_GetComponentInfo (int component);
		virtual MISTRING v_GetComponentName (int component) const;
		virtual const void* v_GetElemDefaultStyle (ELEMTYPE ElemType) const;
		virtual UINT16 v_GetElemDrawSelectionMode (ELEMTYPE ElemType) const;
		virtual UINT16 v_GetElemDrawStyleMode (ELEMTYPE ElemType) const;
		virtual ERRVALUE v_GetElemExtents (ELEMTYPE ElemType,	ELEMNUM ElemNum, DRECT3D& Extents);
		virtual ERRVALUE v_GetElemSetByExtent (ELEMTYPE ElemType, const DRECT2D& Extent, UNSIGNEDSET& ElemSet);
		virtual ELEMTYPE v_GetElemType (int component) const;
		virtual bool v_GetGeorefName (RVC::OBJECTNAME& name, int component = 0) const;
		virtual bool v_GetLabelColor(LAYERDC& dc, const GRE::LABELPLACE& label, COLOR& color);
		virtual const GRE::LEGEND* v_GetLegend (ELEMTYPE ElemType);
		virtual INT64 v_GetNumElements (ELEMTYPE ElemType);
		virtual INT64 v_GetNumElementsDeleted (ELEMTYPE ElemType);
		virtual const RVC::OBJITEM& v_GetObjItem (int component) const;
		virtual double v_GetScaleMaxDetail (const DPOINT2D& point, const SPATREF::COORDREFSYS& crs, double DisplayPixelSizeMM) const;
		virtual ERRVALUE v_GetStyleInfo (ELEMTYPE ElemType, RVC::OBJITEM& ObjItem, RVC::OBJECTNAME& TableName);
		virtual bool v_HasDenseDetail () const;
		virtual bool v_HasDispParmSubObject () const;
		virtual bool v_HasHighlighted () const;
		virtual bool v_HasLegend () const;
		virtual bool v_HasNullImageCells () const;
		virtual int v_HighlightElementsByRegion (ELEMTYPE ElemType, const REGION2D& Region, TESTCRITERIA RegionTest, GRE::VIEW *firstview, GRE::DRAWFLAGS drawflags);
		virtual bool v_IsAntialiased () const;   
		virtual bool v_IsLineWidthHinted () const;
		virtual bool v_IsProportionVariable (DIMENSION dimension) const;
		virtual bool v_IsTableUsableForComponent (int component, const RVC::DBTABLE& table) const;
	#ifdef USE_MGUI
		virtual void v_MenuAddRegionGenItems (MGUI::MENU& menu, ELEMTYPE ElemType);
		virtual void v_MenuAddViewPopupItems (MGUI::MENU& menu, const DPOINT2D *point, VIEW *view);
	#endif

		virtual void v_OnNotifySurfaceChanged ();
		virtual int v_OpenObject (MDLGPARENT dlgparent);
	#ifdef USE_MGUI
		virtual ERRVALUE v_PromptObject (bool OpenCtrlPanel, MDLGPARENT dlgparent);
	#endif
		virtual ERRVALUE v_ReportRawElemInfo (MISTRING& infostr, ELEMTYPE ElemType, ELEMNUM ElemNum);
		virtual ERRVALUE v_SaveDispParmSubObject (RVC::DISPPARMUSAGE usage);
		virtual const char* v_SerialGetTagName () const = 0;
		virtual ERRVALUE v_SerialRead (SERIALIZER& serializer);
		virtual ERRVALUE v_SerialWrite (SERIALIZER& serializer, const char *tagname) const;
		virtual void v_SetAntialiased (bool antialias);
		virtual void v_SetDftName (bool ReplaceExisting);
		virtual void v_SetElemDrawSelectionMode (ELEMTYPE ElemType, UINT16 SelectMode);
		virtual void v_SetElemDrawStyleMode (ELEMTYPE ElemType, UINT16 StyleMode);
		virtual void v_SetHintThinLines (bool hint);
		virtual void v_SetLegendSampleFlags (MGD::STYLESAMPLEFLAGS SampleFlags);
		virtual void v_SetObjectDescriptor (int component, const RVC::DESCRIPTOR& descriptor);
		virtual void v_SetRenderSize (DIMENSION dimension, double size);
		virtual ERRVALUE v_SetStyleObjItem (const RVC::OBJITEM& ObjItem);
		virtual int v_SnapPoint (DPOINT3D& ptObject, SNAPTO SnapTo, double MaxDistance, VIEW *view) const;
		virtual bool v_SnapPointIsAllowed (SNAPTO SnapTo) const;
		virtual ERRVALUE v_TemplateRead (SERIALIZER& serializer);
		virtual int v_TestElementByRegion (ELEMTYPE ElemType, ELEMNUM ElemNum, const REGION2D& Region, TESTCRITERIA TestMode);
	#ifdef USE_MGUI
		virtual MGUI::CTRL_TREE::ITEMHND v_TreeViewAddUpdateLegend (GRE::CTRL_TREEVIEW& TreeView, MGUI::CTRL_TREE::ITEMHND hndParent, MGUI::CTRL_TREE::ITEMHND hndInsertAfter);
		virtual void v_TreeViewAddUpdateDetails (GRE::CTRL_TREEVIEW& TreeView, MGUI::CTRL_TREE::ITEMHND hndParent, MGUI::CTRL_TREE::ITEMHND hndInsertAfter);
	#endif
		virtual int v_UpdateExtents (bool forceupdate);
		virtual bool v_UsesTransparency (const MGD::DEVICE *pDevice = 0) const;


		#ifndef GENERATING_DOXYGEN_OUTPUT

		LAYER (const LAYER&);
		LAYER& operator= (const LAYER&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};	// End of LAYER

#ifndef	GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(LAYER::CREATEFLAGS)
DEFINE_ENUM_OP_BITWISE(LAYER::SETACTIVEFLAGS)
DEFINE_ENUM_OP_INCREMENT(LAYER::TYPE)
#endif

//===================================================================================================================

//! Shared display parameters common to most layer types
class LAYER::DISPPARM : public SERIALIZABLE {
	public:

		UINT32 dispflags;							//!< Display flags - DISPFLAG_xxxx...
		UINT8 NumObjects;							//!< Number of objects
		UINT8 NumComponents;						//!< Number of components
		UINT8 NoGeoreference;					//!< Don't use georeference subobject
		UINT8 DataTipSearchDist;				//!< DataTip search distance in pixels
		double zinit;								//!< Minimum Z value for surface or layer (unscaled) (XXX move to layer
		double zlast;								//!< Maximum Z value for surface or layer (unscaled) (XXX move to layer
		double zscale;								//!< Scaling for surface or Z coordinate
		double zoffset;							//!< Offset for surface or above surface if layer
		COLOR ActiveColor;						//!< Active element color
		COLOR HighlightColor;					//!< Highlight color
		COLOR WireframeColor;					//!< Wireframe color in non-stereo mode
		COLOR BaseColor;							//!< Pedestal base color in 3D
		double MinScaleVisible;					//!< Minimum scale at which this layer is visible
		double MaxScaleVisible;					//!< Maximum scale at which this layer is visible
		double m_MaxScaleRegion;				//!< Maximum scale for showing region if zoomed out past MaxScaleVisible
		POLYSTYLE m_RegionStyle;				//!< Style for region
		bool m_ShowRegionZoomedOut;			//!< Show region if zoomed out past MaxScaleVisible
		bool m_bManageTextLabels;				//!< If true, use LABELPLACEMENT for text labels
		bool m_bManagePointSymbols;			//!< If true, use LABELPLACEMENT for point symbols

		//! Assignment operator.
		// This should be 'protected' but compiler doesn't like it.
		DISPPARM& operator= (const DISPPARM& rhs) {
			if (this != &rhs) Copy(rhs);
			return (*this);
			}

		//! Load dispparm via SerialRead from subobject.
		//! @return TRUE if loaded, FALSE if no subobject, < 0 if error.
		int LoadSubObject (
			const RVC::OBJECT& ParentObj,					//!< Parent 'object'
			RVC::DISPPARMUSAGE usage = RVC::DISPPARMUSAGE_Unspecified	//!< Usage, unspecified to determine automatically
			);

		//! Set transparency usage to "indeterminant".
		void SetMayUseTransparency (
			) { dispflags |= DISPFLAG_MayUseTransparency; dispflags &= ~DISPFLAG_UsesTransparency; }

		//! Validate dispparm, adjusting if possible.
		ERRVALUE Validate ();

		//! Is 3D pedestal set on / off.
		bool IsPedestalSet (
			) { return (BaseColor.GetTransparency() != 255); }

		//! Set 3D pedestal on / off.
		void SetPedestal (
			const bool pedestal
			) { BaseColor.SetTransparency(pedestal ? 0 : 255 );  }

	protected:

		//! Default constructor.
		DISPPARM ();

		//! Copy constructor.
		DISPPARM (const DISPPARM& rhs) : SERIALIZABLE(rhs) { Copy(rhs); }

		virtual ~DISPPARM ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		void Copy (const DISPPARM& rhs);

		//! Overrides from SERIALIZABLE.
		virtual ERRVALUE SerialRead (SERIALIZER& serializer);
		virtual ERRVALUE SerialWrite (SERIALIZER& serializer, const char *tagname = 0) const;

		#endif // GENERATING_DOXYGEN_OUTPUT

		//! LAYER::DISPPARM new overridables.
		virtual const SERIALIZERITEM* SerialGetItemDef (SERIALIZER& serializer) const = 0;
		virtual ERRVALUE v_SetObjItem (const RVC::OBJITEM& objitem, bool replace);
		virtual ERRVALUE v_Validate ();

	};	// End of LAYER::DISPPARM

//===================================================================================================================

//! Base class for shared layer display parameters.
//! Generally accessed through GRE::GROUP methods.
class LAYER::COMMONPARM {
	public:

		enum STATE {
			STATE_False =		0,
			STATE_True =		1,
			STATE_ByLayer =	2
			};

	#ifdef USE_MGUI
		//! Base class for user interface form.
		class FORM {
			public:

				FORM ();

				virtual ~FORM () = 0;

				//! Create form.
				void Create (
					MGUI::LAYOUT_PANE& ParentPane,
					const COMMONPARM *pValues		//! Initial values
					) { v_Create(ParentPane,pValues); }

				//! Get current values
				void GetValues (
					COMMONPARM *pValues				//! Container for result
					) { v_GetValues(pValues); }

				void SetDelegateOnChanged (
					DELEGATE_VOID_NOPARMS delegate
					) { m_DelegateOnChanged = delegate; }

				//! Set values.
				void SetValues (
					const COMMONPARM *pValues		//! Values to set
					) { v_SetValues(pValues); }

			protected:

				//! Called by derived class to notify of changes.
				void NotifyChanged (
					);

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT

				DELEGATE_VOID_NOPARMS m_DelegateOnChanged;

				#endif

				// Overridables.

				//! Create form.
				virtual void v_Create (
					MGUI::LAYOUT_PANE& ParentPane,
					const COMMONPARM *pValues		//! Initial values
					) = 0;

				//! Get current values
				virtual void v_GetValues (
					COMMONPARM *pValues				//! Container for result
					) = 0;

				//! Set values.
				virtual void v_SetValues (
					const COMMONPARM *pValues		//! Values to set
					) = 0;

			};
	#endif

		virtual ~COMMONPARM () = 0;

	#ifdef USE_MGUI
		//! Construct user interface form.
		FORM* ConstructForm (
			) const { return (v_ConstructForm()); }
	#endif

	#ifdef USE_MGUI
		//! Open modal dialog to edit values.
		//! @return true if changed, false if not
		bool DlgEditValues (
			MDLGPARENT dlgparent
			) { return (v_DlgEditValues(dlgparent)); }
	#endif

		//! Get group attached to.
		GROUP* GetGroup (
			) const { return (m_group); }

		//! Get layer type parameters apply to.
		LAYER::TYPE GetLayerType (
			) const { return (m_LayerType); }

		//! Determine if has user interface methods.
		bool HasUI (
			) const { return (m_HasUI); }

		//! Get serialization item definition
		const SERIALIZERITEM* SerialGetItemDef (
			) const { return (v_SerialGetItemDef()); }

		//! Get tag name for serialization.
		const char* SerialGetTagName (
			) const { return (v_SerialGetTagName()); }

		//! Read from serialization.
		ERRVALUE SerialRead (
			SERIALIZER& serializer
			) { return (v_SerialRead(serializer)); }

		//! Write to serialization.
		ERRVALUE SerialWrite (
			SERIALIZER& serializer
			) const { return (v_SerialWrite(serializer)); }

	protected:

		//! Constructor.
		explicit COMMONPARM (
			GROUP *group,
			LAYER::TYPE layertype,
			bool HasUI = true
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		GROUP *m_group;
		LAYER::TYPE m_LayerType;
		bool m_HasUI;

		// Unimplemented
		COMMONPARM (const COMMONPARM& rhs);
		COMMONPARM& operator= (const COMMONPARM& rhs);

		#endif // GENERATING_DOXYGEN_OUTPUT

		// Overridables.

	#ifdef USE_MGUI
		//! Construct user interface form.
		virtual FORM* v_ConstructForm () const;
	#endif

	#ifdef USE_MGUI
		//! Open modal dialog to edit values.
		//! @return true if changed, false if not
		//! Default implementation creates simple dialog with ConstructForm used for contents.
		virtual bool v_DlgEditValues (MDLGPARENT dlgparent);
	#endif

		//! Get serialization item definition.
		virtual const SERIALIZERITEM* v_SerialGetItemDef () const = 0;

		//! Get serialization tag name.
		virtual const char* v_SerialGetTagName () const = 0;

		//! Read from serialization.
		//! Default implementation reads based on item definition array.
		virtual ERRVALUE v_SerialRead (SERIALIZER& serializer);

		//! Write to serialization.
		//! Default implementation writes using item definition array and tag name returned by SerialGet... methods.
		virtual ERRVALUE v_SerialWrite (SERIALIZER& serializer) const;

	};	// End of LAYER::COMMONPARM

//===================================================================================================================

//! Layer type definition structure.
class LAYER::TYPEDEFN {
	public:

		enum FLAGS {
			FLAG_None =							0,
			FLAG_NotEditable =				0x0001,	//!< Layer is not editable
			FLAG_OnePerGroup =				0x0002,	//!< Only one layer of this type per group
			FLAG_NoHighlightColors =		0x0010,	//!< No "Highlight" or "Active" color for layer
			FLAG_CanQuickAdd =				0x0020,	//!< Can do "quick-add" for this layer type
			FLAG_CanShowDataTip =			0x0040,	//!< Can show DataTips for layer
			FLAG_IsRemote =					0x0080,	//!< Layer content delivered by remote service
			FLAG_CanShowRegionZoomedOut =	0x0100,	//!< Can display region when zoomed out
			FLAG_HasPossibleOverlay =		0x0200,	//!< Layer has possible overlay
			};

		//! Add file/object types and delegates to filter.
		void AddTypesToFilter (
			MGUI::DLG_GETOBJECT_FILTER& filter
			) const;

		bool CanQuickAdd (
			) const { return ((m_flags & FLAG_CanQuickAdd)!=0); }

		bool CanShowDataTip (
			) const { return ((m_flags & FLAG_CanShowDataTip)!=0); }

		bool CanShowRegionZoomedOut (
			) const { return ((m_flags & FLAG_CanShowRegionZoomedOut)!=0); }

	#ifdef USE_MGUI
		//! Create MGUI::MENU for add-layer options.
		//! @return true if has menu, false if not.
		//! Only layers which have multiple choices for creating layer will typically have a menu.
		bool CreateMenuAdd (
			MGUI::MENU& menuret								//!< Menu created and returned
			) const { return (v_CreateMenuAdd(menuret)); }
	#endif

		//! Construct layer for this type.
		ERRVALUE Construct (
			GRE::GROUP *group,									//!< Group to contain new layer
			LAYER **retlayer = 0,								//!< New layer returned
			CREATEFLAGS createflags = CREATEFLAG_None,	//!< Creation flags
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,		//!< Position of new layer in list
			LAYER *reflayer = 0									//!< Reference layer for insertion
			) const;

		//! Get file extensions.
		//! @return NULL if no extensions supported by layer type.
		const char* GetFileExtns (
			) const { return (m_FileExtns); }

		//! Get icon ID.
		ICONID GetIconID (
			) const { return (m_IconID); }

		//! Get layer type.
		TYPE GetLayerType (
			) const { return (m_LayerType); }

	#if defined(X_NATIVE) && !defined(DEPRECATE_ALL)
		//! Get MENUITEM array for layer creation menu choices (X only).
		//! \deprecated, replaced by CreateMenuAdd.
		DEPRECATED MENUITEM* GetMenuItemCreate (
			) const { return (v_GetMenuItemCreate()); }
	#endif

		//! Get set of valid object types for this layer.
		const RVC::OBJTYPESET& GetObjTypeSet (
			) const { return (m_ObjTypeSet); }

		//! Get serializer tag for common parameters.
		const char* GetSerializeTagCommonParm (
			) const { return (m_SerializeTagCommonParm); }

		//! Get serializer tag for layer.
		const char* GetSerializeTagLayer (
			) const { return (m_SerializeTagLayer); }

		//! Get serializer tag for template.
		const char* GetSerializeTagTemplate (
			) const { return (m_SerializeTagTemplate); }

		//! Get image index for layer in object tree.
		int GetTreeViewImageIdx (
			) const { return (m_TreeViewImageIdx); }

		//! Get image index for editable layer in object tree.
		int GetTreeViewImageIdxEdit (
			) const { return (m_TreeViewImageIdxEdit); }

		TEXTID GetTypeNameTextID (
			) const { return (m_TypeNameTextID); }

		bool HasPossibleOverlay () const
			{ return ((m_flags & FLAG_HasPossibleOverlay) != 0); }

		bool IsEditable (
			) const { return (!(m_flags & FLAG_NotEditable)); }

		//! Determine if file is selectable for constructing layer of this type.
		//! @return TRUE if selectable, FALSE if not, < 0 if error.
		int IsFileSelectable (
			const FILEPATH& filepath,
			MISTRING& msg,
			int position = -1
			) const;

		//! Determine if object is selectable for constructing layer of this type.
		//! @return TRUE if selectable, FALSE if not, < 0 if error.
		int IsObjectSelectable (
			const RVC::OBJITEM& objitem,
			MISTRING& msg,
			int position = -1
			) const;

		bool IsOnePerGroup (
			) const { return ((m_flags & FLAG_OnePerGroup)!=0); }

		bool IsRemote (
			) const { return ((m_flags & FLAG_IsRemote)!=0); }

		void SetupTreeViewShared (
			);

		bool UsesHighlightColors (
			) const { return (!(m_flags & FLAG_NoHighlightColors)); }

	protected:

		//! Construct type that does not use file/object.
		TYPEDEFN (
			TYPE LayerType,
			TEXTID TypeNameTextID,
			const char *SerializeTagLayer,
			const char *SerializeTagTemplate,
			ICONID IconID,
			FLAGS flags,
			const char *SerializeTagCommonParm = 0
			);

		//! Construct with RVC::OBJTYPESET.			
		TYPEDEFN (
			TYPE LayerType,
			TEXTID TypeNameTextID,
			const RVC::OBJTYPESET& ObjTypeSet,
			const char *SerializeTagLayer,
			const char *SerializeTagTemplate,
			ICONID IconID,
			FLAGS flags,
			const char *SerializeTagCommonParm = 0
			);

		//! Construct with file extensions.
		TYPEDEFN (
			TYPE LayerType,
			TEXTID TypeNameTextID,
			const char *FileExtns,
			const char *SerializeTagLayer,
			const char *SerializeTagTemplate,
			ICONID IconID,
			FLAGS flags,
			const char *SerializeTagCommonParm = 0
			);

		//! Construct with file extensions and RVC::OBJTYPESET.
		TYPEDEFN (
			TYPE LayerType,
			TEXTID TypeNameTextID,
			const RVC::OBJTYPESET& ObjTypeSet,
			const char *FileExtns,
			const char *SerializeTagLayer,
			const char *SerializeTagTemplate,
			ICONID IconID,
			FLAGS flags,
			const char *SerializeTagCommonParm = 0
			);

		virtual ~TYPEDEFN () = 0;

		void SetIconIdEdit (
			ICONID iconid
			) { m_IconIdEdit = iconid; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		TYPE m_LayerType;								//!< Layer type, 0 for automatic assignment
		TEXTID m_TypeNameTextID;
		FLAGS m_flags;
		RVC::OBJTYPESET m_ObjTypeSet;
		const char *m_FileExtns;
		const char *m_SerializeTagLayer;
		const char *m_SerializeTagTemplate;
		const char *m_SerializeTagCommonParm;
		ICONID m_IconID;
		int m_TreeViewImageIdx;
		ICONID m_IconIdEdit;
		int m_TreeViewImageIdxEdit;

		virtual void v_Construct (
			GRE::GROUP *group,					//!< Group to contain new layer
			LAYER **retlayer,			//!< New layer returned
			CREATEFLAGS createflags,		//!< Creation flags
			GRE::LISTPOS listpos,			//!< Position of new layer in list
			LAYER *reflayer				//!< Reference layer for insertion
			) const = 0;

	#ifdef USE_MGUI
		//! Create MGUI::MENU for add-layer options.
		//! Default implementation returns false
		virtual bool v_CreateMenuAdd (
			MGUI::MENU& menuret								//!< Menu created and returned
			) const;
	#endif

	#ifdef X_NATIVE
		virtual MENUITEM* v_GetMenuItemCreate () const;
	#endif

		//! Determine if file is selectable for constructing layer of this type.
		//! @return TRUE if selectable, FALSE if not, < 0 if error.
		//! Default implemention returns FALSE.
		virtual int v_IsFileSelectable (
			const FILEPATH& filepath,
			MISTRING& msg,
			int position
			) const;

		//! Determine if object is selectable for constructing layer of this type.
		//! @return TRUE if selectable, FALSE if not, < 0 if error.
		//! Object type will be checked in IsObjectSelectable before calling.
		//! Default implemention returns TRUE.
		virtual int v_IsObjectSelectable (
			const RVC::OBJITEM& objitem,
			MISTRING& msg,
			int position
			) const;

	#ifdef USE_MGUI
		//! Setup shared items for use in object tree.
		virtual void v_SetupTreeViewShared ();
	#endif

		#endif // GENERATING_DOXYGEN_OUTPUT
	};	// End of LAYER::TYPEDEFN
#ifndef	GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(LAYER::TYPEDEFN::FLAGS);
#endif

//===================================================================================================================

//! Layer component information.
class LAYER::COMPINFO {
	public:

	#if defined(USE_MGUI)
		class TABLEINFO {
			public:

				enum TYPE {
					TYPE_None =			0,
					TYPE_User =			1,
					TYPE_Form =			2,
					TYPE_System =	3
					};

				//! Constructor.
				TABLEINFO (
					COMPINFO& CompInfo
					);

				//! Copy constructor.
				TABLEINFO (
					const TABLEINFO& rhs
					);

				//! Destructor.
				~TABLEINFO (
					);

				ERRVALUE HideTable (
					);

				bool IsForm (
					) const { return (m_Type == TYPE_Form); }

				bool IsSystemTable (
					) const { return (m_Type == TYPE_System); }

				bool IsOpen (
					) const;

				bool IsUser (
					) const { return (m_Type == TYPE_User); }

				const RVC::OBJITEM* GetObjItemForm (
					) const { return (m_pObjItemForm); }

				bool SetForm (
					const RVC::OBJITEM& ObjItemForm
					);

				bool SetTable (
					INT32 TableNum
					);

				ERRVALUE ShowTable (
					);

				//! Add item for this table to TreeView.
				MGUI::CTRL_TREE::ITEMHND TreeViewAddItem (
					GRE::CTRL_TREEVIEW& TreeView,
					MGUI::CTRL_TREE::ITEMHND hndParent,
					MGUI::CTRL_TREE::ITEMHND hndInsertAfter
					);

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT

				COMPINFO& m_CompInfo;
				TYPE m_Type;
				INT32 m_TableNum;					//!< Table number
				RVC::OBJITEM *m_pObjItemForm;	//!< Form object item
				bool m_IsTableViewable;

				bool TreeViewOnPointerButton (GRE::CTRL_TREEVIEW&, MGUI::CTRL_TREE::ITEMHND, MGUI::CTRL_TREE::HITRESULT, const LPOINT2D&, MGUI::POINTERBUTTON, MGUI::KEYSTATE);
				bool TreeViewOnToolTipRequest (GRE::CTRL_TREEVIEW&, MGUI::CTRL_TREE::ITEMHND, MGUI::CTRL_TREE::HITRESULT, const LPOINT2D&, MGUI::CTRL_TOOLTIP&);
				void TreeViewPopupMenu (GRE::CTRL_TREEVIEW&, MGUI::CTRL_TREE::ITEMHND, const LPOINT2D& MousePoint);

				TABLEINFO& operator= (const TABLEINFO& rhs);	// Not implemented

				friend class COMPINFO;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};	// End of LAYER::COMPINFO::TABLEINFO

		typedef MILIST<TABLEINFO> TABLEINFOLIST;

	#endif

		class DLG_RECLASSIFY;

		//! Default constructor for use in arrays.
		COMPINFO (
			);

		//! Standard constructor.
		COMPINFO (
			LAYER *layer,
			int component,
			ELEMTYPE ElemType,
			bool AllowDatabase = true,
			bool AllowQuery = true
			);

		//! Destructor.
		~COMPINFO (
			);

	#ifdef USE_MGUI
		//! Make "next" highlighted element "active"
		void ActivateNextSelected (
			);
	#endif

	#ifdef USE_MGUI
		//! Make "previous" highlighted element "active"
		void ActivatePrevSelected (
			);
	#endif

	#ifdef USE_MGUI
		//! (Internal)
		ERRVALUE ApplyReclassify (DLG_RECLASSIFY*);
	#endif

	#ifdef USE_MGUI
		//! Create DBEDIT instance for component.
		ERRVALUE CreateDbEdit (
			);
	#endif

		//! Destroy component forms, dialogs, table lists, etc.
		void Destroy (
			);

	#ifdef USE_MGUI
		//! Dialog to mark elements by query.
		ERRVALUE DlgHighlightByQuery ();
	#endif

	#ifdef USE_MGUI
		//! Remove specified "form" from database.
		void DropForm (
			const RVC::OBJITEM& ObjItemForm
			);
	#endif

	#ifdef USE_MGUI
		//! Drop (remove) specified table from database.
		void DropTable (
			INT32 TableNum								//!< Table number
			);
	#endif

	#ifdef USE_MGUI
		//! Find TABLEINFO associated with "form"
		TABLEINFO* FindFormInfo (
			const RVC::OBJITEM& ObjItemForm
			);
	#endif

	#ifdef USE_MGUI
		//! Find TABLEINFO associated with specified table number.
		TABLEINFO* FindTableInfo (
			INT32 TableNum								//!< Table number
			);
	#endif

		//! Get active element number for component.
		ELEMNUM GetActiveElemNum (
			) const { return (m_ActiveElemNum); }

		//! Get whether allows query for selection/marking.
		bool GetAllowQuery () const
			{ return (m_AllowQuery); }

		//! Get "base" table number for component.
		INT32 GetBaseTableNum (
			) const { return (m_BaseTableNum); }

		//! Get component index.
		int GetComponent (
			) const { return (m_component); }

	#ifdef USE_MGUI
		//! Get DBEDIT pointer for component.
		DBEDIT* GetDbEdit (
			);
	#endif

		//! Get element type.
		ELEMTYPE GetElemType (
			) const { return (m_ElemType); }

		//! Get highlight set (const).
		const UNSIGNEDSET& GetHighlightSet (
			) const { return (m_HighlightSet); }

		//! Get highlight set (non-const).
		UNSIGNEDSET& GetHighlightSet (
			) { return (m_HighlightSet); }

		//! Get layer for component.
		LAYER* GetLayer (
			) const { return (m_layer); }

		bool GetShowSystemTables (
			) const { return (m_ShowSystemTables); }

	#ifdef USE_MGUI
		TABLEINFOLIST& GetTableInfoList (
			) { return (m_TableInfoList); }
	#endif

	#ifdef USE_MGUI
		//! Determine if has 'database editor' for component.
		bool HasDbEdit (
			) const { return (m_pDbEdit != 0); }
	#endif

		void Init (
			LAYER *layer,
			int component,
			ELEMTYPE ElemType,
			bool AllowDatabase,
			bool AllowQuery = true
			);

		bool IsElemSelectable (
			) const { return (m_ElemSelectable); }

		//! Notify that elements have been selected.
		void NotifyElementSelected (
			);

	#ifdef USE_MGUI
		void OnMenuRegionGen (MGUI::ID, bool);
	#endif

	#ifdef USE_MGUI
		//! Open dialog to reclassify values in specified table.
		//! Table must have attachment of Implied or OneToOne.
		ERRVALUE OpenDlgReclassify (
			INT32 TableNum,							//!< Table number to reclassify
			INT32 FieldNum,							//!< Field to relclassify
			INT32 RelFieldDisplay = -1,			//!< Field number in related picklist table to display values from
			INT32 RelFieldSortBy = -1				//!< Field number in related picklist table to sort by
			);
	#endif

	#ifdef USE_MGUI
		//! Select elements with given association to table
		void SelectTableElements (
			MDLGPARENT dlgparent,
			INT32 TableNum,							//!< Table number
			int mode							//!< XXX change to enum
			);
	#endif

		//! Set active element number for component.
		void SetActiveElemNum (
			ELEMNUM ElemNum
			) { m_ActiveElemNum = ElemNum; }

		//! Set whether can auto-open default table on selection.
		void SetAllowShowDefaultTable (
			bool AllowShowDefaultTable
			) { m_AllowShowDefaultTable = AllowShowDefaultTable; }

		//! Set "base" table number for component.
		void SetBaseTableNum (
			INT32 BaseTableNum
			) { m_BaseTableNum = BaseTableNum; }

	#ifdef USE_MGUI
		void SetDbEditStyleObject (
			);
	#endif

		void SetElemSelectable (
			bool ElemSelectable
			);

		void SetElemType (
			ELEMTYPE ElemType
			) { m_ElemType = ElemType; }

	#ifdef USE_MGUI
		void ShowDefaultTable (
			);
	#endif

	#ifdef USE_MGUI
		//! Add item for component to TreeView using default text.
		MGUI::CTRL_TREE::ITEMHND TreeViewAddUpdateItem (
			GRE::CTRL_TREEVIEW& TreeView,
			MGUI::CTRL_TREE::ITEMHND hndParent,
			MGUI::CTRL_TREE::ITEMHND hndInsertAfter,
			int ImageIdx
			);
	#endif

	#ifdef USE_MGUI
		//! Add item for component to TreeView using specified text.
		MGUI::CTRL_TREE::ITEMHND TreeViewAddUpdateItem (
			GRE::CTRL_TREEVIEW& TreeView,
			MGUI::CTRL_TREE::ITEMHND hndParent,
			MGUI::CTRL_TREE::ITEMHND hndInsertAfter,
			const MISTRING& label,
			int ImageIdx
			);
	#endif

	#ifdef USE_MGUI
		//! Add items for attribute tables to TreeView.
		void TreeViewAddTables (
			GRE::CTRL_TREEVIEW& TreeView,
			MGUI::CTRL_TREE::ITEMHND hndParent,
			MGUI::CTRL_TREE::ITEMHND hndInsertAfter
			);
	#endif

	#ifdef USE_MGUI
		//! Delete item for component from TreeView.
		void TreeViewDeleteItem (
			GRE::CTRL_TREEVIEW& TreeView
			);
	#endif

	#ifdef USE_MGUI
		void TreeViewShowTableList (
			bool show
			);
	#endif

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		enum SELGROUP {
			SELGROUP_Select,
			SELGROUP_Clear,
			SELGROUP_Invert
			};

		class DBOBSERVER;
		friend class COMPINFO::DBOBSERVER;

		LAYER *m_layer;								// Layer this applies to
		int m_component;								// Component index in layer
		ELEMTYPE m_ElemType;							// Component element type
		UNSIGNEDSET m_HighlightSet;
		bool m_AllowDatabase;						// Element type supports database views
		bool m_AllowQuery;							// Element type supports selection by query
		bool m_AllowShowDefaultTable;				// Can automatically show default table
	#if defined(USE_MGUI)
		DBEDIT *m_pDbEdit;							// Database 'editor'
		RVC::DBASE_ELEMENT *m_pObjDbElem;		// Database for element
		DBOBSERVER *m_pObsDbElem;					// Observer on the database
		TABLEINFOLIST m_TableInfoList;			// Information about each table
		MGUI::DLGSHELL *m_pDlgMarkByQuery;		// Dialog for 'mark by query'
		SIMPLE_ARRAY<DLG_RECLASSIFY*> m_DlgReclassifyPtrs;
	#endif
		ELEMNUM m_ActiveElemNum;					// Active element for this component
		bool m_ElemSelectable;						// Element is selectable
		bool m_ShowSystemTables;					// Show internal tables in list
		INT32 m_BaseTableNum;						// Base table for component

	#ifdef X_NATIVE
		MXDHANDLE m_SelQueryDlgHandle;			// Handle for select-by-query dialog
	#endif

	#ifdef USE_MGUI
		static void CB_NotifyDbEdit (const DBEDIT_MSG *cbs, void *vpCompInfo);
		TABLEINFO* AddTableInfo (const TABLEINFO& NewTableInfo);
		bool HasNonSystemTables ();
		ERRVALUE MarkAttachedElements (const DBEDIT_MSG *msg);
		void OnCmdAddTable ();
		void OnCmdEditRelations ();
		void OnCmdLinkToDataSource ();
		void OnCmdMakeForm ();
		void OnCmdMakeTable ();
		void OnNotifyDbEdit (const DBEDIT_MSG *cbs);
		void OnTableFormRemoved (TABLEINFO*);
		void OnTableNameChanged (INT32 TableNum);
		void ShowSelectedRecords (INT32 TableNum, bool force=false);
		bool TreeViewOnChildVisibilityChanging (GRE::CTRL_TREEVIEW& TreeView, MGUI::CTRL_TREE::ITEMHND hndCompItem, MGUI::CTRL_TREE::CHILDVISIBILITY visibility);
		bool TreeViewOnPointerButton (GRE::CTRL_TREEVIEW&, MGUI::CTRL_TREE::ITEMHND, MGUI::CTRL_TREE::HITRESULT, const LPOINT2D&, MGUI::POINTERBUTTON, MGUI::KEYSTATE);
		bool TreeViewOnToolTipRequest (GRE::CTRL_TREEVIEW&, MGUI::CTRL_TREE::ITEMHND, MGUI::CTRL_TREE::HITRESULT, const LPOINT2D&, MGUI::CTRL_TOOLTIP&);
		void TreeViewPopupMenu (GRE::CTRL_TREEVIEW& TreeView, MGUI::CTRL_TREE::ITEMHND hndItem, const LPOINT2D& MousePoint);
	#endif

		COMPINFO (const COMPINFO&);
		COMPINFO& operator= (const COMPINFO&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};	// End of LAYER::COMPINFO


//! Macro for definition of sml functions and class members.
#define SMLMEMBER_LAYERDISPFLAG(name,invert,flag,desc) {name, STYPE_Num, LAYER::SMLGET_LayerDispFlag, LAYER::SMLSET_LayerDispFlag, 0, 0, invert, flag, 0,0,0,0, desc}


//===================================================================================================================

//! Filter for whether to include layer in an operation.
class LAYER::FILTER {
	public:
		//! Test specified layer against filter.
		bool Filter (
			const LAYER *layer
			) { return (v_Filter(layer)); }
	private:
		//! Overridable method to determine whether to include the layer or not.
		//! @return true to include, false to exclude.
		virtual bool v_Filter (
			const LAYER *layer
			) = 0;
	};

//===================================================================================================================

//! Convenience filter for excluding a single specific layer from an operation.
class LAYER::FILTER_EXCLUDE : public LAYER::FILTER {
	public:

		//! Constructor.
		FILTER_EXCLUDE (
			const LAYER *exlayer			//!< Layer to exclude from operation
			) : m_exlayer(exlayer) { }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		const LAYER *m_exlayer;
		virtual bool v_Filter (const LAYER *layer);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================
}	// End namespace GRE

#endif	// INC_GRE_LAYER_H
