/**
 * \file gre/layout.h
 * \brief GRE::LAYOUT definitions
 *
 * \if NODOC
 * $Id: layout.h_v 1.47 2011/10/20 12:45:52 mju Exp $
 *
 * $Log: layout.h_v $
 * Revision 1.47  2011/10/20 12:45:52  mju
 * Remove redundant hdrs.
 *
 * Revision 1.46  2011/03/01 22:43:00  bcolombe
 * Add SavedLocations
 *
 * Revision 1.45  2009/09/23 21:54:31  mju
 * Add isSortLegendNatural.
 *
 * Revision 1.44  2008/01/25 15:33:53  mju
 * Add v_clear.
 *
 * Revision 1.43  2007/11/26 18:43:41  mju
 * Add v_getActiveLayer, v_unhighlightAllElements.
 *
 * Revision 1.42  2007/10/04 21:21:24  mju
 * Exclude macro from docs.
 *
 * Revision 1.41  2007/08/03 14:17:41  dwilliss
 * Added static protected method to get sml class members
 *
 * Revision 1.40  2006/10/10 13:57:24  mju
 * Use miconstruct* instead of micon.
 *
 * Revision 1.39  2006/09/15 15:30:12  mju
 * Serializer changes.
 *
 * Revision 1.38  2006/09/01 17:19:17  mju
 * Add setMatte.
 *
 * Revision 1.37  2006/07/13 17:40:58  mju
 * Make all layout members private.
 *
 * Revision 1.36  2006/07/12 21:10:00  mju
 * Add getHardcopyDPI.
 *
 * Revision 1.35  2006/05/12 17:36:59  mju
 * Add getName.
 *
 * Revision 1.34  2006/05/05 13:48:11  mju
 * Support deferred creation.
 *
 * Revision 1.33  2005/12/13 14:46:36  mju
 * Add createGroupNoNotify.
 *
 * Revision 1.32  2005/11/28 18:53:42  mju
 * Add getMax/MinScaleVisibleAny.
 *
 * Revision 1.31  2005/11/11 22:52:00  mju
 * Use mistring for layout and reference group names.
 *
 * Revision 1.30  2005/11/04 18:52:24  mju
 * enums in gre namespace.
 *
 * Revision 1.29  2005/11/02 23:00:49  mju
 * Remove objmgr stuff.
 *
 * Revision 1.28  2005/11/02 20:43:33  mju
 * Remove unused methods.
 *
 * Revision 1.27  2005/10/27 14:04:46  mju
 * Add dlgEditLayoutSettings.
 *
 * Revision 1.26  2005/10/19 20:05:52  mju
 * Use gre namespace instead of legacy typedefs.
 *
 * Revision 1.25  2005/10/17 16:51:56  mju
 * Add createflag_NoCallCreateCB.
 *
 * Revision 1.24  2005/10/14 17:30:40  mju
 * GRE namespace.
 *
 * Revision 1.23  2005/06/10 20:36:06  vdronov
 * combined old and new 3d rendering
 *
 * Revision 1.22  2005/05/13 14:37:58  scowan
 * Moved global layout list to system.
 *
 * Revision 1.21  2005/05/02 17:44:17  mju
 * Move highlight mode to viewable.
 *
 * Revision 1.20  2005/03/31 16:57:06  fileserver!dwilliss
 * Rename one of our types to MIUNICODE because it conflicted with a Microsoft #define
 *
 * Revision 1.19  2005/03/25 21:14:58  mju
 * Add invalidateAllViews.
 *
 * Revision 1.17  2005/01/20 16:55:54  mju
 * Enums for highlightmode.
 *
 * Revision 1.16  2004/06/28 17:53:24  mju
 * Convert to spatref system.
 *
 * Revision 1.15  2004/03/26 16:47:19  mju
 * Add optional parm for addGroupLayer to return pointer to layer.
 *
 * Revision 1.14  2004/03/26 15:25:31  mju
 * Add addGroupLayer variant with no control dialog.
 *
 * Revision 1.13  2004/03/05 22:36:58  mju
 * Make activegroup, prevlayout, nextlayout private.
 *
 * Revision 1.12  2004/03/05 21:49:35  mju
 * Add getNext().
 *
 * Revision 1.11  2004/02/06 16:01:41  mju
 * Use GRE::FINDFLAGS.
 *
 * Revision 1.10  2003/11/12 22:41:52  mju
 * Use serializer to log errors instead of group.
 *
 * Revision 1.9  2003/11/10 18:43:48  mju
 * REmove savefilename/inode as need in base.
 *
 * Revision 1.3  2003/08/29 20:14:18  mju
 * Add v_GetExtents.
 *
 * Revision 1.2  2003/07/30 15:20:34  mju
 * Ignore private section.
 *
 * Revision 1.1  2003/06/16 19:39:47  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_LAYOUT_H
#define	INC_GRE_LAYOUT_H

#ifndef	INC_GRE_GROUP_H
	#include <gre/group.h>
#endif

#ifndef INC_MI32_MPLAYOUT_H
	#include <mi32/mplayout.h>
#endif


#define	POSNMODE_Tiled				0
#define	POSNMODE_Overlapped		1
#define	POSNMODE_MapLayout		2

#define	GRE_WorldRangeFP	1000000.0		//!< Range in floating-point

namespace GRE {
//===================================================================================================================

class LAYOUT : public GRE::VIEWABLE, public SERIALIZABLE {

	public:

		//! Flags used in creation
		enum CREATEFLAGS {
			CREATEFLAG_None =					0x0000,
			CREATEFLAG_NoCallCreateCB =	0x0001,	//!< Don't call callback list when object created
			CREATEFLAG_Temporary =			0x0002,	//!< Create as 'temporary'
			CREATEFLAG_Hardcopy =			0x0010,	//!< Create layout in "hardcopy" mode
			CREATEFLAG_DeferCreate =		0x0020	//!< Defer creation until Create() called
			};

		//! Install SML class, called by internal SML function.
		static void SMLClassInstall (
			SML::CONTEXT *context							//!< SML context
			);

		//! Constructor.
		explicit LAYOUT (
			CREATEFLAGS flags = CREATEFLAG_None
			);

		//! Destructor.
		virtual ~LAYOUT ();

	#ifdef USE_MGUI
		//! Dialog to add single-layer group and layer of specified type.
		//! Should only be used for single-layer-group types (eg scalebar, text, legend).
		ERRVALUE AddGroupLayer (
			MDLGPARENT dlgparent,
			GRE::LAYER::TYPE layertype,
			GRE::LAYER **player = 0						//! Pointer to layer returned
			);
	#endif

	#ifdef USE_MGUI
		//! Dialog to prompt user for objects and add as multiple groups.
		ERRVALUE AddGroupMulti (
			MDLGPARENT dlgparent
			);
	#endif

	#ifdef USE_MGUI
		//! Dialog to prompt user for saved group and add to layout.
		int AddGroupSaved (
			MDLGPARENT dlgparent,
			bool fromtext
			);
	#endif

		//! Create layout if CREATEFLAG_DeferCreate was specified in constructor.
		void Create (
			CREATEFLAGS createflags = CREATEFLAG_None
			);

		//! Create new group in this layout
		ERRVALUE CreateGroup (
			GRE::GROUP **retgroup = 0,					//!< New group returned
			GRE::GROUP::CREATEFLAGS createflags = GRE::GROUP::CREATEFLAG_None,
			GRE::LISTPOS listpos = GRE::LISTPOS_After,
			GRE::GROUP *refgroup = 0						//!< Reference group for InsertBefore/InsertAfter
			);

		//! Destroy all groups in this layout
		//! This method must not be used if any groups exist which have been created on the stack.
		void DestroyAllGroups (
			);

		//! Find layer using unique ID.
		virtual GRE::LAYER *FindLayerByID (
			UINT32 LayerID
			) const;

		//! Find group containing specified point
		GRE::GROUP* FindPointGroup (
			const DPOINT2D& point,
			GRE::FINDFLAGS findflags,
			const SPATREF::COORDREFSYS *pPointCRS = NULL
			) const;

		//! Get active group for this layout
		GRE::GROUP* GetActiveGroup (
			) const { return (m_ActiveGroup); }

		//! Get first group in layout.
		GRE::GROUP* GetFirstGroup (
			) const { return (m_FirstGroup); }

		//! Get full layout name
		virtual void GetFullName (
			MISTRING& name									//!< Name returned
			) const;

		//! Get hardcopy DPI value.
		double GetHardcopyDPI (
			) const { return (m_HardcopyLayout.dpi); }

		//! Get "margin" rectangle in layout (page) coordinates
		DRECT2D GetHardcopyMarginRect (
			) const;

		//! Get "page" rectangle in layout (page) coordinates
		DRECT2D GetHardcopyPageRect (
			) const;

		//! Get "printable" rectangle in layout (page) coordinates
		DRECT2D GetHardcopyPrintableRect (
			) const;

		//! Get hardcopy layout scale.
		double GetHardcopyScale (
			) const { return (m_HardcopyLayout.Scale); }

		//! Get hardcopy layout values.
		const LAYOUTPARMS& GetHardcopyValues (
			) const { return (m_HardcopyLayout); }

		//! Get hardcopy zoom factor.
		double GetHardcopyZoom (
			) const { return (m_HardcopyLayout.Zoom); }

		//! Get last group in layout.
		GRE::GROUP* GetLastGroup (
			) const { return (m_LastGroup); }

		//! Get matte reference for layout.
		MATTE& GetMatte (
			) { return (m_Matte); }

		//! Get name of layout.
		const MISTRING& GetName (
			) const { return (m_name); }

		//! Get next layout.
		LAYOUT* GetNext (
			) const { return (m_NextLayout); }

		//! Get next available unique ID.
		virtual UINT32 GetNextLayerID (
			) const;

		//! Get group positioning mode.
		UINT8 GetPosnMode (
			) const { return (m_PosnMode); }

		//! Get Saved Locations string
		const MISTRING& GetSavedLocations (
			) const { return (m_SavedLocations); }

		//! Compute dimensions of object based on scale and georeference
		virtual ERRVALUE GetSizeMeters (
			DPOINT2D& sizemeters							//!< Size returned in meters
			);

		//! Return true if Auto-Redraw toggle is set on the Group Controls dialog.
		bool GroupControlsGetAutoRedraw (
			) const;

		//! Return the units for measurements in the Layout tab of the Group Controls dialog.
		int GroupControlsGetLayoutUnits (
			) const;

		//! Determine if the Group Controls dialog is open.
		bool GroupControlsIsOpen (
			) const;

		//! Open the Group Controls dialog.
		//! If already open, it is brought to the front.
		//! If group passed in is NULL (default) use the Active group.
		void GroupControlsOpen (
			MDLGPARENT dlgparent = 0,		//!< Parent (0 to use default)
			GRE::GROUP* group = 0,			//!< Group to show initially,
			UINT32 flags = 0 					//!<	MDISP_GCP_Xxxx
			);

		//! Set the values of the size, spacing and relative zoom prompts in the Group Controls dialog.
		//!
		//! This is used to temporarly change the values being reported in
		//! several of the prompts while a group is being moved or resized by
		//! an external tool.  It is assumed that when the user releases the mouse,
		//! the tool will actually modify the group and the prompts will be
		//! restored to new "real" values.
		//!
		//! This function should only be called by GRE::TOOL_PLACEMENT
		void GroupControlsSetPrompts (
			double hoffset,		//!< value for "Spacing" (horizontal)
			double hsize,			//!< value for "Width"
			double voffset,		//!< value for "Spacing" (vertical)
			double vsize,			//!< value for "Height"
			double relscale		//!< value for "Relative Zoom"
			);
		
		//! Set the group controls dialog (if open) to the "layout" tab
		void GroupControlsSetTabLayout (
			);

		//! Determine if layout has any layers
		virtual bool HasLayers (
			) const;

		//! Determine if layout has any "temporary" layers.
		bool HasTempLayers (
			) const;

		//! Invalidate all view buffers for this layout.
		void InvalidateAllViews (
			);

		//! Determine if layout is currently being loaded.
		bool IsBeingLoaded (
			) const { return (m_pSerializer != 0); }

		//! Determine if layout has been created.
		//! This is done automatically by constructor unless CREATEFLAG_DeferCreate was specified.
		bool IsCreated (
			) const { return (m_IsCreated); }

		//! Determine if layout is in "hardcopy" mode.
		bool IsHardcopy (
			) const { return (m_PosnMode == POSNMODE_MapLayout); }

		//! Determine if legends should be sorted in "natural" order.
		bool IsSortLegendNatural (
			) const { return (m_SortLegendNatural); }

		//! Determine if layout is an existing GRE object.
		bool IsValid (
			) const;

		//! Determine if layout should be kept in TNTserver cache.
		bool KeepInTNTserverCache (
			) const { return (m_KeepInTNTserverCache); }

		//! Read from serialization
		virtual ERRVALUE SerialRead (
			SERIALIZER& serializer
			);

		//! Write values to serialization
		virtual ERRVALUE SerialWrite (
			SERIALIZER& serializer,
			const char *tagname = 0
			) const;

		//! Set active group for layout.
		void SetActiveGroup (
			GRE::GROUP *group
			);

		//! Set hardcopy layout values.
		void SetHardcopyValues (
			const LAYOUTPARMS& values,
			bool notify = true
			);

		//! Set whether layout should be retained in TNTserver cache.
		void SetKeepInTNTserverCache (
			bool keepInTNTserverCache
			) {
			if (m_KeepInTNTserverCache != keepInTNTserverCache) m_IsModified = true;
			m_KeepInTNTserverCache = keepInTNTserverCache;
			}

		//! Set matte values.
		void SetMatte (
			const MATTE& matte,						//!< Matte values to set
			bool notify = true						//!< Set to invoke callbacks with "Changed" message
			);

		//! Set that layout has been modified.
		void SetModified (
			bool notify = true,						//!< Set to invoke callbacks with "Changed" message
			bool ViewsNeedRedraw = true			//!< Set all views of layout as needing redraw
			);

		//! Set new layout name.
		ERRVALUE SetName (
			const MISTRING& newname
			);

		//! Set that group positions need to be recomputed.
		void SetNeedUpdatePositions (
			) { m_NeedUpdatePositions = 1; }

		//! Set group positioning mode
		void SetPosnMode (
			UINT8 posnmode
			);

		//! Set saved locations string
		void SetSavedLocations (
			const MISTRING& string
			) { m_SavedLocations = string; }

		//! Set hardcopy zoom to fit to page size.
		int SetZoomToFitPage (
			);

		//! Read from template (serialization)
		ERRVALUE TemplateRead (
			SERIALIZER& serializer
			);

		//! Write template to serialization
		ERRVALUE TemplateWrite (
			SERIALIZER& serializer
			);

		//! Recompute group positions.
		int UpdatePositions (
			bool ForceUpdate = true
			);

		//! Validate and correct group attachments.
		void ValidateGroupAttachments (
			);

		#ifndef GENERATING_DOXYGEN_OUTPUT
	protected:
		static const SML::CLASSMEMBER* GetSMLClassMembers (
			);

	private:

		static const SERIALIZERITEM* SerialGetItemDef (
			SERIALIZER& serializer
			);

		static ERRVALUE SerializerCB_Group (
			SERIALIZER& serializer,
			const SERIALIZERITEM*,
			void *vlayout,
			SERIALIZERITEM::ACTION action
			);

		static ERRVALUE TemplateCB_Group (
			SERIALIZER& serializer,
			const SERIALIZERITEM*,
			void *vlayout,
			SERIALIZERITEM::ACTION action
			);

		static const SERIALIZERITEM* TemplateGetItemDef (
			);

		GRE::GROUP *m_FirstGroup;				// First group in layout
		GRE::GROUP *m_LastGroup;				// Last group in layout
		UINT8 m_PosnMode;							// Positioning mode, POSNMODE_...
		LAYOUTPARMS m_HardcopyLayout;			// Used in map/poster layout mode
		double m_PosnScale;						// Scale used for attachment offsets in group positioning (eg. 24000)
		DRECT2D m_FullExtents;					// Extents of entire layout in layout coordinates
		GRE::GROUP* m_ActiveGroup;				// Currently active group for layout
		LAYOUT *m_PrevLayout;					// Pointer to "previous" layout if multiple layouts are in use
		LAYOUT *m_NextLayout;					// Pointer to "next" layout if multiple layouts are in use
		bool m_IsCreated;							// Has been created
		bool m_NeedUpdatePositions;			// Need to update positions of layout children
		bool m_KeepInTNTserverCache;			// Keep layout in TNTserver cache
		bool m_SortLegendNatural;				// Sort legend in natural order
		SERIALIZER *m_pSerializer;				// Serializer used while loading
		INT32 m_tilecols;
		INT32 m_tilerows;
		MISTRING m_name;
		mutable MISTRING m_activegroupname;	// Name of active group used when saving/loading layout
		MATTE m_Matte;
		MISTRING m_SavedLocations;

		// Create group without CreateChildrenBegin/End notification.
		ERRVALUE CreateGroupNoNotify (
			GRE::GROUP **retgroup = 0,
			GRE::GROUP::CREATEFLAGS createflags = GRE::GROUP::CREATEFLAG_None,
			GRE::LISTPOS listpos = GRE::LISTPOS_After,
			GRE::GROUP *refgroup = 0
			);

		// GRE::OBJECT overrides.
		virtual bool v_Destroy (bool NotifyParent);
		virtual MICONSTRUCT* v_GetIcon () const;
		virtual const char* v_GetTypeName () const;

		// GRE::VIEWABLE overrides.
		virtual void v_Clear ();
		virtual ERRVALUE v_DrawAllViews (GRE::DRAWCONDITION condition);
		virtual GRE::LAYER* v_GetActiveLayer () const;
		virtual double v_GetDesignScale () const;
		virtual const DRECT2D& v_GetExtents () const;
		virtual double v_GetMaxScaleVisibleAny () const;
		virtual double v_GetMinScaleVisibleAny () const;
		virtual void v_SetDesignScale (double DesignScale, bool notify);
		virtual void v_UnhighlightAllElements (bool redraw);

		LAYOUT (const LAYOUT&);
		LAYOUT& operator= (const LAYOUT&);

		friend class GRE::GROUP;

		#endif // GENERATING_DOXYGEN_OUTPUT
	};	// End of LAYOUT

#ifndef	GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(LAYOUT::CREATEFLAGS);
#endif

//! Dialog to edit layout properties/settings.
ERRVALUE DlgEditLayoutSettings (
	MDLGPARENT dlgparent,			//!< Parent for dialog
	LAYOUT *layout						//!< Layout to edit settings for
	);

//===================================================================================================================
}	// End namespace GRE

#endif	// INC_GRE_LAYOUT_H
