/**
 * \file gre/lrimagespatmod.h
 * \brief GRE LAYER_IMAGE_SPATMOD definitions
 *
 * \if NODOC
 * $Id: lrimagespatmod.h_v 1.2 2008/08/28 13:19:32 mju Exp $
 *
 * $Log: lrimagespatmod.h_v $
 * Revision 1.2  2008/08/28 13:19:32  mju
 * Fix doxygen problem.
 *
 * Revision 1.1  2006/11/16 15:50:42  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_LRIMAGESPATMOD_H
#define	INC_GRE_LRIMAGESPATMOD_H

#ifndef	INC_GRE_LRIMAGE_H
	#include <gre/lrimage.h>
#endif

#ifndef	INC_SPATMOD_IMAGESTAGEAUTOPTR_H
	#include <spatmod/imagestageautoptr.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
#endif


namespace GRE {
//===================================================================================================================

//! SPATMOD (image pipeline) layer class
class LAYER_IMAGE_SPATMOD : public GRE::LAYER_IMAGE {
	public:

		//! Register this layer type
		static void RegisterType (
			);

		//! Construct layer and insert into list
		explicit LAYER_IMAGE_SPATMOD (
			GRE::GROUP *group,										//!< Group to contain layer
			CREATEFLAGS createflags = CREATEFLAG_None,	//!< Creation flags
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,		//!< Position in list
			LAYER *reflayer = 0								//!< Reference layer for insertion
			);

		virtual ~LAYER_IMAGE_SPATMOD ();

		ERRVALUE SetInputStage (
			SPATMOD::IMAGE::STAGE_AUTOPTR& StagePtr,
			bool TakeOwnership
			);

		ERRVALUE SetInputStage (
			SPATMOD::IMAGE::STAGE *StagePtr,
			bool TakeOwnership
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		typedef LAYER_IMAGE BASECLASS;

		// LAYER_IMAGE_SPATMOD::DISPPARM
		class DISPPARM : public LAYER::DISPPARM {
			public:

				DISPPARM ();
				virtual ~DISPPARM ();

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				virtual const SERIALIZERITEM* SerialGetItemDef (SERIALIZER& serializer) const;
				virtual ERRVALUE SerialWrite (SERIALIZER& serializer, const char *tagname = 0) const;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		DISPPARM m_dispparm;								//!< Layer-specific display parameters
		SPATMOD::IMAGE::STAGE_AUTOPTR m_StagePtr;

		ERRVALUE Draw2D (LAYERDC& layerdc);
		ERRVALUE AddStagesAfterSet ();

		// Overrides from LAYER.
	#ifdef USE_MGUI
		virtual int v_ControlPanel (MDLGPARENT dlgparent, UINT32 flags = 0);
	#endif
		virtual LAYER* v_Copy (GRE::GROUP *group) const;
		virtual ERRVALUE v_Draw (LAYERDC& layerdc);
		virtual bool v_HasNullImageCells () const;
		virtual const char* v_SerialGetTagName () const;
		virtual void v_SetDftName (bool ReplaceExisting);
		virtual ERRVALUE v_UpdateExtents (bool forceupdate);

		LAYER_IMAGE_SPATMOD (const LAYER_IMAGE_SPATMOD&);
		LAYER_IMAGE_SPATMOD& operator= (const LAYER_IMAGE_SPATMOD&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================
}	// End namespace GRE

#endif	// INC_GRE_LRIMAGESPATMOD_H
