/**
 * \file gre/lrshape.h
 * \brief  GRE::LAYER_SHAPE class definition.
 *
 * \if NODOC
 * $Id: lrshape.h_v 1.51 2013/02/15 20:47:13 mju Exp $
 *
 * $Log: lrshape.h_v $
 * Revision 1.51  2013/02/15 20:47:13  mju
 * Fix dox hdr.
 *
 * Revision 1.50  2012/05/24 19:30:42  mju
 * Add dispparm typedefs for base/thisclass.
 *
 * Revision 1.49  2012/05/23 21:16:03  mju
 * Make dispparm members private and remove unused.
 *
 * Revision 1.48  2011/11/08 13:38:24  mju
 * Use elemnum for some methods.
 *
 * Revision 1.47  2011/10/20 13:05:26  mju
 * Remove redundant hdrs.
 *
 * Revision 1.46  2011/10/12 16:38:05  mju
 * Remove scalePointsZ as now method on layer.
 *
 * Revision 1.45  2011/10/10 17:48:28  mju
 * Move callDrawFilter to layer.
 *
 * Revision 1.43  2010/11/15 18:11:16  mju
 * Add v_snapPoint.
 *
 * Revision 1.37  2008/03/17 21:33:38  mju
 * Change v_getLayerView to v_getLayerSection.
 *
 * Revision 1.36  2007/06/06 17:40:02  vdronov
 * added 3D rendering
 *
 * Revision 1.35  2007/04/13 17:51:23  mju
 * Add method to get menu for region generation.
 *
 * Revision 1.34  2007/03/12 17:42:51  mju
 * Add v_setObjectDescriptor.
 *
 * Revision 1.16  2005/02/23 18:54:58  mju
 * Move label placement stuff to layerview.
 *
 * Revision 1.14  2005/01/26 23:41:45  ldyas
 * added auto label element.
 *
 * Revision 1.1  2003/06/06 22:08:02  scowan
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_LRSHAPE_H
#define	INC_GRE_LRSHAPE_H

#ifndef	INC_GRE_LAYER_H
#include <gre/layer.h>
#endif

#ifndef	INC_GRE_LEGEND_H
#include <gre/legend.h>
#endif

#ifndef	INC_GRE_ELEMPARMS_H
#include <gre/elemparms.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
namespace RVC {
	class SHAPE;
	}
#endif // GENERATING_DOXYGEN_OUTPUT


namespace GRE {	
//===================================================================================================================

//! GRE layer class for viewing Shape spatial object.
class LAYER_SHAPE : public LAYER {
	public:

		//! Shape layer display parameters.
		class DISPPARM : public LAYER::DISPPARM {
			public:

				class DLG;

				//! Default constructor.
				DISPPARM ();

				//! Copy constructor.
				DISPPARM (
					const DISPPARM& rhs
					) { Copy(rhs); }

				//! Destructor.
				virtual ~DISPPARM ();

				//! Assignment.
				DISPPARM& operator= (
					const DISPPARM& rhs
					) { if (this != &rhs) { Free(); Copy(rhs); } return (*this); }

				void ClearGeorefName ()
					{ m_GeorefName.Clear(); }

				const AUTOLABELPARMS& GetAutoLabelParms () const
					{ return (m_AutoLabel); }

				const DATATIPPARMS& GetDataTip () const
					{ return (m_DataTip); }

				const RVC::OBJECTNAME& GetGeorefName () const
					{ return (m_GeorefName); }

				const RVC::OBJECTNAME GetLegendElemTableName () const
					{ return (m_LegendElemTableName); }

				const LEGENDPARMS& GetLegendParms () const
					{ return (m_Legend); }

				const LINEPARMS::LINESTYLEPARMS GetLineStyle () const
					{ return (m_LineStyle); }

				const RVC::OBJITEM& GetObjItem () const
					{ return (m_ObjItem); }

				const RVC::OBJITEM& GetObjItemStyle () const
					{ return (m_ObjItemStyle); }

				const POINTPARMS::POINTSTYLEPARMS GetPointStyle () const
					{ return (m_PointStyle); }

				const POLYPARMS::POLYSTYLEPARMS GetPolygonStyle () const
					{ return (m_PolyStyle); }

				const SELECTPARMS& GetSelectParms () const
					{ return (m_Select); }

				void SetDataTip (
					const DATATIPPARMS DataTip
					) { m_DataTip = DataTip; }

				void SetEditable (
					bool editable = true
					) { m_IsEditable = editable; }

				void SetGeorefName (
					const RVC::OBJECTNAME& GeorefName
					) { m_GeorefName = GeorefName; }

				void SetLineStyleMode (
					UINT16 StyleMode
					) { m_LineStyle.Mode = StyleMode; }

				//! Set to use specified object.
				ERRVALUE SetObject (
					const RVC::OBJITEM& objitem,		//!< Object to set
					bool keepsettings = false			//!< Retain current display settings
					);

				void SetObjectDescriptor (
					const RVC::DESCRIPTOR descriptor
					) { m_ObjItem.SetObjectDescriptor(descriptor); }

				void SetObjItemStyle (
					const RVC::OBJITEM& ObjItemStyle
					);

				void SetPointStyleMode (
					UINT16 StyleMode
					) { m_PointStyle.Mode = StyleMode; }

				void SetPolygonStyleMode (
					UINT16 StyleMode
					) { m_PolyStyle.Mode = StyleMode; }

				void SetSaveObjItems (
					const RVC::OBJITEM& ObjItemCAD,
					const RVC::OBJITEM& ObjItemStyle
					) { m_SaveObjItem = ObjItemCAD; m_SaveObjItemStyle = ObjItemStyle; m_HasSaveInfo = true; }

				void SetSelectMode (
					SELECTMODE SelectMode
					) { m_Select.Mode = static_cast<UINT16>(SelectMode); }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				typedef LAYER::DISPPARM BASECLASS;
				typedef LAYER_SHAPE::DISPPARM THISCLASS;

				RVC::OBJITEM m_ObjItem;
				RVC::OBJECTNAME m_GeorefName;
				RVC::OBJITEM m_ObjItemStyle;
				SELECTPARMS m_Select;
				POINTPARMS::POINTSTYLEPARMS m_PointStyle;
				LINEPARMS::LINESTYLEPARMS m_LineStyle;
				POLYPARMS::POLYSTYLEPARMS m_PolyStyle;
				DATATIPPARMS m_DataTip;
				LEGENDPARMS m_Legend;
				RVC::OBJECTNAME m_LegendElemTableName;
				AUTOLABELPARMS m_AutoLabel;
				mutable RVC::OBJITEM m_SaveObjItem;
				mutable RVC::OBJITEM m_SaveObjItemStyle;
				bool m_IsEditable;
				bool m_HasSaveInfo;

				static ERRVALUE SerializerCB_StyleFileName (SERIALIZER&, const SERIALIZERITEM *, void *, SERIALIZERITEM::ACTION);

				void Copy (const DISPPARM&);
				void Free ();
				void UpdateSaveInfo () const;

				// Overrides from SERIALIZABLE.
				virtual ERRVALUE SerialRead (SERIALIZER& serializer);
				virtual ERRVALUE SerialWrite (SERIALIZER& serializer, const char *tagname = 0) const;

				// Overrides from LAYER::DISPPARM.
				virtual const SERIALIZERITEM* SerialGetItemDef (SERIALIZER& serializer) const;
				virtual ERRVALUE v_SetObjItem (const RVC::OBJITEM& objitem, bool replace);
				virtual ERRVALUE v_Validate ();

				#endif // GENERATING_DOXYGEN_OUTPUT
			};	// End of LAYER_SHAPE::DISPPARM

		//! Register this layer type
		static void RegisterType ();

		//! Method to install SML class, called by internal SML function.
		static void SMLClassInstall (
			SMLCONTEXT *context							//!< SML context
			);

		//! Construct layer and insert into list.
		explicit LAYER_SHAPE (
			GRE::GROUP *group,										//!< Group to contain layer
			CREATEFLAGS createflags = CREATEFLAG_None,		//!< Creation flags
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,			//!< Position in list
			LAYER *reflayer = 0										//!< Reference layer for insertion
			);

		//! Destructor.
		virtual ~LAYER_SHAPE ();

		//! Retrieve layer-specific display parameters
		const DISPPARM& GetDispParm (
			) const { return (m_dispparm); }

		//! Set layer-specific display parameters.
		//! This is the recommended method for changing how an existing layer is to be
		//! rendered.  Validation will be performed on the specified parameters and
		//! adjustments made to the layer's copy if necessary.  Use of this method
		//! usually result in a ChangeBegin / ChangeEnd message pair, and will also
		//! generate a ChangeObject message if the spatial object used in the layer
		//! is changed.
		ERRVALUE SetDispParm (
			const DISPPARM& dispparm					//!< Parameters to set
			);

	protected:

		DISPPARM m_dispparm;								//!< Layer-specific display parameters

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIVDRAWDATA;

		class OPENCLOSE {
			public:
				OPENCLOSE ();
				~OPENCLOSE ();
				ERRVALUE Open (const LAYER_SHAPE *layer);
			private:
				const LAYER_SHAPE *m_layer;
			};

		static void LegendMsgHandler (GRAPHICTREEITEM_MSG *msg);

		GRE::LEGEND_POINT m_PointLegend;
		GRE::LEGEND_LINE m_LineLegend;
		GRE::LEGEND_POLYGON m_PolyLegend;
		COMPINFO m_ElementCompInfo;
		GRE::SCRIPTCONTEXT_STYLE *m_pStyleSC;
		int m_StyleSegPoint;
		int m_StyleSegLine;
		int m_StyleSegPoly;
		int m_StyleSegPoint3dBase;
		int m_StyleSegPoint3dExt;
		int m_StyleSegPoly3dExt;
		bool m_HaveTempDrapeRasts;
		RVC::OBJECT m_TempDrapeFile;
		RVC::OBJITEM m_TempDrapeRast;
		RVC::OBJITEM m_TempDrapeMask;
	#ifdef USE_MGUI
		DISPPARM::DLG *m_pDlgControlPanel;
	#endif
		RVC::SHAPE *m_pObjShape;
		GRE::GEOMETRIC3D *m_pGeometric3D;
		GRE::WIREFRAME *m_pWireframe;
		mutable INT32 m_PrivOpenCount;
		
		void ClosePrivateObject () const;
		ERRVALUE Draw2D (LAYERDC& layerdc);
		ERRVALUE Draw3D (LAYERDC& layerdc);
		int DrawPolyFilled3D (LAYERDC&);
		ERRVALUE DrawWireframe (LAYERDC& layerdc);
	#ifdef USE_MGUI
		void OnMenuLegendElemListAddRmv (MGUI::ID, bool);
	#endif
		ERRVALUE OpenPrivateObject () const;
		void Setup3D ();
		ERRVALUE SetupDrape3D (LAYERDC&);
		ERRVALUE SetupLegend ();
		int SetupRegion ();
		ERRVALUE SetupScriptContext (GRE::SCRIPTCONTEXT_DRAW& sc);
		ERRVALUE WireReadShape (LAYERDC& layerdc);

		// Overrides from LAYER.
		virtual ERRVALUE v_AssignObject (const RVC::OBJITEM& objitem, MDLGPARENT dlgparent);
		virtual bool v_CanShowDetails () const;
		virtual GRE::LAYERSECTION* v_ConstructLayerSection (GRE::VIEW* view);
	#ifdef USE_MGUI
		virtual int v_ControlPanel (MDLGPARENT dlgparent,UINT32 flags = 0);
	#endif
		virtual LAYER* v_Copy (GRE::GROUP *group) const;
		virtual const DATATIPPARMS* v_DataTipGetParms (int component) const;
		virtual ERRVALUE v_DataTipSetParms (int component, const DATATIPPARMS& DataTipParms);
		virtual ERRVALUE v_Draw (LAYERDC& layerdc);
		virtual ERRVALUE v_DrawElementLow (LAYERDC& layerdc, ELEMTYPE ElemType, ELEMNUM ElemNum, const void *style);
		virtual ERRVALUE v_DrawElementsLow (LAYERDC& layerdc, ELEMTYPE ElemType, const UNSIGNEDSET *pElemSet, const void *style);
		virtual ERRVALUE v_DrawElementSample (const GRE::ELEMSAMPLE_DC& sampledc, const RVC::ELEMENT& element, const LRECT2D& samplerect);
		virtual ELEMNUM v_FindClosestElement (const DPOINT2D& point, double searchdist, double *distret, int component, const UNSIGNEDSET *pSearchSet, GRE::FINDFLAGS findflags);
		virtual int v_GetComponent (ELEMTYPE ElemType) const;
		virtual MICONSTRUCT* v_GetComponentIcon (int component) const;
		virtual COMPINFO* v_GetComponentInfo (int component);
		virtual MISTRING v_GetComponentName (int component) const;
		virtual const void* v_GetElemDefaultStyle (ELEMTYPE ElemType) const;
		virtual UINT16 v_GetElemDrawSelectionMode (ELEMTYPE ElemType) const;
		virtual UINT16 v_GetElemDrawStyleMode (ELEMTYPE ElemType) const;
		virtual ERRVALUE v_GetElemExtents (ELEMTYPE ElemType, ELEMNUM ElemNum, DRECT3D& Extents);
		virtual ERRVALUE v_GetElemSetByExtent (ELEMTYPE ElemType, const DRECT2D& Extent, UNSIGNEDSET& ElemSet);
		virtual ELEMTYPE v_GetElemType (int component) const;
		virtual bool v_GetGeorefName (RVC::OBJECTNAME& name, int component = 0) const;
		virtual bool v_GetLabelColor (LAYERDC&, const GRE::LABELPLACE&, COLOR&);
		virtual const GRE::LEGEND* v_GetLegend (ELEMTYPE ElemType);
		virtual INT64 v_GetNumElements (ELEMTYPE ElemType);
		virtual const RVC::OBJITEM& v_GetObjItem (int component) const;
		virtual ERRVALUE v_GetStyleInfo (ELEMTYPE ElemType, RVC::OBJITEM& ObjItem, RVC::OBJECTNAME& TableName);
		virtual bool v_HasLegend () const;
		virtual bool v_IsTableUsableForComponent (int component, const RVC::DBTABLE& table) const;
	#ifdef USE_MGUI
		virtual void v_MenuAddLayerItems (MGUI::MENU& menu);
		virtual void v_MenuAddRegionGenItems (MGUI::MENU& menu, ELEMTYPE ElemType);
		virtual ERRVALUE v_PromptObject (bool OpenCtrlPanel, MDLGPARENT dlgparent);
	#endif
		virtual ERRVALUE v_ReportRawElemInfo (MISTRING& infostr, ELEMTYPE ElemType, ELEMNUM ElemNum);
		virtual const char* v_SerialGetTagName () const;
		virtual ERRVALUE v_SerialRead (SERIALIZER& serializer);
		virtual void v_SetElemDrawSelectionMode (ELEMTYPE ElemType, UINT16 SelectMode);
		virtual void v_SetElemDrawStyleMode (ELEMTYPE ElemType, UINT16 StyleMode);
		virtual void v_SetLegendSampleFlags (MGD::STYLESAMPLEFLAGS SampleFlags);
		virtual void v_SetObjectDescriptor (int component, const RVC::DESCRIPTOR& descriptor);
		virtual ERRVALUE v_SetStyleObjItem (const RVC::OBJITEM& ObjItem);
		virtual int v_SnapPoint (DPOINT3D& ptObject, SNAPTO SnapTo, double MaxDistance, VIEW *view) const;
		virtual bool v_SnapPointIsAllowed (SNAPTO SnapTo) const;
		virtual int v_TestElementByRegion (ELEMTYPE ElemType, ELEMNUM ElemNum, const REGION2D& Region, TESTCRITERIA TestMode);
	#ifdef USE_MGUI
		virtual void v_TreeViewAddUpdateDetails (GRE::CTRL_TREEVIEW& TreeView, MGUI::CTRL_TREE::ITEMHND hndParent, MGUI::CTRL_TREE::ITEMHND hndInsertAfter);
		virtual MGUI::CTRL_TREE::ITEMHND v_TreeViewAddUpdateLegend (GRE::CTRL_TREEVIEW& TreeView, MGUI::CTRL_TREE::ITEMHND hndParent, MGUI::CTRL_TREE::ITEMHND hndInsertAfter);
	#endif
		virtual int v_UpdateExtents (bool forceupdate);

		LAYER_SHAPE (const LAYER_SHAPE&);
		LAYER_SHAPE& operator= (const LAYER_SHAPE&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}	// End namespace GRE

#endif	// INC_GRE_LRSHAPE_H

