/**
 * \file mgui/dlgselectmulti.h
 * \brief MGUI::DLG_SELECT_MULTI class definition.
 *
 * \if NODOC
 * $Id: dlgselectmulti.h_v 1.2 2011/11/22 13:32:02 mju Exp $
 *
 * $Log: dlgselectmulti.h_v $
 * Revision 1.2  2011/11/22 13:32:02  mju
 * Remove redundant hdr.
 *
 * Revision 1.1  2010/02/17 18:33:37  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_DLGSELECTMULTI_H
#define	INC_MGUI_DLGSELECTMULTI_H

#ifndef	INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

#ifndef	INC_MGUI_LISTITEM_H
#include <mgui/listitem.h>
#endif

namespace MGUI {

//! Dialog to display a set of togglebutton choices to select from.
class DLG_SELECT_MULTI : public DLGSHELL {
	public:

		//! Construct with title.
		explicit DLG_SELECT_MULTI (
			const MISTRING& dlgtitle,				//!< Text for title
			HELPID helpid = HELPID__None			//!< Help ID
			);

		//! Constructor with title and prompt string.
		DLG_SELECT_MULTI (
			const MISTRING& dlgtitle,				//!< Text for title
			const MISTRING& prompt,					//!< Text for prompt
			HELPID helpid = HELPID__None			//!< Help ID
			);

		//! Destructor.
		~DLG_SELECT_MULTI (
			);

		//! Add choice, must be used before the call to DoModal().
		void AddChoice (
			MGUI::ID id,						//!< MGUI::ID to assign to the item
			const MISTRING& string,			//!< The string to show
			bool selected,						//!< Initially selected or not
			bool canchange = true			//!< Allow selection to be changed
			);

		//! Determine if choice is selected (toggled on) or not.
		bool IsSelectedID (
			MGUI::ID id							//!< MGUI::ID of item
			) const;
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class CHOICE {
			public:
				CHOICE (const MISTRING& string, ID id, bool isSelected, bool isChangable)
					: m_string(string), m_id(id), m_isSelected(isSelected), m_isChangable(isChangable) { }

				ID GetID () const { return (m_id); }
				const MISTRING& GetString () const { return (m_string); }
				bool IsChangable () const { return (m_isChangable); }
				bool IsSelected () const { return (m_isSelected); }

			private:
				MISTRING m_string;
				ID m_id;
				bool m_isSelected;
				bool m_isChangable;
			};

		MILIST<CHOICE> m_ChoiceList;
		CTRL_LABEL m_PromptLabel;
		CTRL_TOGGLEBUTTON *m_Toggles;
		MISTRING m_PromptString;

		virtual ERRVALUE v_CreateContent ();
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


}	// End namespace MGUI

#endif	// INC_MGUI_DLGSELECTMULTI_H
