/**
 * \file mgui/html.h
 * \brief Definitions for MGUI::FORM_HTML.
 *
 * \if NODOC
 * $Id: html.h_v 1.10 2012/02/06 16:48:55 bcolombe Exp $
 *
 * $Log: html.h_v $
 * Revision 1.10  2012/02/06 16:48:55  bcolombe
 * Add redraw
 *
 * Revision 1.9  2011/11/22 21:37:37  mju
 * Fix dox dependencies.
 *
 * Revision 1.7  2008/05/15 21:48:44  scowan
 * Added setBackgroundColor method.
 *
 * Revision 1.6  2007/08/29 19:57:08  honerkamp
 * Added a GetRenderedDocSize function to find the area the html took to render
 *
 * Revision 1.5  2007/08/22 21:09:26  dwilliss
 * Added DLG_HTML
 *
 * Revision 1.4  2007/07/26 14:04:19  dwilliss
 * Added method to set minimum size in pixels
 *
 * Revision 1.1  2006/10/13 14:26:25  dwilliss
 * Initial revision
 * \endif
 **/

#ifndef	INC_MGUI_HTML_H
#define	INC_MGUI_HTML_H

#ifndef	INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class FILEPATH;
#endif


namespace MGUI {

//===================================================================================================================

//! An HTML Control
//!
//! This control is currently only implemented for X.  An MFC version could
//! probably be implemented if needed.
//!
//! The control is based on the XmHTML widget which is very old and doesn't
//! support any of the newer features that you'd find in a real browser (such
//! as CSS, Javascript, etc).
class FORM_HTML : public MGUI::FORM_COMPOSITE {
	public:
		
		typedef fastdelegate::FastDelegate<void(const MISTRING& href)> DELEGATE_HYPERLINK;

		//! Constructor.
		FORM_HTML (
			);

		//! Destructor.
		~FORM_HTML (
			);

		void Create (
			LAYOUT_PANE_BASE& ParentPane,
			int height,										//!< Height in pixels
			int width,										//!< Width in pixels
			LAYOUT_SIZEALIGN sizealign = LAYOUT_SIZEALIGN_Expand
			);

		//! Gets the height and width of the area that the htmldoc has used for rendering. 
		//! Note: the layout must already be rendered to call this.
		void GetRenderedDocSize(
			int &height,  //! height returned
			int &width	  //! width returned;
			) const;
		
		//! Return the title of the HTML document being shown
		void GetTitle (
			MISTRING& title
			) const;

		//! Redraw
		void Redraw ();

		//! Set the initial background color for this widget.
		//! Default is COLOR::WHITE			
		void SetBackgroundColor (
			const COLOR& BgColor
			);

		//! Set delegate to call when user clicks on a link
		void SetDelegateOnHyperLink (
			DELEGATE_HYPERLINK delegate
			) { m_DelegateOnHyperLink = delegate; }

		// Set the minimum size in pixels
		void SetMinimumSize (
			int height,		//!< Minimum height in pixels, -1 to leave unchanged
			int width		//!< Minimum width in pixels, -1 to leave unchanged
			);

		void SetValue (
			const MISTRING& value
			);

		void SetValue (
			const FILEPATH& filepath
			);

	private:
	#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV* m_priv;
		friend class PRIV;

		DELEGATE_HYPERLINK m_DelegateOnHyperLink;

		FORM_HTML (const FORM_HTML&);
		FORM_HTML& operator= (const FORM_HTML&);
	#endif // GENERATING_DOXYGEN_OUTPUT


	};

//! A Simple HTML viewer window
class DLG_HTML : public DLGSHELL {
	public:
		enum STYLE {
			STYLE_None = 0x00000000,
			STYLE_ShowAddressBar = 0x00000001,
			STYLE_ShowToolBar = 0x00000002,
			STYLE_AllLinksToExternalBrowser = 0x00000004
			};

		DLG_HTML (
			STYLE style = STYLE_None
			);

		virtual ~DLG_HTML();

		//! Gets the height and width of the area that the htmldoc has used for rendering. 
		//! Note: the layout must already be rendered to call this.
		void GetRenderedDocSize(
			int &height,  //! height returned
			int &width	  //! width returned;
			) const;
		
		//! Set the initial size in pixels
		void SetSize (
			INT32 height,
			INT32 width
			);

		//! Set delegate to call when user clicks on a link.
		//! By default, all links to a local html file or to an external
		//! url which produces a document type of text/html will open in
		//! this window.  The STYLE_AllLinkstoExternalBrowser flag
		//! changes this.
		//! NOTE: if you call this, you must do so BEFORE opening the window
		void SetDelegateOnHyperLink (
			FORM_HTML::DELEGATE_HYPERLINK delegate
			);

		void SetURL (
			const MISTRING& url
			);

		void SetValue (
			const FILEPATH& filepath
			);

		void SetValue (
			const MISTRING& value
			);

	private:
	#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV* m_priv;
		friend class PRIV;

		ERRVALUE v_CreateContent();

		DLG_HTML (const DLG_HTML&);
		DLG_HTML& operator= (const DLG_HTML&);
	#endif // GENERATING_DOXYGEN_OUTPUT

	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(DLG_HTML::STYLE)
#endif

//===================================================================================================================
}	// End namespace MGUI

#endif	// INC_MGUI_HTML_H
