/****
 *	\file mi32/contactinfo.h
 *	\brief Class CONTACTINFO - Handle user contact information
 *
 *	\if NODOC
 *	$Id: contactinfo.h_v 1.4 2011/09/06 21:28:51 mju Exp $
 *
 * $Log: contactinfo.h_v $
 * Revision 1.4  2011/09/06 21:28:51  mju
 * Rename define_enum macro.
 *
 * Revision 1.3  2006/10/09 17:50:33  scowan
 * Added two more strings.
 *
 * Revision 1.2  2006/07/28 19:54:36  scowan
 * Moved the required enum to here with cooresponding check.
 *
 * Revision 1.1  2006/05/12 16:31:42  scowan
 * Initial revision
 *
 *
 * \endif
****/

#ifndef	INC_MI32_CONTACTINFO_H
#define	INC_MI32_CONTACTINFO_H

#ifndef	INC_MI32_MISTRING_H
#include <mi32/mistring.h>
#endif

#ifdef MISYSTEMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

//! System file and directory services class.
class CLASSLIBEXPORT CONTACTINFO {
	public:
		enum REQUIRED {
			REQUIRED_None =		 		0x0000,
			REQUIRED_AdminArea = 		0x0001,
			REQUIRED_City = 				0x0002,
			REQUIRED_Country =			0x0004,
			REQUIRED_DeliveryPoint =	0x0008,
			REQUIRED_Email =				0x0010,
			REQUIRED_Fax =					0x0020,
			REQUIRED_FullName =			0x0040,
			REQUIRED_Organization =		0x0080,
			REQUIRED_Position =			0x0100,
			REQUIRED_PostalCode =		0x0200,
			REQUIRED_URL =					0x0400,
			REQUIRED_Voice =				0x0800,
			REQUIRED_HoursOfService =	0x1000,
			REQUIRED_Instructions =		0x2000,
			REQUIRED_All =					0x3FFF,
			REQUIRED_AllButPhone =		0x37DF
			};
	
		CONTACTINFO (
			);

		CONTACTINFO (
			const CONTACTINFO& rhs
			);

		~CONTACTINFO (
			);

		CONTACTINFO& operator= (
			const CONTACTINFO& rhs
			);

		bool CheckRequiredSet (
			REQUIRED RequiredSet
			) const;
			
		void Clear (
			);
			
		const MISTRING& GetAdministrativeArea (
			) const { return (m_AdministrativeArea); }
			
		const MISTRING& GetCity (
			) const { return (m_City); }
			
		const MISTRING& GetCountry (
			) const { return (m_Country); }
			
		const MISTRING& GetDeliveryPoint (
			) const { return (m_DeliveryPoint); }
			
		const MISTRING& GetEmail (
			) const { return (m_Email); }
			
		const MISTRING& GetFax (
			) const { return (m_Fax); }
			
		const MISTRING& GetFullName (
			) const { return (m_FullName); }
			
		const MISTRING& GetHoursOfService (
			) const { return (m_HoursOfService); }
			
		const MISTRING& GetInstructions (
			) const { return (m_Instructions); }
			
		const MISTRING& GetOrganization (
			) const { return (m_Organization); }
			
		const MISTRING& GetPosition (
			) const { return (m_Position); }
			
		const MISTRING& GetPostalCode (
			) const { return (m_PostalCode); }
			
		MISTRING GetReportString (
			) const;
			
		const MISTRING& GetURL (
			) const { return (m_URL); }

		const MISTRING& GetUserName (
			) const { return (m_UserName); }
			
		const MISTRING& GetVoice (
			) const { return (m_Voice); }
			
		bool IsEmpty (
			) const;
			
		bool IsEqualTo (
			const CONTACTINFO& rhs
			) const;

		void SetAdministrativeArea (
			const MISTRING& AdministrativeArea
			) { m_AdministrativeArea = AdministrativeArea; }
			
		void SetCity (
			const MISTRING& City
			) { m_City = City; }
			
		void SetCountry (
			const MISTRING& Country
			) { m_Country = Country; }
			
		void SetDeliveryPoint (
			const MISTRING& DeliveryPoint
			) { m_DeliveryPoint = DeliveryPoint; }
			
		void SetEmail (
			const MISTRING& Email
			) { m_Email = Email; }
			
		void SetFax (
			const MISTRING& Fax
			) { m_Fax = Fax; }
			
		void SetFullName (
			const MISTRING& FullName
			) { m_FullName = FullName; }
			
		void SetHoursOfService (
			const MISTRING& HoursOfService
			) { m_HoursOfService = HoursOfService; }
			
		void SetInstructions (
			const MISTRING& Instructions
			) { m_Instructions = Instructions; }
			
		void SetPosition (
			const MISTRING& Position
			) { m_Position = Position; }
			
		void SetPostalCode (
			const MISTRING& PostalCode
			) { m_PostalCode = PostalCode; }
			
		void SetOrganization (
			const MISTRING& Organization
			) { m_Organization = Organization; }
			
		void SetURL (
			const MISTRING& URL
			) { m_URL = URL; }
			
		void SetUserName (
			const MISTRING& UserName
			) { m_UserName = UserName; }
			
		void SetVoice (
			const MISTRING& Voice
			) { m_Voice = Voice; }
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MISTRING m_UserName;
		MISTRING m_FullName;
		MISTRING m_Position;
		MISTRING m_Organization;
		MISTRING m_Voice;
		MISTRING m_Fax;
		MISTRING m_DeliveryPoint;
		MISTRING m_City;
		MISTRING m_AdministrativeArea;
		MISTRING m_PostalCode;
		MISTRING m_Country;
		MISTRING m_HoursOfService;
		MISTRING m_Instructions;
		MISTRING m_Email;
		MISTRING m_URL;
		#endif // GENERATING_DOXYGEN_OUTPUT
			
	};

#undef	CLASSLIBEXPORT

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(CONTACTINFO::REQUIRED)
#endif

#endif
