/**
 * \file mi32/geofrmla.h
 * \brief GeoFormula class definitions
 *
 * \if NODOC
 * $Id: geofrmla.h_v 1.36 2011/09/06 21:12:28 mju Exp $
 *
 * $Log: geofrmla.h_v $
 * Revision 1.36  2011/09/06 21:12:28  mju
 * Exclude macro from dox.
 *
 * Revision 1.35  2009/08/26 22:11:13  bcolombe
 * Added Get and Set objitemlist
 *
 * Revision 1.34  2008/11/19 18:05:15  mju
 * Remove transparm usage.
 *
 * Revision 1.33  2007/05/09 14:36:32  dwilliss
 * missed an SML namespace issue
 *
 * Revision 1.32  2007/05/08 15:44:22  dwilliss
 * SML namespace
 *
 * Revision 1.31  2006/12/21 18:15:26  mju
 * No legacy rvc stuff.
 *
 * Revision 1.30  2006/10/27 17:56:39  scowan
 * Removed unnecessary use of vector and tin headers.
 *
 * Revision 1.29  2006/10/19 15:39:31  mju
 * Rmv sml, xdefns hdrs.
 *
 * Revision 1.28  2006/10/18 14:01:29  mju
 * Use contparm hdr.
 *
 * Revision 1.27  2006/09/29 14:39:30  scowan
 * Fixed deprecated items.
 *
 * Revision 1.26  2006/09/21 17:36:22  mju
 * Temporarily use xdefns hdr.
 *
 * Revision 1.25  2006/09/15 15:25:24  mju
 * Serializer changes.
 *
 * Revision 1.24  2005/09/16 15:02:30  mju
 * Disable warnings about deprecattion for also-deprecated method.
 *
 * Revision 1.23  2005/03/31 16:57:11  fileserver!dwilliss
 * Rename one of our types to MIUNICODE because it conflicted with a Microsoft #define
 *
 * Revision 1.22  2005/02/04 21:07:37  mju
 * Deprecate stuff using objinfo.
 *
 * Revision 1.21  2004/07/23 14:23:26  mju
 * Add getTransToExtCRS, remove getTransObjToMap.
 *
 * Revision 1.20  2004/07/22 19:43:47  mju
 * Keep separate objregion and mapregion.
 *
 * Revision 1.19  2004/06/30 19:27:34  mju
 * *** empty log message ***
 *
 * Revision 1.18  2004/06/29 14:31:44  mju
 * Add GetTransObjToMap.
 *
 * Revision 1.17  2004/06/29 14:25:29  mju
 * Use region2d.
 *
 * Revision 1.16  2004/06/28 17:55:27  mju
 * Convert to spatref system.
 *
 * Revision 1.15  2004/04/28 15:28:04  mju
 * Add ability to disable scriptcontext updates in editor.
 *
 * Revision 1.13  2004/02/07 16:16:53  scowan
 * Removed vector and tin object info's.
 *
 * Revision 1.12  2004/02/07 16:06:11  scowan
 * Changed to use rvc interface classes instead of qtree.
 *
 * Revision 1.10  2003/08/26 21:06:24  mju
 * When create formula from objects adjust duplicate names.
 *
 * Revision 1.1  2000/11/08 21:47:41  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_GEOFRMLA_H
#define	INC_MI32_GEOFRMLA_H

#ifndef	INC_RVC_TIN_H
#include <rvc/tin.h>
#endif

#ifndef	INC_RVC_VECTOR_H
#include <rvc/vector.h>
#endif

#ifndef	INC_RVC_RASTER_H
#include <rvc/raster.h>
#endif

#ifndef	INC_MI32_CONTRASTPARM_H
#include <mi32/contrastparm.h>
#endif

#ifndef	INC_MI32_SERIALIZERITEM_H
#include <mi32/serializeritem.h>
#endif

#if !defined(INC_MI32_XBASE_H) && defined(X_NATIVE)
#include <mi32/xbase.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
class RASTINTERP;
struct MENUITEM;
struct MxPromptCallbackStruct;
namespace SML {
	class SYMBOL;
	class EDITOR;
	class CONTEXT;
	}
#endif


//! Geospatial Formula class.
class GEOFORMULA {
	public:

		//! Output type of formula.
		// Do not change the values as they are stored in serializations.
		enum OUTPUTTYPE {
			OUTPUTTYPE_Numeric =		0,
			OUTPUTTYPE_Color =		1
			};

		//! Mode for determining extents.
		// Do not change the values as they are stored in serializations.
		enum EXTENTSMODE {
			EXTENTSMODE_UserDefined =		0,
			EXTENTSMODE_Intersection =		1,
			EXTENTSMODE_Union =				2,
			EXTENTSMODE_Region =				3
			};

		enum SCRIPTFLAGS {
			SCRIPTFLAG_None =				0x00,
			SCRIPTFLAG_AllSymbols =		0x01,
			SCRIPTFLAG_NoParse =			0x02
			};
			//-------------------------------------------------------------------------

		class EDITOR;

		//! GEOFORMULA::OBJECT
		class OBJECT {
			public:

				//! Copy constructor.
				OBJECT (
					const OBJECT& rhs
					);

				//! Destructor.
				virtual ~OBJECT (
					) = 0;

				//! Assignment.
				OBJECT& operator= (
					const OBJECT& rhs
					);

				ERRVALUE AddSymbols (
					GEOFORMULA& formula,
					bool OnlyIfUsed = true
					);

				void ClearSymbols (
					);

				void Close (
					);

				const RVC::OBJITEM& GetObjectItem (
					) const { return (m_objitem); }

				const FILEPATH& GetPath (
					) const { return (m_objitem.GetFilePath()); }

				const SPATREF::COORDREFSYS& GetCoordRefSys (
					) const { return (GetMapRegion().GetCoordRefSys()); }

				const REGION2D& GetMapRegion (
					) const { return (m_MapRegion); }

				const REGION2D& GetObjRegion (
					) const { return (m_ObjRegion); }

				const MISTRING& GetSymbolName (
					) const { return (m_SymbolName); }

				ERRVALUE Open (
					);

			protected:

				MISTRING m_SymbolName;					//!< Name used in formula script
				SML::SYMBOL *m_symobj;						//!< Symbol for object in context
				SIMPLE_ARRAY<DPOINT2D> m_pointbuf;	//!< Point buffer

				//! Construct from RVC object.
				OBJECT (
					const RVC::OBJITEM& objitem
					);

				//! Constructor used when reading formula definition.
				OBJECT (
					RVC::OBJTYPE objtype,
					INIHANDLE handle,
					int objnum
					);

				//! Constructor used in reading from serialization.
				OBJECT (
					RVC::OBJTYPE objtype,
					const MISTRING& SymbolName,
					const RVC::OBJITEM& ObjItem
					);

				ERRVALUE SetObjPoint (
					int idx
					);
					
				void WriteDefn (
					INIHANDLE handle,
					int objnum
					) const;

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT

				SML::SYMBOL *m_symx;						// Symbol for object X coordinate
				SML::SYMBOL *m_symy;						// Symbol for object Y coordinate
				MAT3X3 m_fwd, m_inv;						// Affine transformation, used if 'm_IsAffine' is set
				bool m_IsAffine;							// Set if transformation is affine

				RVC::OBJITEM m_objitem;
				REGION2D m_ObjRegion;					// Region defining object extents
				REGION2D m_MapRegion;					// Region defining object extents
				TRANS2D_MAPGEN m_trans;					// Transformation from object to virtual raster

				ERRVALUE CalcObjPoints (
					const GEOFORMULA& formula,
					const DPOINT2D& start,
					const DPOINT2D& inc,
					INT32 numvals
					);

				ERRVALUE SetupTrans (
					GEOFORMULA& formula
					);

				virtual ERRVALUE v_AddSymbols (
					GEOFORMULA& formula,
					bool OnlyIfUsed
					);

				virtual void v_ClearSymbols (
					);

				virtual void v_Close (
					) = 0;

				virtual ERRVALUE v_Open (
					) = 0;

				virtual ERRVALUE v_SetObjPoint (
					int idx
					) = 0;

				virtual void v_WriteDefn (
					INIHANDLE handle,
					int objnum
					) const = 0;

			friend class GEOFORMULA;
			#endif // GENERATING_DOXYGEN_OUTPUT
			};	// End of GEOFORMULA::OBJECT
			//-------------------------------------------------------------------------

		//! GEOFORMULA::RASTER
		class RASTER : public OBJECT {
			public:

				//! Copy constructor.
				RASTER (
					const RASTER& rhs
					);

				//! Destructor.
				virtual ~RASTER (
					);

				//! Assignment.
				RASTER& operator= (
					const RASTER& rhs
					);

				void SetContrastType (
					CONTRASTPARM::METHOD ContrastMethod
					) { m_ContrastMethod = ContrastMethod; }

				void SetContrastUseTable (
					bool contusetable
					) { m_UseContrastTable = contusetable; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT

				enum DATATYPE {
					DATATYPE_RasterNumeric =		0,
					DATATYPE_RasterColor =			1,
					DATATYPE_RasterComplex =		2
					};

				RVC::RASTER m_ObjRaster;
				DATATYPE m_datatype;					//!< Allowed data type
				CONTRASTPARM::METHOD m_ContrastMethod;						//!< Default contrast type (CTYPE_...)
				bool m_UseContrastTable;					//!< Use contrast table if available
				RASTINTERP *m_pRastInterp;
				SML::SYMBOL *m_symval;
				SML::SYMBOL *m_symred;
				SML::SYMBOL *m_symgreen;
				SML::SYMBOL *m_symblue;
				bool m_NeedValue;						//!< Set if need interpolated value

				//! Construct from RVC object.
				RASTER (
					const RVC::OBJITEM& ObjItem
					);

				//! Constructor used when reading formula definition.
				RASTER (
					INIHANDLE handle,
					int objnum
					);

				//! Constructor used in reading from serialization.
				RASTER (
					const MISTRING& SymbolName,
					const RVC::OBJITEM& ObjItem
					);

				ERRVALUE SerialWrite (
					SERIALIZER& serializer
					);
				#endif // GENERATING_DOXYGEN_OUTPUT

				virtual ERRVALUE v_AddSymbols (
					GEOFORMULA& formula,
					bool OnlyIfUsed
					);

				virtual void v_ClearSymbols (
					);

				virtual void v_Close (
					);

				virtual ERRVALUE v_Open (
					);

				virtual ERRVALUE v_SetObjPoint (
					int idx
					);

				virtual void v_WriteDefn (
					INIHANDLE handle,
					int objnum
					) const;

			friend class GEOFORMULA;
			friend class EDITOR;
			};	// End of GEOFORMULA::RASTER
			//-------------------------------------------------------------------------

		//! GEOFORMULA::VECTOR
		class VECTOR : public OBJECT {
			public:

				//! Default constructor.
				VECTOR (
					);

				//! Copy constructor.
				VECTOR (
					const VECTOR& rhs
					);

				//! Destructor.
				virtual ~VECTOR (
					);

				//! Assignment.
				VECTOR& operator= (
					const VECTOR& rhs
					);

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT

				RVC::VECTOR m_VectObj;
				
				//! Construct from RVC object.
				VECTOR (
					const RVC::OBJITEM& ObjItem
					);

				//! Constructor used when reading formula definition.
				VECTOR (
					INIHANDLE handle,
					int objnum
					):
					OBJECT(RVC::OBJTYPE_Vector,handle,objnum)
					{ }

				//! Constructor used in reading from serialization.
				VECTOR (
					const MISTRING& SymbolName,
					const RVC::OBJITEM& ObjItem
					);

				ERRVALUE SerialWrite (
					SERIALIZER& serializer
					);
				#endif // GENERATING_DOXYGEN_OUTPUT

				virtual void v_Close (
					);

				virtual ERRVALUE v_Open (
					);

				virtual ERRVALUE v_SetObjPoint (
					int idx
					);

				virtual void v_WriteDefn (
					INIHANDLE handle,
					int objnum
					) const;

			friend class GEOFORMULA;
			};	//! End of GEOFORMULA::VECTOR
			//-------------------------------------------------------------------------

		//! GEOFORMULA::TIN
		class TIN : public OBJECT {
			public:

				//! Default constructor.
				TIN (
					);

				//! Copy constructor.
				TIN (
					const TIN& rhs
					);

				//! Destructor.
				virtual ~TIN (
					);

				//! Assignment.
				TIN& operator= (
					const TIN& rhs
					);

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT

				RVC::TIN m_TinObj;
				SML::SYMBOL *m_symval;

				//! Construct from RVC object.
				TIN (
					const RVC::OBJITEM& ObjItem
					);

				//! Constructor used when reading formula definition.
				TIN (
					INIHANDLE handle,
					int objnum
					):
					OBJECT(RVC::OBJTYPE_TIN,handle,objnum)
					{ }

				//! Constructor used in reading from serialization.
				TIN (
					const MISTRING& SymbolName,
					const RVC::OBJITEM& ObjItem
					);

				ERRVALUE SerialWrite (
					SERIALIZER& serializer
					);
				#endif // GENERATING_DOXYGEN_OUTPUT

				virtual ERRVALUE v_AddSymbols (
					GEOFORMULA& formula,
					bool OnlyIfUsed
					);

				virtual void v_ClearSymbols (
					);

				virtual void v_Close (
					);

				virtual ERRVALUE v_Open (
					);

				virtual ERRVALUE v_SetObjPoint (
					int idx
					);

				virtual void v_WriteDefn (
					INIHANDLE handle,
					int objnum
					) const;

			friend class GEOFORMULA;
			};	// End of GEOFORMULA::TIN
			//-------------------------------------------------------------------------

		//! GEOFORMULA::VALUE
		class VALUE {
			public:

				VALUE (
					);

				VALUE (
					const VALUE& rhs
					);

				virtual ~VALUE (
					) {
					}

				VALUE& operator= (
					const VALUE& rhs
					);

				void ClearSymbol (
					) { m_Symbol = 0; }

				void CreateSymbol (
					SML::CONTEXT *context
					);

				double GetDefaultValue (
					) const { return (m_DefaultValue); }

				const MISTRING& GetSymbolName (
					) const { return (m_SymbolName); }

				void SetDefaultValue (
					double value
					) { m_DefaultValue = m_UserValue = value; }

				void SetSymbolName (
					const MIUNICODE* SymbolName
					) { m_SymbolName = SymbolName; }

				void SetUserValue (
					double value
					) { m_UserValue = value; }

				void WriteDefn (
					INIHANDLE handle,
					int idx
					) const;

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT

				MISTRING m_SymbolName;					//!< Name used in formula script
				double m_DefaultValue;					//!< Default value
				double m_UserValue;						//!< User-specified value
				SML::SYMBOL *m_Symbol;						//!< Symbol in context

				//! Constructor used to read definition.
				VALUE (
					INIHANDLE handle,
					int valnum
					);

				//! Constructor used to create new value
				VALUE (
					const MIUNICODE *name,
					double dftvalue
					);

				static const SERIALIZERITEM* SerialGetItemDef ();

				ERRVALUE SerialRead (
					SERIALIZER& serializer
					);

				ERRVALUE SerialWrite (
					SERIALIZER& serializer
					);

			#endif // GENERATING_DOXYGEN_OUTPUT
			friend class GEOFORMULA;
			friend class EDITOR;
			};	// End of GEOFORMULA::VALUE
			//-------------------------------------------------------------------------

		//! GEOFORMULA::EDITOR
	#ifdef X_NATIVE
		class EDITOR {
			public:

				//! Construct editor for specific formula.
				explicit EDITOR (
					GEOFORMULA& formula
					);

				//! Destructor.
				~EDITOR (
					) {
					}

				bool CheckValid (
					Widget pageform,
					int scriptpage
					);

				Widget CreateFormExtents (
					Widget pwidget,
					void *arglist,
					int argcount
					);

				void CreatePageObjects (Widget);

				void CreatePageValues (Widget);

				void CreatePageScript (Widget);

				void DisableUpdateContext (
					) { ++m_UpdateContextDisabled; }

				void EnableUpdateContext (
					) { if (m_UpdateContextDisabled) --m_UpdateContextDisabled; }

				SML::EDITOR* GetScriptEditHandle (
					) const {
					return (m_qehandle);
					}

				const MISTRING& GetScript (
					) const;

				void* GetTextEditHandle (
					) const {
					return (m_tehandle);
					}

				bool IsChanged (
					) const { return (m_IsChanged); }

				int Reset (
					);

				ERRVALUE SelectExtentsRegion (
					);

				void SetChanged (
					bool ischanged = true
					) { m_IsChanged = ischanged; }

				void SetMsgLine (
					Widget MsgLine
					);

				void ShowAllDetails (
					);

				void ShowObjListDetails (
					bool show = true
					);

				void ShowValListDetails (
					bool show = true
					);

				int UpdateObjects (
					);

				void UpdateScriptContext (
					bool doinit
					);

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT

				struct OBJINFO {

					EDITOR& m_editor;
					OBJECT& m_object;
					Widget m_form;
					Widget m_DetailsTB;
					Widget m_SymbolP;
					Widget m_ObjectLabel;
					Widget m_detform;
					Widget m_ContrastMethodOM;
					Widget m_usecontabTB;
					MIUNICODE m_symname[32];

					OBJINFO (
						EDITOR& editor,
						OBJECT& object
						):
						m_editor(editor),
						m_object(object)
						{
						}
					};

				struct VALINFO {

					EDITOR& m_editor;
					VALUE& m_value;
					Widget m_form;
					Widget m_SymbolP;
					Widget m_detform;
					MIUNICODE m_symname[32];

					VALINFO (
						EDITOR& editor,
						VALUE& value
						):
						m_editor(editor),
						m_value(value)
						{
						}
					};

				GEOFORMULA& m_Formula;						//!< Reference to formula being edited
				Widget m_Shell;
				Widget m_MsgLine;
				Widget m_ObjectsPage;
				Widget m_ObjectsRC;
				Widget m_ShowObjListDetailsTB;
				Widget m_ValuesPage;
				Widget m_ValuesRC;
				Widget m_ShowValListDetailsTB;
				Widget m_ScriptPage;
				Widget m_ScriptXeT;
				Widget m_ExtentsOM;
				Widget m_ExtentsRegionPB;
				Widget m_ExtentsRegionXeT;
				void *m_tehandle;								//!< MxTextEdit handle
				SML::EDITOR *m_qehandle;						//!< Query editor handle
				bool m_IsChanged;
				bool m_HasRaster;
				bool m_ShowObjListDetails;
				bool m_ShowValListDetails;
				UINT32 m_UpdateContextDisabled;
				MIUNICODE m_NewValueName[16];
				double m_NewValueDefault;

				MILIST<OBJINFO> m_ObjInfoList;
				MILIST<VALINFO> m_ValInfoList;
				REGION2D m_MapRegionUnion;						// Union of selected object MapRegions
				REGION2D m_MapRegionIntersection;			// Intersection of selected object MapRegions

				static void CB_AddValue (Widget, void*, void*);
				static void CB_OptionExtentsMode (Widget, void*, void*);
				static void CB_OptionRastContType (Widget, void*, void*);
				static void CB_SelectExtentsRegion (Widget, void*, void*);
				static void CB_ToggleRastContUseTab (Widget, void*, void*);

				static MENUITEM* GetMenuOptionContrast ();
				static MENUITEM* GetMenuOptionExtentsMode ();

				static void PCB_ChangeObjectSymbolName (
					MxPromptCallbackStruct *cbs,
					void *vobjinfo
					);

				static void PCB_ChangeValueSymbolName (
					MxPromptCallbackStruct *cbs,
					void *vobjinfo
					);

				static bool NewValueCheckOK (
					void *veditor
					);

				static Widget NewValueCreateForm (
					Widget pwidget,
					void *veditor,
					void *arglist,
					int argcount
					);

				void ClearMessage (
					) const;

				void CreateObjectForm (
					OBJECT& obj,
					const char *IconName
					);

				void CreateValueForm (
					VALUE& val
					);

				void SetSensitive () const;

				EDITOR (const EDITOR&);
				EDITOR& operator= (const EDITOR&);
				#endif // GENERATING_DOXYGEN_OUTPUT

			friend class GEOFORMULA;
			};	// End of GEOFORMULA::EDITOR
	#endif
			//-------------------------------------------------------------------------

		//! Default constructor.
		GEOFORMULA (
			);

		//! Copy constructor.
		GEOFORMULA (
			const GEOFORMULA& rhs
			);

		//! Destructor.
		~GEOFORMULA (
			);

		//! Assignment.
		GEOFORMULA& operator= (
			const GEOFORMULA& rhs
			);

		//! Add new "value" symbol to formula.
		ERRVALUE AddValue (
			const MIUNICODE *SymbolName,				//!< Symbol name
			double value								//!< Initial value
			);

		//! Create new formula from specified list of objects.
		ERRVALUE CreateFromObjects (
			const RVC::OBJITEMLIST& objitems
			);

	#ifdef USE_MGUI
		//! Display dialog allowing user to select region for extents.
		ERRVALUE DlgSelectExtentsRegion (
			MDLGPARENT dlgparent,
			const REGION2D *testregion = 0					//!< Region to test for overlap with, 0 to allow all regions
			);
	#endif

	#ifdef USE_MGUI
		//! Display dialog allowing user to select specific objects to use.
		ERRVALUE DlgSelectObjects (
			MDLGPARENT dlgparent
			);
	#endif

		//! Evaluate formula for points along specified interval.
		ERRVALUE Evaluate (
			const DPOINT2D& start,
			const DPOINT2D& increment,
			INT32 numvals,
			void *buffer,
			UINT8 *omask
			);

		//! Get extents region.
		const REGION2D& GetExtMapRegion (
			) const { return (m_ExtMapRegion); }

		//! Get coordinate reference system of first object used in formula (for auto-match).
		ERRVALUE GetFirstObjectCoordRefSys (
			SPATREF::COORDREFSYS& CoordRefSys	//!< Coordinate Reference System returned
			) const;

		//! Get minimum cell size for all rasters used in formula.
		ERRVALUE GetMinRasterCellSize (
			double& MinCellSize						//!< Cell size returned, DBL_MAX if no rasters in formula
			) const;

		//! Get number of objects used in formula.
		int GetNumObjects (
			) const {
			return (m_Rasters.GetNumItems() + m_Vectors.GetNumItems() + m_TINs.GetNumItems());
			}

		//! Get objitem list
		void GetObjItems(RVC::OBJITEMLIST& objitems);

		//! Get raster object list.
		const MILIST<RASTER>& GetObjectListRaster (
			) const {
			return (m_Rasters);
			}

		//! Get TIN object list.
		const MILIST<TIN>& GetObjectListTIN (
			) const {
			return (m_TINs);
			}

		//! Get vector object list.
		const MILIST<VECTOR>& GetObjectListVector (
			) const {
			return (m_Vectors);
			}

		//! Get formula output type.
		OUTPUTTYPE GetOutputType (
			) const {
			return (m_OutputType);
			}

		//! Get formula script.
		const MISTRING& GetScript (
			) const {
			return (m_Script);
			}

		//! Get transformation from virtual formula object to extents CRS.
		ERRVALUE GetTransToExtCRS (
			TRANS2D_MAPGEN& Trans,					//!< Transformation returned
			bool dooutput = false					//!< Place in output size of transformation
			) const;

		//! Open objects to use in formula.
		ERRVALUE OpenObjects (
			);

		//! Read previously saved formula definition.
		ERRVALUE ReadDefn (
			const FILEPATH& filepath
			);

		//! Read from serialization.
		ERRVALUE SerialRead (
			SERIALIZER& serializer
			);

		//! Write to serialization.
		ERRVALUE SerialWrite (
			SERIALIZER& serializer,
			const char *tagname = 0
			);

		//! Set coordinate reference system for defining extents.
		void SetExtMapCoordRefSys (
			const SPATREF::COORDREFSYS& CoordRefSys
			) { m_ExtMapRegion.SetCoordRefSys(CoordRefSys); m_Trans.SetOutputMapCoordRefSys(CoordRefSys); }

		//! Set objitem list, must be created first
		ERRVALUE SetObjItems (const RVC::OBJITEMLIST& objitems);

		//! Set output coordinate reference system
		void SetOutputCoordRefSys (
			const SPATREF::COORDREFSYS& CoordRefSys
			) { m_Trans.SetInputMapCoordRefSys(CoordRefSys); }

		//! Set transformation from output object to map coordinates.
		void SetOutputTrans (
			const MAT3X3 fwd,
			const MAT3X3 inv
			) { m_Trans.SetInputTrans(fwd,inv); }

		//! Set transformation from output object to map coordinates.
		void SetOutputTrans (
			const TRANS2D_AFFINE& trans
			) { m_Trans.SetInputTrans(trans); }

		//! Set output transformation from "output side" of a TRANS2D_MAPGEN reference.
		void SetOutputTrans (
			const TRANS2D_MAPGEN& trans
			) { m_Trans.CopyOutputToInput(trans); }

		//! Set transformation from output object to map coordinates.
		void SetOutputTransIdentity (
			) { m_Trans.SetInputTransIdentity(); }

		//! Set formula output type
		void SetOutputType (
			OUTPUTTYPE OutputType
			) { m_OutputType = OutputType; }

		//! Set script to use in formula.
		void SetScript (
			const MIUNICODE* script
			) { m_Script = script; }

		//! Start formula evaluation.
		ERRVALUE Start (
			const DRECT2D *extents,
			SCRIPTFLAGS flags = SCRIPTFLAG_None
			);

		//! Stop formula evaluation.
		void Stop (
			);

		//! Write formula definition to specified file.
		ERRVALUE WriteDefn (
			const FILEPATH& filepath
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		OUTPUTTYPE m_OutputType;						//!< Formula type
		EXTENTSMODE m_ExtentsMode;						//!< Extents mode
		SML::CONTEXT *m_Context;							//!< Query context
		int m_Segment;										//!< Query segment
		SML::SYMBOL *m_symred;								//!< Query symbol for red output value
		SML::SYMBOL *m_symgreen;							//!< Query symbol for green output value
		SML::SYMBOL *m_symblue;							//!< Query symbol for blue output value
		bool m_AllAffine;									//!< All transformations are affine
	#ifdef X_NATIVE
		EDITOR *m_editor;
	#endif

		MILIST<RASTER> m_Rasters;						//!< Raster objects used in formula
		MILIST<VECTOR> m_Vectors;						//!< Vector objects used in formula
		MILIST<TIN> m_TINs;								//!< TIN objects used in formula
		MILIST<VALUE> m_Values;
		MISTRING m_Script;								//!< Formula script expression
		RVC::OBJITEM m_ObjItemRegion;
		REGION2D m_ExtMapRegion;						//!< Extents region
		TRANS2D_MAPGEN m_Trans;							//!< Transformation from output raster to map
		SIMPLE_ARRAY<DPOINT2D> m_ipointbuf;

		static const SERIALIZERITEM* SerialGetItemDef ();
		static ERRVALUE SerializerCB_Object (SERIALIZER&, const SERIALIZERITEM*, void*, SERIALIZERITEM::ACTION);
		static ERRVALUE SerializerCB_Value (SERIALIZER&, const SERIALIZERITEM*, void*, SERIALIZERITEM::ACTION);

		ERRVALUE AddObjectRegion (
			const OBJECT& object
			);

		SML::SYMBOL* AddSymbol (
			const MISTRING& basename,
			const char* subname,
			bool OnlyIfUsed
			);

		void Clear ();

		void Copy (
			const GEOFORMULA& rhs
			);

		OBJECT* FindObjectByName (
			const MIUNICODE *symname
			);

		const VALUE* FindValueByName (
			const MIUNICODE *symname
			) const;

		const OBJECT* GetFirstObject () const;

		int GetNumNameUses (const MISTRING& symname) const;

		ERRVALUE InitQuery (
			SCRIPTFLAGS flags = SCRIPTFLAG_None
			);

		//! @return true if successful, false if name used by another object or value.
		bool SetObjectSymbolName (
			OBJECT& obj,
			const MIUNICODE *symname
			);

		//! @return true if successful, false if name used by another object or value.
		bool SetValueSymbolName (
			VALUE& obj,
			const MIUNICODE *symname
			);

		void StopQuery ();
		#endif // GENERATING_DOXYGEN_OUTPUT

	friend class EDITOR;
	friend class OBJECT;
	friend class RASTER;
	friend class VECTOR;
	friend class TIN;
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(GEOFORMULA::SCRIPTFLAGS)
#endif

#endif	// INC_MI32_GEOFRMLA_H
