/**
 * \file mi32/kmlbase.h
 * \brief MIKML base definitions
 *
 * \if NODOC
 * $Id: kmlbase.h_v 1.3 2013/01/04 15:44:10 mju Exp $
 *
 * $Log: kmlbase.h_v $
 * Revision 1.3  2013/01/04 15:44:10  mju
 * Add vec2 class.
 *
 * Revision 1.2  2012/12/21 15:42:37  mju
 * Add simplefieldtype enum.
 *
 * Revision 1.1  2012/12/17 16:15:51  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MIKML_BASE_H
#define	INC_MIKML_BASE_H

//=====================================================================================================================
#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
class DATETIME;
class POLYLINE;
class POLYLINELIST;
class SUBREGION2D;
class REGION2D;
#endif

namespace MIKML {

//=====================================================================================================================
//		Enumeration types - Begin

enum ALTITUDEMODE {
	ALTITUDEMODE_ClampToGround,
	ALTITUDEMODE_RelativeToGround,
	ALTITUDEMODE_Absolute,
	ALTITUDEMODE_RelativeToSeaFloor,
	ALTITUDEMODE_ClampToSeaFloor
	};

enum COLORMODE {
	COLORMODE_Normal,
	COLORMODE_Random
	};

enum GRIDORIGIN {
	GRIDORIGIN_LowerLeft,
	GRIDORIGIN_UpperLeft
	};

enum LISTITEMTYPE {
	LISTITEMTYPE_Check,
	LISTITEMTYPE_CheckOffOnly,
	LISTITEMTYPE_CheckHideChildren,
	LISTITEMTYPE_RadioFolder
	};

enum REFRESHMODE {
	REFRESHMODE_OnChange,
	REFRESHMODE_OnInterval,
	REFRESHMODE_OnExpire
	};

enum SHAPE {
	SHAPE_Rectangle,
	SHAPE_Cylinder,
	SHAPE_Sphere
	};

enum SIMPLEFIELDTYPE {
	SIMPLEFIELDTYPE_String,
	SIMPLEFIELDTYPE_Int,
	SIMPLEFIELDTYPE_Uint,
	SIMPLEFIELDTYPE_Short,
	SIMPLEFIELDTYPE_Ushort,
	SIMPLEFIELDTYPE_Float,
	SIMPLEFIELDTYPE_Double,
	SIMPLEFIELDTYPE_Bool
	};

enum STYLESTATE {
	STYLESTATE_Normal = 0,
	STYLESTATE_Highlight,
	STYLESTATE_NUMTYPES
	};

enum UNITS {
	UNITS_Fraction,
	UNITS_Pixels,
	UNITS_InsetPixels
	};

enum VIEWREFRESHMODE {
	VIEWREFRESHMODE_Never,
	VIEWREFRESHMODE_OnStop,
	VIEWREFRESHMODE_OnRequest,
	VIEWREFRESHMODE_OnRegion
	};

//		Enumeration types - End
//=====================================================================================================================
//		Special primitive types - Begin

typedef double ANGLE90;			//!< Value >= -90 and <= 90
typedef double ANGLEPOS90;		//!< Value >= 0 and <= 90
typedef double ANGLE180;		//!< Value >= -180 and <= 180
typedef double ANGLE360;		//!< Value >= -360 and <= 360

//		Special primitive types - End
//=====================================================================================================================

//! Container for x/y/xunits/yunits attributes.
class VEC2 {
	public:

		//! Construct with X/Y and optional units.
		VEC2 (
			double x,
			double y,
			UNITS units = UNITS_Fraction
			): m_x(x), m_y(y), m_xunits(units), m_yunits(units) { }

		//! Construct with X/Y and separate units.
		VEC2 (
			double x,
			double y,
			UNITS xunits,
			UNITS yunits
			): m_x(x), m_y(y), m_xunits(xunits), m_yunits(yunits) { }

		//! Get "x" value.
		double GetX () const
			{ return (m_x); }

		//! Get "y" value.
		double GetY () const
			{ return (m_y); }

		//! Get "xunits" value.
		UNITS GetXUnits () const
			{ return (m_xunits); }

		//! Get "yunits" value.
		UNITS GetYUnits () const
			{ return (m_yunits); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		double m_x;
		double m_y;
		UNITS m_xunits;
		UNITS m_yunits;
		#endif
	};

//=====================================================================================================================

}	// End namespace MIKML

#endif	// INC_MIKML_BASE_H
