/**
 * \file mi32/mg2rasterbinary.h
 * \brief MGD::DEVICE_RASTER_BINARY definitions
 *
 * \if NODOC
 * $Id: mg2rasterbinary.h_v 1.3 2011/10/26 17:58:18 mju Exp $
 *
 * $Log
 * \endif
**/

#ifndef INC_MI32_MG2RASTERBINARY_H
#define INC_MI32_MG2RASTERBINARY_H

#ifndef INC_MGD2_H
#include <mi32/mgd2.h>
#endif

#ifndef INC_RVC_RASTER_H
#include <rvc/raster.h>
#endif

namespace MGD {

//! Device for rendering to binary RVC::RASTER.
class DEVICE_RASTER_BINARY: public DEVICE {

	public:

		//! Construct binary raster device.
		//! After constructing, you should call Create()
		DEVICE_RASTER_BINARY ();

		//! Destructor
		virtual ~DEVICE_RASTER_BINARY ();

		//! Create new raster to draw into.
		ERRVALUE Create (
			RVC::OBJITEM& ObjItem,						//!< Information to create raster, location of created raster RETURNED
			RVC::RASTER::MAKEPARMS& MakeParms,
			const char *Source = 0
			);

		//! Open existing raster to draw into.
		ERRVALUE Open (
			const RVC::OBJITEM& ObjItem
			);

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT
		class CONTEXT_RASTER_BINARY;
		friend class CONTEXT_RASTER_BINARY;

		RVC::RASTER m_ObjRaster;
		SIMPLE_ARRAY<UINT8> m_buffer;

		ERRVALUE ReadAndInvert (INT32 col, INT32 row, INT32 len);
		ERRVALUE Setup();

		// Overrides
		virtual CONTEXT* v_CreateContext ();
		ERRVALUE v_FillSpan (MGD::CONTEXT* gc, INT32 x1, INT32 x2, INT32 y);
		MGD::PIXEL v_GetColorPixel (const COLOR& color);
		virtual UINT8* v_GetImagePointer (INT32 x, INT32 y, INT32 len, UINT16 FrameBuffer);
		virtual UINT8* v_GetImagePointerNoRead (INT32 x, INT32 y, INT32 len, UINT16 FrameBuffer);
		virtual ERRVALUE v_WriteImageBuffer (INT32 x, INT32 y, INT32 len, UINT8* buf, UINT16 FrameBuffer);

		DEVICE_RASTER_BINARY (const DEVICE_RASTER_BINARY&);
		DEVICE_RASTER_BINARY& operator= (const DEVICE_RASTER_BINARY&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

} // End namespace MGD


#endif	// INC_MG2RASTERBINARY_H

