/**
 * \file mi32/mijobstatus.h
 * \brief MIJOBSTATUS definition.
 *
 * \if NODOC
 * $Id: mijobstatus.h_v 1.4 2009/06/25 20:34:08 mju Exp $
 *
 * $Log: mijobstatus.h_v $
 * Revision 1.4  2009/06/25 20:34:08  mju
 * Don't override v_reportBegin so will log messages.
 *
 * Revision 1.3  2009/03/13 21:17:45  bcolombe
 * Added m_LastSentTime member for throttling sending of progress
 *
 * Revision 1.2  2009/01/23 18:54:35  bcolombe
 * *** empty log message ***
 *
 * Revision 1.1  2009/01/23 18:51:36  bcolombe
 * Initial revision
 *
 *
 * \endif
**/

#ifndef	INC_MI32_MIJOBSTATUS_H
#define	INC_MI32_MIJOBSTATUS_H

#ifndef	INC_MI32_MISTATUS_H
#include <mi32/mistatus.h>
#endif

#ifndef INC_MI32_SOCKET_H
#include <mi32/socket.h>
#endif

#ifndef INC_MI32_FILEPATH_H
#include <mi32/filepath.h>
#endif

#ifndef INC_MI32_ERRHANDLER_H
#include <mi32/errhandler.h>
#endif

//===================================================================================================================
class MIJOBSTATUS : public MISTATUSLOGGER {
	public:
		//! Constructor.
		MIJOBSTATUS (
			);

		//! Destructor.
		virtual ~MIJOBSTATUS (
			);

		//! Connect to job manager.  Path used to report which job the connection is for. 
		ERRVALUE Connect(const FILEPATH& path);

		MIJOBSTATUS (const MIJOBSTATUS& rhs);
		MIJOBSTATUS& operator= (const MIJOBSTATUS& rhs);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class CLIENT : public MISOCKET::CLIENT {
			public:
				CLIENT () {}
				~CLIENT() {}

				void SetDelegate (
					FastDelegate<void()> delegate
					) { m_Delegate = delegate; }

			private:
				virtual void v_OnReadReady () {m_Delegate();}
				FastDelegate<void()> m_Delegate;
			};

		void ReadReady();
		bool m_pause;
		bool m_cancel;
		double m_LastSentTime;
		CLIENT m_client;

		// MISTATUSMANAGER/LOGGER overrides.
		virtual bool v_IsCancelled ();
		virtual void v_ReportMessage (MISTATUSCONTEXT& context, const MISTRING& msgstr);
		virtual void v_ReportProgress (MISTATUSCONTEXT& context, double pctdone);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
//===================================================================================================================
#endif	// INC_MI32_MIJOBSTATUS_H
