/**
 * \file mi32/namedrgn.h
 * \brief NAMEDREGION class definition.
 *
 * \if NODOC
 * $Id: namedrgn.h_v 1.8 2011/11/04 12:58:30 mju Exp $
 *
 * $Log: namedrgn.h_v $
 * Revision 1.8  2011/11/04 12:58:30  mju
 * Remove redundant hdr.
 *
 * Revision 1.7  2007/03/23 14:36:21  scowan
 * Now uses descriptor.
 *
 * Revision 1.5  2005/03/07 18:25:01  scowan
 * Changed part of class to use descriptor.
 *
 * Revision 1.2  2001/01/26 23:03:53  mju
 * Add Load() method.
 *
 * Revision 1.1  2001/01/23 23:00:20  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_NAMEDRGN_H
#define	INC_MI32_NAMEDRGN_H

#ifndef	INC_RVC_REGIONCL_H
	#include <rvc/regioncl.h>
#endif

#ifndef	INC_MI32_OBSLIST_H
	#include <mi32/obslist.h>
#endif


//! Named region class.
class NAMEDREGION : public RVC::REGION {
	public:
		//! Default constructor.
		NAMEDREGION (
			);

		//! Copy constructor.
		NAMEDREGION (
			const NAMEDREGION& rhs
			);

		//! Assignment from NAMEDREGION.
		NAMEDREGION& operator= (
			const NAMEDREGION& rhs
			);

		//! Get region descriptor.
		const RVC::DESCRIPTOR& GetDescriptor (
			) const;

		//! Attach region to RVC object and load into memory.
		ERRVALUE Load (
			const RVC::OBJITEM& objitem
			);

		//! Set region descriptor.
		//! If region is bound to an object the object will be changed.
		ERRVALUE SetDescriptor (
			const RVC::DESCRIPTOR& Descriptor
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		RVC::DESCRIPTOR m_Descriptor;

		virtual ERRVALUE GetDftNameDesc (
			const RVC::OBJECT& parent,
			RVC::DESCRIPTOR& Descriptor
			);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//! List (observable) of named regions.
typedef OBSERVABLE_LIST<NAMEDREGION> NAMEDREGION_LIST;


#endif	//!< INC_MI32_NAMEDRGN_H
