/**
 * suncalc.h
 *
 * $Id: suncalc.h_v 1.1 2006/11/20 20:44:42 drb Exp $
 *
 * $Log: suncalc.h_v $
 * Revision 1.1  2006/11/20 20:44:42  drb
 * Initial revision
 *
 *
**/

#ifndef INC_MI32_SUNCALC_H 
#define INC_MI32_SUNCALC_H

#ifndef INC_MI32_DATETIME_H 
#include <mi32/datetime.h>
#endif

#ifndef INC_MI32_POINT_H 
#include <mi32/point.h>
#endif

//! Class that calculates the position of the sun based on location (Lat-Lon), date and time
class SUN_POSN_CALCULATOR {
	public: 
		SUN_POSN_CALCULATOR ( ) { };

		SUN_POSN_CALCULATOR ( 
			const DATETIME& datetime,
			const DPOINT2D& latlon
			);

		~SUN_POSN_CALCULATOR () { }

		//! Set the location in Latitude and Longitude
		void SetLocation (const DPOINT2D& location);

		//! Get the location in Latitude and Longitude
		const DPOINT2D& GetLocation () const;

		//! Set the date and time to use
		void SetDateTime (const DATETIME& datetime);

		//! Get the date and time used
		const DATETIME& GetDateTime() const; 

		//! Calculate the elevation and azimuth to the sun from the set
		//! location, date and time.
		ERRVALUE Calculate ( 
			double& SunAzimuth,			//! Where north is zero, clockwise 
			double& SunElevation		//! Where horizontal is zero, up is positive
			);

		//! Get the last calculated azimuth angle
		double GetSunAzimuth ( ) { return (m_SunAzimuth); };

		//! Get the last calculated elevation angle
		double GetSunElevationAngle ( ) { return (m_SunElevation); };

	private:
		double CalcMeanObliquityOfEcliptic ( double &t );
		double CalcObliquityCorrection ( double &t );
		double CalcGeomMeanLongSun ( double &t );
		double CalcEccentricityEarthOrbit ( double &t );
		double CalcSunEqOfCenter ( double &t );
		double CalcGeomMeanAnomalySun ( double &t );
		double CalcEquationOfTime ( double &t );
		double CalcSunApparentLong ( double &t );
		double CalcSunDeclination ( double &t );
		double CalcSunTrueLong ( double &t );
		double CalcTimeJulianCent ( double );

		DPOINT2D m_Location;
		DATETIME m_DateTime;

		double m_SunAzimuth;
		double m_SunElevation;
	};

#endif

