/**
 * \file mi32/trans2dmodel.h
 * \brief TRANS2DMODEL definitions
 *
 * $Id: trans2dmodel.h_v 1.3 2012/11/09 21:11:34 mju Exp $
 *
 * $Log: trans2dmodel.h_v $
 * Revision 1.3  2012/11/09 21:11:34  mju
 * Change rubberSheet to thinPlateSpline.
 *
 * Revision 1.2  2011/09/23 17:48:48  mju
 * Add flag_orientSame.
 *
 * Revision 1.1  2011/09/23 15:43:33  mju
 * Initial revision
**/

#ifndef	INC_MI32_TRANS2DMODEL_H
#define	INC_MI32_TRANS2DMODEL_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif


//! 2D transformation model enumeration.
// Values are stored in files, so must fit in UINT16 and may not be changed.
enum TRANS2DMODEL {
	TRANS2DMODEL_Order_2 =					2,
	TRANS2DMODEL_Order_3 =					3,
	TRANS2DMODEL_Order_4 =					4,
	TRANS2DMODEL_Order_5 =					5,
	TRANS2DMODEL_Order_6 =					6,
	TRANS2DMODEL_Order_7 =					7,
	TRANS2DMODEL_Order_8 =					8,
	TRANS2DMODEL_Order_9 =					9,

	TRANS2DMODEL_Identity =					0x0000,	//!< Identity
	TRANS2DMODEL_Affine =					0x0100,	//!< Affine
	TRANS2DMODEL_PlaneProj =				0x0200,	//!< Plane-projective 
	TRANS2DMODEL_Bilinear =					0x0300,
	TRANS2DMODEL_Polynomial =				0x0400,	//!< Polynomial, requires order to be specified
	TRANS2DMODEL_Quintic =					0x0500,
	TRANS2DMODEL_SpaceResect =				0x0600,	//!< Space resection
	TRANS2DMODEL_Conformal =				0x0700,	//!< Conformal transformation (equal scaling and simple rotation)
	TRANS2DMODEL_ThinPlateSpline =		0x0800,	//!< "Rubber Sheeting" or "Morphing" model based on thin plate spline
	TRANS2DMODEL_RationalPolynomial =	0x0900,

	TRANS2DMODEL_Piecewise =				0x8000,	//!< Model is "piecewise"
	TRANS2DMODEL_OrientSame =				0x0000,	//!< Coordinate systems have same orientation
	TRANS2DMODEL_OrientInvert =			0x4000,	//!< Coordinate systems have opposite orientations (eg. raster to map)
	TRANS2DMODEL_3D =							0x2000,	//!< 3D model

	TRANS2DMODEL_Polynomial2 =				TRANS2DMODEL_Polynomial|2,		//!< Order 2 polynomial
	TRANS2DMODEL_Polynomial3 =				TRANS2DMODEL_Polynomial|3,		//!< Order 3 polynomial
	TRANS2DMODEL_Polynomial4 =				TRANS2DMODEL_Polynomial|4,		//!< Order 4 polynomial
	TRANS2DMODEL_PiecewiseAffine2D =		TRANS2DMODEL_Piecewise|TRANS2DMODEL_Affine,
	TRANS2DMODEL_PiecewiseAffine3D =		TRANS2DMODEL_Piecewise|TRANS2DMODEL_Affine|TRANS2DMODEL_3D,

	TRANS2DMODEL_MASK_Order =				0x00FF,	//!< Mask for order (low 8 bits)
	TRANS2DMODEL_MASK_Type =				0x0F00,
	TRANS2DMODEL_MASK_Flags =				0xF000
	};

#define	TRANS2DMODEL_OrderMask			TRANS2DMODEL_MASK_Order
#define	TRANS2DMODEL_TypeMask			TRANS2DMODEL_MASK_Type
#define	TRANS2DMODEL_FlagMask			TRANS2DMODEL_MASK_Flags

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(TRANS2DMODEL)
#endif

inline TRANS2DMODEL TRANS2DMODEL_PolynomialN (int order)
	{ return (static_cast<TRANS2DMODEL>(TRANS2DMODEL_Polynomial|order)); }

#endif	// INC_MI32_TRANS2DMODEL_H
