/**
 * \file winfixup.h <mi32/winfixup.h>
 * \brief Change commonly conflicting Windows macros to inlines
 *
 * \if NODOC
 * $Id: winfixup.h_v 1.12 2006/05/12 21:37:05 scowan Exp $
 *
 * $Log: winfixup.h_v $
 * Revision 1.12  2006/05/12 21:37:05  scowan
 * Added getUserName.
 *
 * Revision 1.11  2006/04/10 13:23:18  mju
 * Undefine bandinfo.
 *
 * Revision 1.10  2006/03/21 17:15:34  scowan
 * Removed sendMessage.
 *
 * Revision 1.9  2006/03/20 23:22:30  scowan
 * Added sendMessage.
 *
 * Revision 1.8  2004/10/26 14:52:22  mju
 * Add inline for getForm.
 *
 * Revision 1.7  2003/09/15 17:24:52  dwilliss
 * Remove the include guard that the doxygen cleanup script added.
 *
 * Revision 1.6  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.5  2002/06/26 13:56:31  mju
 * Add check for NOWINFIXUP as some MFC-only modules don't want stuff renamed.
 *
 * Revision 1.4  2001/08/02 20:45:47  scowan
 * Fixed GetObject() inline, do not use the COM version.
 *
 * Revision 1.3  2001/08/02 20:17:44  mju
 * Add GetObject() inline.
 *
 * Revision 1.2  2001/07/25 21:03:56  dwilliss
 * Can't define an inline function unless you're compiling for C++
 * (MI/X has some modules which are and some that are not)
 *
 * Revision 1.1  2001/07/11 19:48:40  mju
 * Initial revision
 * \endif
**/

#if !defined(NOWINFIXUP) && defined(WIN32)

// \if NODOC

// This does not use inclusion guards as it may be included multiple times
// depending on order of including Windows files.

#ifndef __cplusplus
#define inline __inline		// Inline keyword only good for C++, __inline good for C
#endif

#ifdef BANDINFO	// Obsolete GDI escape
#undef BANDINFO
#endif

// Convert conflicting Windows #defines to inlines so don't clobber other usages.

#ifdef GetMessage
#undef GetMessage
inline BOOL GetMessage (LPMSG lpMsg, HWND hWnd, UINT wMsgFilterMin, UINT wMsgFilterMax) { return (GetMessageA(lpMsg,hWnd,wMsgFilterMin,wMsgFilterMax)); }
#endif

#ifdef GetObject
#undef GetObject
inline int GetObject (HGDIOBJ pDesc, int riid, LPVOID ppv) { return (GetObjectA(pDesc,riid,ppv)); }
#endif

#ifdef GetForm
#undef GetForm
inline int GetForm (HANDLE hPrinter, LPSTR pFormName, DWORD Level, LPBYTE pForm, DWORD cbBuf, LPDWORD pcbNeeded) { return (GetFormA(hPrinter,pFormName,Level,pForm,cbBuf,pcbNeeded)); }
#endif

#ifdef GetUserName
#undef GetUserName
inline BOOL GetUserName ( LPTSTR lpBuffer, LPDWORD nSize) { return (GetUserNameA(lpBuffer, nSize)); }
#endif

// Note, do not inline SendMessage().  Too many things use it


// \endif

#endif	// NOWINFIXUP && WIN32
