/**
 * \file mi32/xmlobject.h
 * \brief XMLOBJECT class definitions.
 *
 * \if NODOC
 * $Id: xmlobject.h_v 1.2 2011/11/23 20:38:02 mju Exp $
 *
 * $Log: xmlobject.h_v $
 * Revision 1.2  2011/11/23 20:38:02  mju
 * Use classlibexport.
 *
 * Revision 1.1  2011/11/23 20:35:52  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_XMLOBJECT_H
#define	INC_MI32_XMLOBJECT_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifdef MISYSTEMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class XMLDOC;
class XMLNODE;
#endif

//=====================================================================================================================
//! Base interface class for objects represented as XML.
class CLASSLIBEXPORT XMLOBJECT {
	public:

		XMLOBJECT ();

		virtual ~XMLOBJECT () = 0;

		//! Get contents into new child XMLNODE.
		ERRVALUE XmlGet (
			XMLNODE *nodeParent,						//!< Parent node
			const char *nodeTagName = 0			//!< Child node tagname, 0 to use value from v_XmlGetTagName
			) const;

		//! Get contents into new child XMLNODE.
		ERRVALUE XmlGet (
			XMLNODE *nodeParent,						//!< Parent node
			XMLNODE *& nodeObject,					//!< New child node associated with object returned
			const char *nodeTagName = 0			//!< Child node tagname, 0 to use value from v_XmlGetTagName
			) const;

		//! Get contents into XMLDOC.
		ERRVALUE XmlGet (
			XMLDOC& doc,
			const char *rootTagName = 0			//!< Root node tagname, 0 to use value from v_XmlGetTagName
			) const;

		//! Set contents from XMLNODE.
		ERRVALUE XmlSet (
			const XMLNODE *nodeObject				//!< Node associated with object, to set contents from
			);

	private:

		//! Get contents into XMLNODE.
		virtual ERRVALUE v_XmlGet (
			XMLNODE *nodeObject						//!< Node associated with object
			) const = 0;

		//! Get XML tagname to use if not otherwise specified.
		virtual const char* v_XmlGetTagName () const = 0;

		//! Set contents from XMLNODE.
		virtual ERRVALUE v_XmlSet (
			const XMLNODE *nodeObject				//!< Node associated with object
			) = 0;

	};

//=====================================================================================================================

#undef	CLASSLIBEXPORT

#endif	// INC_MI32_XMLOBJECT_H
