/**
 * \file rvc/dbtabsty.h 
 * \brief interface for the RVC::DBTABLE_STYLE class.
 *
 * \if NODOC
 * $Id: dbtabsty.h_v 1.10 2005/12/23 22:31:47 scowan Exp $
 *
 * $Log: dbtabsty.h_v $
 * Revision 1.10  2005/12/23 22:31:47  scowan
 * Added two get object selection filters.
 *
 * Revision 1.9  2004/05/27 17:57:11  scowan
 * Added convert style assign table to method.
 *
 * Revision 1.8  2004/03/10 21:31:26  scowan
 * Fixed virtual method issue.
 *
 * Revision 1.7  2003/09/30 16:52:06  dwilliss
 * Doxygen
 *
 * Revision 1.6  2003/09/25 21:20:04  dwilliss
 * Fixed doxygen comment
 *
 * Revision 1.5  2003/09/15 13:49:45  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.4  2002/05/24 17:22:49  scowan
 * Added set style link method.
 *
 * Revision 1.3  2002/04/22 17:53:02  scowan
 * added privated method.
 *
 * Revision 1.2  2002/04/11 22:08:58  scowan
 * Virtual method.
 *
 * Revision 1.1  2002/04/05 18:48:21  scowan
 * Initial revision
 *
 * \endif
**/

//!
//!	\class RVC::DBTABLE_STYLE rvc/dbtabsty.h
//!	\brief Manages RVC::OBJTYPE_Table objects with RVC::DBTABLE::Usage_XXXStyle. 
//!
//!	The RVC::DBTABLE_STYLE class manages RVC::OBJTYPE_Table objects with RVC::DBTABLE::Usage_XXXStyle.  The interface currently wraps the MfDatabaseTable functions.
//!
//!	RVC Object Types that this class supports: RVC::OBJTYPE_Table.
//!
//!	RVC Database Table Usage that this class supports: 
//!		- RVC::DBTABLE::USAGE_PointStyle
//!		- RVC::DBTABLE::USAGE_LineStyle
//!		- RVC::DBTABLE::USAGE_PolyStyle
//!		- RVC::DBTABLE::USAGE_CADStyle
//!
//!	Default RVC Object Type for this class: RVC::OBJTYPE_Table.
//!
//!	Valid parents for this class:
//!		RVC::OBJTYPE_Database
//!		RVC::OBJTYPE_Database_Raster
//!		RVC::OBJTYPE_Database_VectorPoint
//!		RVC::OBJTYPE_Database_VectorLine
//!		RVC::OBJTYPE_Database_VectorPolygon
//!		RVC::OBJTYPE_Database_CAD
//!		RVC::OBJTYPE_Database_TIN
//!

#ifndef INC_RVC_DBTABSTY_H
#define INC_RVC_DBTABSTY_H

#ifndef INC_RVC_DBTABLE_H
	#include <rvc/dbtable.h>
#endif

namespace RVC {

#ifndef GENERATING_DOXYGEN_OUTPUT
class STYLE;		// Forward declarations
#endif // GENERATING_DOXYGEN_OUTPUT

class DBTABLE_STYLE : public RVC::DBTABLE {
	public:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class GETOBJFILTER_ISSELECTABLE;
		class GETOBJFILTER_HASSTYLESUBOBJECT;
		#endif // GENERATING_DOXYGEN_OUTPUT

		//! Default constructor
		DBTABLE_STYLE (
			);

		//! Copy constructor
		DBTABLE_STYLE (
			const RVC::DBTABLE_STYLE& rhs
			);

		//! Destructor
		virtual ~DBTABLE_STYLE (
			);

		//! Assignment
		RVC::DBTABLE_STYLE& operator= (
			const RVC::DBTABLE_STYLE& rhs
			);
			
		//! Convert style assignment table from 'this' to 'DestStyleTable' based on the 
		//! table usage types.
		ERRVALUE ConvertTableTo (
			DBTABLE_STYLE& DestStyleTable				//!< Must be created outside this function and be open
			);

		//! Locate style object referenced by this style assignment table
		//! 'StyleObj' will be attached if one is found, unattached if not
		ERRVALUE FindStyleObject (
			RVC::STYLE& StyleObj
			);

		//! Set the style object that the style assignment table links to			
		ERRVALUE SetStyleLink (
			const RVC::STYLE& StyleObj			//!< Style object to link to
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
	
		ERRVALUE MakeStyleLink (STYLE& StyleObj);

		virtual bool v_IsValidTable (const RVC::DBTABLE& table);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};


//! Database table object selection filter that excludes all objects except for style assignment tables.
class DBTABLE_STYLE::GETOBJFILTER_ISSELECTABLE : public RVC::GETOBJFILTER {
	public:
		GETOBJFILTER_ISSELECTABLE() : GETOBJFILTER() { }
		virtual ~GETOBJFILTER_ISSELECTABLE() { }

	private:
		virtual bool v_IsSelectable (const RVC::OBJITEM& objitem, int position);
		virtual bool v_HasSelectableSubObjects (const RVC::OBJITEM& objitem);
	};


//! Database table object selection filter that excludes all objects except for style assignment tables with locatable style objects.
class DBTABLE_STYLE::GETOBJFILTER_HASSTYLESUBOBJECT : public RVC::GETOBJFILTER {
	public:
		GETOBJFILTER_HASSTYLESUBOBJECT() : GETOBJFILTER() { }
		virtual ~GETOBJFILTER_HASSTYLESUBOBJECT() { }

	private:
		virtual bool v_IsSelectable (const RVC::OBJITEM& objitem, int position);
		virtual bool v_HasSelectableSubObjects (const RVC::OBJITEM& objitem);
	};
	
}		//! End of RVC namespace

#endif 	//!< INC_RVC_DBTABSTY_H
