/**
 * \file rvc/opcrosssection.h
 * \brief Operation to generate a cross section based on a line over a surface to vector polygons
 *
 * \if NODOC
 * $Id: opcrosssection.h_v 1.2 2011/09/02 12:53:03 mju Exp $
 *
 * $Log: opcrosssection.h_v $
 * Revision 1.2  2011/09/02 12:53:03  mju
 * Dox hdr.
 *
 * Revision 1.1  2006/08/30 15:38:22  scowan
 * Initial revision
 * \endif
**/

#ifndef	INC_RVC_OPCROSSSECTION_H
#define	INC_RVC_OPCROSSSECTION_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT  
class POLYLINE;
namespace RVC {
	class OBJITEM;
	}
#endif // GENERATING_DOXYGEN_OUTPUT

namespace RVC {
namespace OP {

//=====================================================================================================================

//! Create polygon cross section based on surface and reference layer.
class CROSSSECTION {
	public:
		enum FLAGS {
			FLAG_None = 0x0000,
			FLAG_NoGeoreference = 0x0001		//!< Do not generate a manifold georeference
			};

		//! Create polygon cross section based on surface and reference layer.
		static ERRVALUE Process (
			const POLYLINE& ProfileLine,		//!< Assumed to be in reference object coords, not surface
			const RVC::OBJITEM& SrcObjItem,	//!< Source reference layer
			const RVC::OBJITEM& SurfObjItem,	//!< Surface reference layer
			RVC::OBJITEM& DestObjItem,			//!< Destination vector object RETURNED
			double BaseValue,						//!< Base Z value to start cross section on
			FLAGS flags = FLAG_None				//!< Flags
			);

	};

//=====================================================================================================================

}	// End namespace OP
}	// End namespace RVC

#endif	// INC_RVC_OPCROSSSECTION_H
