/**
 * \file imagefilterreplacenull.h <spatmod/imagefilterreplacenull.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_REPLACENULL definition.
 *
 * \if NODOC
 * $Id: imagefilterreplacenull.h_v 1.3 2008/05/21 21:54:12 mju Exp $
 *
 * $Log: imagefilterreplacenull.h_v $
 * Revision 1.3  2008/05/21 21:54:12  mju
 * Add allByte and newValueByte members.
 *
 * Revision 1.2  2007/07/11 15:36:37  mju
 * Add parm to set whether to make replaced values "valid" or not.
 *
 * Revision 1.1  2006/08/02 20:26:41  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERREPLACENULL_H
#define	INC_SPATMOD_IMAGEFILTERREPLACENULL_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

#ifndef	INC_SPATMOD_IMAGEBUFFER_H
#include <spatmod/imagebuffer.h>
#endif

#ifndef	INC_SPATMOD_IMAGESECTION_H
#include <spatmod/imagesection.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Filter to replace null pixels with specified value.
//! Can be useful for controlling the actual value of "null" pixels prior to writing to target.
class FILTER_REPLACENULL : public FILTER {
	public:

		//! Construct with numeric replacement value.
		//! Value must be in allowed range or error will be returned in Initialize.
		FILTER_REPLACENULL (
			STAGE *pSourceImage,						//!< Source image stage
			bool TakeOwnership,						//!< Filter takes ownership of source stage and will delete when needed
			double ValueReplace = 0,				//!< Value to replace with, will be used for all samples
			bool MakeValid = true					//!< Make replaced values "valid"
			);

		//! Construct with replacement PIXEL value.
		//! Pixel properties must match input image or error will be returned in Initialize.
		FILTER_REPLACENULL (
			STAGE *pSourceImage,						//!< Source image stage
			bool TakeOwnership,						//!< Filter takes ownership of source stage and will delete when needed
			const PIXEL& PixelReplace,				//!< Value to replace with
			bool MakeValid = true					//!< Make replaced values "valid"
			);

		// Destructor.
		virtual ~FILTER_REPLACENULL ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		PIXEL m_PixelReplace;
		double m_ValueReplace;
		bool m_UseValue;
		bool m_AllByte;
		UINT8 m_NewValidity;
		UINT8 *m_NewValueByte;

		// STAGE/FILTER overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		FILTER_REPLACENULL (const FILTER_REPLACENULL&);
		FILTER_REPLACENULL& operator= (const FILTER_REPLACENULL&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERREPLACENULL_H

